@@ -46,6 +46,10 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #ifndef NLIST_STRUCT
 #include <a.out.h> 
 #else /* NLIST_STRUCT */
+#ifdef SUNOS_5
+#include <fcntl.h>
+#include <kvm.h>
+#endif
 #include <nlist.h>
 #endif /* NLIST_STRUCT */
 #endif /* not VMS */
@@ -1170,6 +1174,10 @@ and then turned into integers).")
 #endif /* eunice */
 #endif /* VMS */
 
+#ifdef SUNOS_5
+  static kvm_t *kd;
+#endif /* SUNOS_5 */
+
   /* If this fails for any reason, we can return (0 0 0) */
   load_ave[0] = 0.0; load_ave[1] = 0.0; load_ave[2] = 0.0;
 
@@ -1240,11 +1248,22 @@ and then turned into integers).")
 	    nl[0].n_value &= 0x7fffffff;
 	}
 #else
+#ifdef SUNOS_5
+      if (initialized == 0) 
+        {
+          if ((kd = kvm_open(0, 0, 0, O_RDONLY, 0)) != 0) 
+            {
+              initialized = 1;
+              kvm_nlist(kd, nl);
+            }
+        }
+#else /* SUNOS_5 */
 #ifdef NeXT
       nlist (KERN_FILE, nl);
 #else      
       nlist (KERNEL_FILE, nl);
 #endif      
+#endif /* SUNOS_5 */
 #endif /* IRIS */
 
 #ifdef FIXUP_KERNEL_SYMBOL_ADDR
@@ -1252,6 +1271,7 @@ and then turned into integers).")
 	nl[0].n_value = (nlp->n_value >> 2) | 0xc0000000;
 #endif /* FIXUP_KERNEL_SYMBOL_ADDR */
     }
+#ifndef SUNOS_5
   /*
    *	Make sure we have /dev/kmem open
    */
@@ -1261,13 +1281,26 @@ and then turned into integers).")
        *	Open /dev/kmem
        */
       channel = open ("/dev/kmem", 0);
-      if (channel >= 0) initialized = 1;
+      if (channel >= 0) 
+        initialized = 1;
     }
+#endif /* !SUNOS_5 */
   /*
    *	If we can, get the load ave values
    */
   if ((nl[0].n_value != 0) && (initialized != 0))
     {
+#ifdef SUNOS_5
+      /*
+       * try to read load
+       */
+      if (kvm_read(kd, nl[0].n_value, (char *)load_ave, sizeof(load_ave))
+          != sizeof(load_ave))
+        {
+          kvm_close(kd);
+          initialized = 0;
+        }
+#else /* SUNOS_5 */
       /*
        *	Seek to the correct address
        */
@@ -1278,6 +1311,7 @@ and then turned into integers).")
 	  close (channel);
 	  initialized = 0;
 	}
+#endif /* SUNOS_5 */
     }
 #endif /* not VMS */
 
