@@ -85,6 +85,10 @@ static dummy () {}
 #endif /* AIX */
 
 #ifdef SYSV_PTYS
+#ifdef titan
+#include <sys/ttyhw.h>
+#include <sys/stream.h>
+#endif /* titan's pty needs ttyhw & stream */
 #include <sys/tty.h>
 #include <sys/pty.h>
 #endif
@@ -189,6 +193,9 @@ extern errno;
 extern sys_nerr;
 extern char *sys_errlist[];
 
+#ifdef SYS_SIGLIST
+#define sys_siglist SYS_SIGLIST
+#else
 #ifndef BSD4_1
 extern char *sys_siglist[];
 #else
@@ -222,6 +229,7 @@ char *sys_siglist[] =
     "exceeded file size limit"
     };
 #endif
+#endif /* SYS_SIGLIST */
 
 #ifdef vipc
 
@@ -1253,7 +1261,9 @@ create_process (process, new_argv)
 	  }
 #endif /* not UNIPLUS and not RTU */
 #ifdef SETUP_SLAVE_PTY
-	SETUP_SLAVE_PTY;
+	if (pty_flag) {
+	  SETUP_SLAVE_PTY;
+	}
 #endif /* SETUP_SLAVE_PTY */
 #ifdef AIX
 	/* On AIX, we've disabled SIGHUP above once we start a child on a pty.
@@ -1748,9 +1758,11 @@ wait_reading_process_input (time_limit, read_kbd, do_display)
 	    error("select error: %s", sys_errlist[xerrno]);
 	}
 #if defined (sun) || defined (APOLLO)
+#ifndef USG5_4
       else if (nfds > 0 && FD_ISSET (0, &Available) && interrupt_input)
 	/* System sometimes fails to deliver SIGIO.  */
 	kill (getpid (), SIGIO);
+#endif
 #endif
 
       /* Check for keyboard input */
@@ -2226,7 +2238,11 @@ process_send_signal (process, signo, current_group, nomsg)
   /* gid may be a pid, or minus a pgrp's number */
 #ifdef TIOCSIGSEND
   if (!NULL (current_group))
+#ifdef TIOCSIGNAL     /* SIGSEND is really SIGNAL */
+    ioctl (XFASTINT (p->infd), TIOCSIGNAL, &signo);
+#else
     ioctl (XFASTINT (p->infd), TIOCSIGSEND, signo);
+#endif
   else
     {
       gid = - XFASTINT (p->pid);
