@@ -20,10 +20,10 @@
 ;;; It should be in a file named COPYING.  Among other things, the
 ;;; copyright notice and this notice must be preserved on all copies. 
 ;;;
-;;; $Revision: 1.6 $
-;;; $Source: /import/kaplan/stable/distrib/epoch-4.0p0/epoch-lisp/RCS/epoch.el,v $
-;;; $Date: 91/12/13 13:57:22 $
-;;; $Author: love $
+;;; $Revision: 1.16 $
+;;; $Source: /import/kaplan/stable/distrib/epoch-4.2/epoch-lisp/RCS/epoch.el,v $
+;;; $Date: 92/08/26 23:17:57 $
+;;; $Author: marca $
 ;;;
 ;;; epoch.el - basic code setup and loading
 ;;;
@@ -39,8 +39,11 @@
 ;; user-definable switches
 
 ;; auto-raise on focus event.
-(defvar auto-raise-screen t
-  "If set t, focus events will raise screen and minibuffer.  If set to 'screen, only edit screen will raise.  If set to 'minibuf, only minibuffer screen will raise.  If set to nil, no screens will be raised."
+(defvar auto-raise-screen 'minibuf
+  "If set t, focus events will raise screen and minibuffer.  If set to
+'screen, only edit screen will raise.  If set to 'minibuf, only
+minibuffer screen will raise.  If set to nil, no screens will be
+raised."
 )
 
 ;; include system name in screen titles
@@ -50,20 +53,21 @@
 ;;; --------------------------------------------------------------------------
 ;; epoch-mode-alist sets default screen creation params by mode
 
-(defvar epoch-mode-alist
-  (list
-    (cons 'plain-TeX-mode
-      (list
-	(cons 'geometry "80x52")
-	(cons 'cursor-glyph 2)))
-    (cons 'LaTeX-mode
-      (list
-	(cons 'geometry "80x52")
-	(cons 'cursor-glyph 2)
-      )
-    )
-  )
-)
+(defvar epoch-mode-alist nil)
+;(defvar epoch-mode-alist
+;  (list
+;    (cons 'plain-TeX-mode
+;      (list
+;	(cons 'geometry "80x52")
+;	(cons 'cursor-glyph 2)))
+;    (cons 'LaTeX-mode
+;      (list
+;	(cons 'geometry "80x52")
+;	(cons 'cursor-glyph 2)
+;      )
+;    )
+;  )
+;)
 
 ;;; --------------------------------------------------------------------------
 ;; This prevents screens from getting real squirrly names like foo@bar@bar...
@@ -80,28 +84,37 @@
       (concat " @ " (system-name))
     ""))
 
-(epoch-add-setup-hook 'epoch::init)
-(defun epoch::init ()
-  (epoch::title (concat (buffer-name (current-buffer)) (sys-name))
-    (epoch::current-screen)
+(defun epoch::do-titles ()
+  ;; do for initial edit screen
+  (let
+    (
+      ;; find buffer of screen's selected window (*scratch*)
+      (buf (window-buffer (epoch::selected-window (current-screen))))
+    )
+    (epoch::title
+      (concat (buffer-name buf) (sys-name))
+      (current-screen)
+    )
+    (epoch::icon-name
+      (epoch::title nil (current-screen))
+      (current-screen)
+    )
   )
-  (epoch::icon-name (concat (buffer-name (current-buffer)) (sys-name))
-    (epoch::current-screen)
-  )
-  ;; If minibuffer has a distinct screen, then deal with its title and
-  ;; icon name
+  ;; do for minibuffer screen (if distinct)
   (if (epoch::minibuf-screen)
     (progn
       (epoch::title
-	(concat (epoch::title nil (epoch::minibuf-screen))
-	  (sys-name)
-	)
+	(concat (epoch::title nil (epoch::minibuf-screen)) (sys-name))
 	(epoch::minibuf-screen)
       )
       (epoch::icon-name
 	(epoch::title nil (epoch::minibuf-screen))
 	(epoch::minibuf-screen)
-  )))
+      )
+)))    
+
+(epoch-add-setup-hook 'epoch::init)
+(defun epoch::init ()
   ;; add update flag to the screen properties and set globally true,
   ;; true for first screen.
   (setq epoch::global-update t)
@@ -119,8 +132,11 @@
 
 ;;; --------------------------------------------------------------------------
 ;; interfaces to prim routines.
-(defvar *create-screen-alist-hook* nil
- "A hook that is called with the create-screen alist just before the screen is actually created. Should return a new alist (or the old one if no change).")
+(defvar *create-screen-alist-hook* nil 
+  "A hook that is called with the
+create-screen alist just before the screen is actually created. Should
+return a new alist (or the old one if no change).")
+
 ;;;
 (defun create-screen (&optional buff alist)
 "Create a new edit screen. Optional BUFFER indicates the buffer to put in\n\
@@ -131,7 +147,10 @@ dependent attributes."
   (let*
     (
       (b (and buff (get-buffer buff)))
-      (xname (or (and b (buffer-name b)) "Edit"))
+      (xname (if b
+	  (concat (buffer-name b) (sys-name))
+	  "Edit"
+      ))
       (real-alist
 	(append
 	  alist
@@ -226,16 +245,22 @@ dependent attributes."
   scr					;return a better value
 )
 
+;; Allow user to not have cursor warping all over the place.
+(defvar epoch::cursor-warps-to-screen t
+  "*T if cursor-to-screen warps cursor to screen; NIL otherwise.")
+
 (defun cursor-to-screen (scr)
   "Warp cursor to SCREEN and select it"
   (interactive)
   (if auto-raise-screen (raise-screen scr))
-  (epoch::warp-mouse (/ (epoch::screen-width scr) 2) 0 scr)
+  (if epoch::cursor-warps-to-screen
+      (epoch::warp-mouse (/ (epoch::screen-width scr) 2) 0 scr))
 )
 
 ;;; --------------------------------------------------------------------------
 (defun find-buffer-other-screen (buffer)
-  "Switch to BUFFER in other screen.  If buffer is already in another screen then select that, else make a new screen."
+  "Switch to BUFFER in other screen.  If buffer is already in another
+screen then select that, else make a new screen."
   (interactive "BSwitch to buffer other screen: ")
   (setq target-buffer (get-buffer buffer))
   (when (not target-buffer)
@@ -294,7 +319,8 @@ dependent attributes."
   (cursor-to-screen  (current-screen)))
 
 (defun switch-screen-noselect ()
-  "Switch to next screen, without altering the focus.  Used to allow circulation through screens without moving the mouse"
+  "Switch to next screen, without altering the focus.  Used to allow
+circulation through screens without moving the mouse"
   (interactive)
   (select-screen (next-screen))
 )
@@ -315,13 +341,19 @@ dependent attributes."
   (interactive)
   (setq target-buffer (current-buffer))
   (select-screen 
-   (create-screen target-buffer
-		    (list (cons 'title (concat (buffer-name target-buffer )
-						 (sys-name))))))
+    (create-screen target-buffer
+      (list
+	(cons 'title (concat (buffer-name target-buffer)
+	    (sys-name))
+	)
+	(cons 'icon-name (concat (buffer-name target-buffer)
+	    (sys-name)
+	))
+  )))
   (sit-for 2)
   (cursor-to-screen (current-screen))
-  (sit-for 0))
-
+  (sit-for 0)
+)
  
 (defun send-focus-to-current-screen ()
   "Focus on current screen"
@@ -366,21 +398,68 @@ dependent attributes."
 ;;; fix syntax entry for c-mode:
 (modify-syntax-entry ?: "." c-mode-syntax-table)
 
+;;; --------------------------------------------------------------------------
+;;; Make sure suspend-emacs is dead.
+(put 'suspend-emacs 'disabled
+     "Suspending a program running in an X window is silly
+and you would not be able to start it again.  Just switch windows instead.\n")
+(setq suspend-hook '(lambda () (error "Suspending an emacs running under X makes no sense")))
+(substitute-key-definition 'suspend-emacs nil global-map)
+(substitute-key-definition 'suspend-emacs nil esc-map)
+(substitute-key-definition 'suspend-emacs nil ctl-x-map)
+
+
 ;;; --------------------------------------------------------------------------
 ;;; default key bindings
 
+;; (global-unset-key "\C-z")
+;; (global-unset-key "\C-x\C-z")
+;; (global-set-key "\C-zo" 'switch-screen)
+;; (global-set-key "\C-z2" 'duplicate-screen)
+;; (global-set-key "\C-z4\C-f" 'find-file-other-screen)
+;; (global-set-key "\C-z4f" 'find-file-other-screen)
+;; (global-set-key "\C-z4b" 'find-buffer-other-screen)
+;; (global-set-key "\C-z0" (definteractive (remove-screen)))
+;; (global-set-key "\C-zm" (definteractive (raise-screen (minibuf-screen))))
+;; (global-set-key "\C-zr" (definteractive (raise-screen)))
+;; (global-set-key "\C-zl" (definteractive (lower-screen)))
+;; (global-set-key "\C-zf" 'focus-current-screen)
+;; (global-set-key "\C-zs" 'switch-screen-noselect)
+;; (global-set-key "\C-zp" 'prev-switch-screen)
+;; (global-set-key "\C-zi" 'iconify-screen)
+;; (global-set-key "\C-ze" 'display-event-status)
+
+(define-prefix-command 'epoch-cmds-prefix)
+
+(defvar epoch-cmds-map (symbol-function 'epoch-cmds-prefix)
+  "Keymap for commands that are specific to Epoch, usually bound to C-z.")
+
+(define-prefix-command 'other-screen-prefix)
+
+(defvar other-screen-map (symbol-function 'other-screen-prefix)
+  "Keymap for commands that act on the other screen, usually bound to
+C-z 4.")
+
 (global-unset-key "\C-z")
-(global-set-key "\C-zo" 'switch-screen)
-(global-set-key "\C-z2" 'duplicate-screen)
-(global-set-key "\C-z4\C-f" 'find-file-other-screen)
-(global-set-key "\C-z4f" 'find-file-other-screen)
-(global-set-key "\C-z4b" 'find-buffer-other-screen)
-(global-set-key "\C-z0" (definteractive (remove-screen)))
-(global-set-key "\C-zm" (definteractive (raise-screen (minibuf-screen))))
-(global-set-key "\C-zr" (definteractive (raise-screen)))
-(global-set-key "\C-zl" (definteractive (lower-screen)))
-(global-set-key "\C-zf" 'focus-current-screen)
-(global-set-key "\C-zs" 'switch-screen-noselect)
-(global-set-key "\C-zp" 'prev-switch-screen)
-(global-set-key "\C-zi" 'iconify-screen)
-(global-set-key "\C-ze" 'display-event-status)
+(global-unset-key "\C-x\C-z")
+(global-set-key "\C-z" 'epoch-cmds-prefix)
+
+;; Define other-screen keys.
+(define-key epoch-cmds-map "4" 'other-screen-prefix)
+(define-key other-screen-map "\C-f" 'find-file-other-screen)
+(define-key other-screen-map "f" 'find-file-other-screen)
+(define-key other-screen-map "b" 'find-buffer-other-screen)
+
+;; Define epoch keys.
+(define-key epoch-cmds-map "o" 'switch-screen)
+(define-key epoch-cmds-map "2" 'duplicate-screen)
+(define-key epoch-cmds-map "0" (definteractive (remove-screen)))
+(define-key epoch-cmds-map "m" 
+  (definteractive (raise-screen (minibuf-screen))))
+(define-key epoch-cmds-map "r" (definteractive (raise-screen)))
+(define-key epoch-cmds-map "l" (definteractive (lower-screen)))
+(define-key epoch-cmds-map "f" 'focus-current-screen)
+(define-key epoch-cmds-map "s" 'switch-screen-noselect)
+(define-key epoch-cmds-map "p" 'prev-switch-screen)
+(define-key epoch-cmds-map "i" 'iconify-screen)
+(define-key epoch-cmds-map "e" 'display-event-status)
