@@ -20,15 +20,18 @@
 ;;; It should be in a file named COPYING.  Among other things, the
 ;;; copyright notice and this notice must be preserved on all copies. 
 ;;;
-;;; $Revision: 1.3 $
-;;; $Source: /import/kaplan/stable/distrib/epoch-4.0p0/epoch-lisp/RCS/event.el,v $
-;;; $Date: 92/01/28 14:37:55 $
-;;; $Author: love $
+;;; $Revision: 1.6 $
+;;; $Source: /import/kaplan/stable/distrib/epoch-4.2/epoch-lisp/RCS/event.el,v $
+;;; $Date: 92/08/01 22:34:39 $
+;;; $Author: marca $
 ;;;
 ;;; event.el - provide general event handling.  Originally written by
 ;;;	       Susan Hinrich, modified by Alan Carroll and Chris Love
 ;;;
-(require 'mini-cl)
+
+;; Changed from mini-cl (due to mdb@NSD.3Com.COM).
+(require 'cl)
+
 (require 'epoch-util)
 (provide 'event)
 ;;;
@@ -188,6 +191,9 @@
 	)
       )
     )
+    (insert
+      "\nEvent dispatcher: " (prin1-to-string epoch::event-handler) "\n"
+    )
     (set-buffer-modified-p nil)
     (setq buffer-read-only t)
   )
@@ -374,13 +380,3 @@ screen and event value)"
   )
   (setq epoch::event-handler epoch::old-handler)
 )
-
-
-
-
-
-
-
-
-
-
