@@ -20,18 +20,19 @@
 ;;; It should be in a file named COPYING.  Among other things, the
 ;;; copyright notice and this notice must be preserved on all copies. 
 ;;;
-;;; $Revision: 1.6 $
-;;; $Source: /import/kaplan/stable/distrib/epoch-4.0/epoch-lisp/RCS/motion.el,v $
-;;; $Date: 91/09/18 21:54:16 $
-;;; $Author: love $
+;;; $Revision: 1.11 $
+;;; $Source: /import/kaplan/stable/distrib/epoch-4.2/epoch-lisp/RCS/motion.el,v $
+;;; $Date: 92/06/02 02:19:20 $
+;;; $Author: marca $
 ;;;
 ;;; motion.el - provide draggin/hi-lighting of primary selection
 ;;;
 ;;; Original version by Alan Carroll
 ;;; Epoch 4.0 modifications by Chris Love
+;;; Abort-isearch and other ideas from Ken Laprade and others.
 ;;;
 (provide 'motion)
-(require 'button)
+(require 'zone)
 (require 'mouse)
 
 ;;; ------------------------------------------------------------------------
@@ -44,18 +45,18 @@
 (defvar mouse::downp nil "State variable for mouse dragging internals")
 (defvar mouse::last-point -1 "Last location of a motion event")
 
-(setq epoch::buttons-modify-buffer nil)
-(defvar motion::style nil "style used by drag buttons")
+(setq epoch::zones-modify-buffer nil)
+(defvar motion::style nil "style used by drag zones")
 
-(defvar drag-button nil
-  "Epoch button to be used for hilighting selected text region."
+(defvar drag-zone nil
+  "Epoch zone to be used for hilighting selected text region."
 )
-(setq-default drag-button nil)
+(setq-default drag-zone nil)
 (setq-default mouse-down-marker nil)
 
 ;;; ------------------------------------------------------------------------
 ;;; Set window-setup-hook to call motion::init(), which sets default
-;;; style for button dragging.  Default style is underlining; can be changed
+;;; style for zone dragging.  Default style is underlining; can be changed
 ;;; in .emacs file.
 (epoch-add-setup-hook 'motion::init)
 
@@ -87,17 +88,28 @@
   (setq mouse::last-point -1)		;always do this cleanup
   (when mouse::downp
     (setq mouse::downp nil)
-    (mouse::copy-button drag-button)
-    (if (buttonp drag-button)
-      (if (<= (point) (button-start drag-button))
-	(progn
-	  (push-mark (button-end drag-button) t)
-	  (goto-char (button-start drag-button))
-	)
-	;; ELSE point is past drag button start
-	(progn
-	  (push-mark (button-start drag-button) t)
-	  (goto-char (button-end drag-button))
+    (mouse::copy-zone drag-zone)
+    (let (
+	(s (and drag-zone (zone-start drag-zone)))
+	(e (and drag-zone (zone-end drag-zone)))
+      )
+      (if (null s) (setq s 1))
+      (if (null e) (setq e 1))
+      (if (zonep drag-zone)
+	(if (<= (point) s)
+	  (progn
+	    (push-mark e t)
+            ;; Bandaid...?
+            ;; (goto-char s)
+	    (goto-char (min s (point-max)))
+	  )
+	;; ELSE point is past drag zone start
+	  (progn
+	    (push-mark s t)
+            ;; Bandaid...?
+            ;; (goto-char e)
+	    (goto-char (min e (point-max)))
+	  )
 	)
       )
     )
@@ -108,12 +120,13 @@
   (when arg
     (setq mouse::downp 'start)
     (mouse::set-point arg)
+    (abort-isearch)
     (set-mouse-marker)
     (setq mouse::last-point (point))
-    (if drag-button
+    (if drag-zone
       (progn
-	(delete-button drag-button)
-	(setq drag-button nil)
+	(delete-zone drag-zone)
+	(setq drag-zone nil)
 	(redisplay-screen)
       )
     )
@@ -124,31 +137,32 @@
   (setq mouse::downp 'extend)
   (let
     (
-      (m1 (and drag-button (button-start drag-button)))
-      (m2 (and drag-button (button-end drag-button)))
-      (spot (car arg))			;point of the mouse click.
+      (m1 (and drag-zone (zone-start drag-zone)))
+      (m2 (and drag-zone (zone-end drag-zone)))
+      (spot (min (car arg) (point-max))) ;point of the mouse click.
     )
     (if (null m1) (setq m1 0))
     (if (null m2) (setq m2 0))
     (cond
-      ((or (null drag-button) (null mouse-down-marker))
-	(setq drag-button (add-button (point) spot motion::style) )
+      ((or (null drag-zone) (null mouse-down-marker))
+	(setq drag-zone (add-zone (point) spot motion::style) )
+	(set-zone-transient drag-zone t)
 	(set-mouse-marker)
       )
       ((<= spot m1)
-	(setq drag-button (move-button drag-button spot m2) )
+	(setq drag-zone (move-zone drag-zone spot m2) )
 	(set-mouse-marker m2)
       )
       ((>= spot m2)
-	(setq drag-button (move-button drag-button m1 spot) )
+	(setq drag-zone (move-zone drag-zone m1 spot) )
 	(set-mouse-marker m1)
       )
       ((<= mouse-down-marker spot)
-	(setq drag-button (move-button drag-button m1 spot) )
+	(setq drag-zone (move-zone drag-zone m1 spot) )
 	(set-mouse-marker m1)
       )
       (t
-	(setq drag-button (move-button drag-button spot m2) )
+	(setq drag-zone (move-zone drag-zone spot m2) )
 	(set-mouse-marker m2)
       )
     )
@@ -182,12 +196,15 @@
       (w-edges (window-pixedges w))
       (left (car w-edges))
       (top (elt w-edges 1))
-      (right (elt w-edges 2))
-      (bottom (elt w-edges 3))
+      (right (- (elt w-edges 2) left 1))
+      (bottom (- (elt w-edges 4) top 1))
     )
-    (if (not drag-button)
-      (setq drag-button
-	(add-button mouse-down-marker (point) motion::style )
+    (if (not drag-zone)
+      (progn
+	(setq drag-zone
+	  (add-zone mouse-down-marker (point) motion::style )
+	)
+	(set-zone-transient drag-zone t)
       )
     )
     (while
@@ -197,25 +214,26 @@
 	(/= 0 (logand mouse-any-mask (elt pos 2)))
       )
       ;;convert to window relative co-ordinates
-      (setq x (car pos))
-      (setq y (elt pos 1))
-      (setq out-of-bounds (coordinates-in-window-p pos w))
-
+      (setq x (- (car pos) left))
+      (setq y (- (elt pos 1) top))
+      (setq out-of-bounds
+	(not (and (<= 0 y) (<= y bottom) (<= 0 x) (<= x right)))
+      )
       ;; scrolling conditions
       (condition-case errno
 	(progn
-	  (if (< y top) (scroll-down vertical-drag-inc))
+	  (if (< y 0) (scroll-down vertical-drag-inc))
 	  (if (> y bottom) (scroll-up vertical-drag-inc))
 	)
 	(error )			;nothing, just catch it
       )
+;; Disable horizontal scrolling.
 ;      (if (< x left) (scroll-right horizontal-drag-inc))
 ;      (if (> x right) (scroll-left horizontal-drag-inc))
-      (setq y (max 0 (- (min (- bottom (nth 2 (font))) y) 1)))
-      (setq x (max left (min right x)))
+      (setq y (max 1 (min bottom y)))
+      (setq x (max 0 (min right x)))
 
-;      (message "X,Y = %d,%d" x y)
-      (setq pnt (car (epoch::coords-to-point x y)))
+      (setq pnt (car (epoch::coords-to-point (+ x left) (+ y top))))
       (when (/= mouse::last-point pnt)
 	(if (> mouse-down-marker pnt)
 	  (progn
@@ -227,18 +245,18 @@
 	    (setq drag-m2 pnt)
 	  )
 	)
-	(move-button drag-button drag-m1 drag-m2)
+	(move-zone drag-zone drag-m1 drag-m2)
 	(redisplay-screen)
       )
-      (setq mouse::last-point pnt)
+      (setq mouse::last-point pnt)	      
     )
   )
 )
     
 ;;; ------------------------------------------------------------------------
 ;;; Code for selecting lines using motion events. Assumes that the line is
-;;; left unmarked on button up
-(defvar mouse::line-button nil "Button for selected line")
+;;; left unmarked on zone up
+(defvar mouse::line-zone nil "Zone for selected line")
 ;;;
 (defun mouse-select-line-start (arg)
   (mouse::set-point arg)		;go there
@@ -248,7 +266,7 @@
       (beginning-of-line)
       (setq bol (point))
       (end-of-line)
-      (setq mouse::line-button (add-button bol (point) motion::style))
+      (setq mouse::line-zone (add-zone bol (point) motion::style))
     )
   )
   (push-event 'motion 'mouse-select-line-update)
@@ -256,7 +274,7 @@
 ;;;
 (defun mouse-select-line-end (arg)
   (setq mouse::last-point -1)
-  (when mouse::line-button (delete-button mouse::line-button))
+  (when mouse::line-zone (delete-zone mouse::line-zone))
   (pop-event 'motion)
 )
 ;;;
@@ -270,13 +288,13 @@
       (epoch::event-handler-abort nil)
       (w-edges (window-pixedges (selected-window)))
       (top (elt w-edges 1))
-      (bottom (- (elt w-edges 3) (+ 2 top)))
+      (bottom (- (elt w-edges 4) top 1))
       max-vscroll
     )
     (while
       (and
 	out-of-bounds
-	(setq pos (query-mouse))
+	(setq pos (query-pointer))
 	(/= 0 (logand mouse-any-mask (elt pos 2)))
       )
       ;;convert to window relative co-ordinates
@@ -310,29 +328,59 @@
       (beginning-of-line) (setq bol (point))
       (end-of-line)
       (when (/= mouse::last-point (point))
-	(move-button mouse::line-button bol (point))
+	(move-zone mouse::line-zone bol (point))
 	(epoch::redisplay-screen)
       )
       (setq mouse::last-point (point))
     )
   )
 )
+;;; --------------------------------------------------------------------------
 ;; Stolen from AMC
 (defun mouse::buffer-line (marg)
   "Show the line number and buffer of the mouse EVENT"
   ;; marg is (point buffer window screen)
   ;; Pop over to the clicked buffer
   (save-excursion (set-buffer (cadr marg))
-    ;; Figure out how far down the mouse point is
-    (let ((n (count-lines (point-min) (car marg))))
+    ;; Figure out how far down the mouse point is.
+    ;; But don't go past point-max.
+    (let* ((end (min (point-max) (car marg)))
+           (n (count-lines (point-min) end)))
       ;; display it. Include the buffer name for good measure.
       (message (format "Line %d in %s" n (buffer-name (cadr marg))))
 )))
+
+;; Blow out of any current isearch
+(defun abort-isearch () "Abort any isearch in progress."
+  (condition-case err
+      (throw 'search-done t)
+    (no-catch nil)))
 ;;; --------------------------------------------------------------------------
-;;; install all our variouse handlers
+;;; install all our various handlers
 (global-set-mouse mouse-left mouse-down 'start-mouse-drag)
 (global-set-mouse mouse-left mouse-shift 'mouse::buffer-line)
 (global-set-mouse mouse-left mouse-up 'end-mouse-drag)
 (global-set-mouse mouse-right mouse-down 'extend-mouse-drag)
 (global-set-mouse mouse-right mouse-up 'end-mouse-drag)
 (global-set-mouse mouse-middle mouse-down 'mouse::paste-cut-buffer)
+
+
+(defun mouse-set-spot (arg)
+  "Set point at mouse.  With double-click, set mark there as well.
+Blinks matching paren if sitting after one.  Intended to be bound
+to a window down button."
+  (start-mouse-drag arg)
+  (let ((buf (current-buffer))
+        (p (point)))
+    (mouse::set-point arg)
+    (if (and (equal p (point))
+             (equal buf (current-buffer)))
+        (if (and (= mouse::clicks 1)
+                 (not (eq (mark) (point))))
+            (push-mark))
+      (setq mouse::clicks 0))
+    (if (eq (char-syntax (preceding-char)) ?\))
+          (blink-matching-open)))
+  (abort-isearch))
+
+
