@@ -20,10 +20,10 @@
 ;;; It should be in a file named COPYING.  Among other things, the
 ;;; copyright notice and this notice must be preserved on all copies. 
 ;;;
-;;; $Revision: 1.10 $
-;;; $Source: /home/kaplan/love/epoch/epoch-lisp/RCS/mouse.el,v $
-;;; $Date: 92/03/26 11:35:45 $
-;;; $Author: love $
+;;; $Revision: 1.12 $
+;;; $Source: /import/kaplan/stable/distrib/epoch-4.2/epoch-lisp/RCS/mouse.el,v $
+;;; $Date: 92/06/30 01:21:08 $
+;;; $Author: marca $
 ;;;
 ;;; mouse.el - event handler for mouse events.
 ;;;
@@ -325,8 +325,15 @@
   "Select Epoch window mouse is on, and move point to mouse position."
   (select-screen (nth 3 arg))
   (if (nth 2 arg) (select-window (nth 2 arg)))
-  (if (car arg) (goto-char (car arg)))
+  (if (car arg) (goto-char (min (car arg) (point-max))))
 )
+
+;;; --------------------------------------------------------------------------
+;; Variable added by marca to allow regression to pre-4.0p2 behavior;
+;; this may go away unless someone out there wants it to remain.
+(defvar mouse::set-point-clears-cut-buffer nil
+  "*T if setting point with left mouse button clears cut buffer; NIL otherwise.")
+
 ;;; --------------------------------------------------------------------------
 ;;; Cut & Paste functions based on an Epoch button.  
 (defun mouse::copy-zone (zone &optional kill)
@@ -334,8 +341,8 @@
   (if (zonep zone)
     (let
       (
-        (beg (epoch::zone-start zone))
-	(end (epoch::zone-end zone))
+        (beg (min (epoch::zone-start zone) (point-max)))
+	(end (min (epoch::zone-end zone) (point-max)))
 	(buf (epoch::zone-buffer zone))
 	text
       )
@@ -369,21 +376,26 @@
 	    (if kill
 	      (delete-region beg end)
 	      (copy-region-as-kill beg end)
+              ;; Eliminate replication on kill.
+              (setq this-command nil)
 	    )
 	  )
 	)
       )
     )
-    ; Don't do this
-    (epoch::store-cut-buffer "")
+    ;; Don't clear out the cut buffer if we don't explicitly want it
+    ;; cleared out; this means that the cut buffer contains a string
+    ;; until that string is replaced.
+    (if mouse::set-point-clears-cut-buffer
+        (epoch::store-cut-buffer ""))
     (setq epoch::selection-alist
-      (cons
-	(cons
-	  mouse::selection-atom ""
-	)
-        (alist-delete mouse::selection-atom epoch::selection-alist)
-    ))
-  )
+          (cons
+           (cons
+            mouse::selection-atom ""
+            )
+           (alist-delete mouse::selection-atom epoch::selection-alist)
+           ))
+    )
 )
 
 ;;;
@@ -394,7 +406,7 @@
       (save-excursion
         (set-buffer buff)
         (undo-boundary)
-        (goto-char (car arg))
+        (goto-char (min (car arg) (point-max)))
 	; get text from selection PRIMARY
 	; insert text once it's there.
 	(let
