@@ -165,6 +165,9 @@ definitions made by the terminal-specific file.")
     (if noninteractive (kill-emacs t))))
 
 (defun command-line-1 (command-line-args-left)
+  ;; Due to unknown strangeness we have to call input-pending-p
+  ;; once before trying to get a real value...
+  (input-pending-p)
   (if (null command-line-args-left)
       (cond ((and (not inhibit-startup-message) (not noninteractive)
 		  ;; Don't clobber a non-scratch buffer if init file
@@ -199,6 +202,8 @@ Copyright (C) 1990 Free Software Foundation, Inc.\n")
 
 Epoch comes with ABSOLUTELY NO WARRANTY; type C-h C-w for full details.
 Epoch is based on GNU Emacs 18.58.
+Epoch modifications to GNU Emacs were made by the Epoch Development Team
+  at the University of Illinois at Urbana-Champaign.
 
 You may give out copies of Epoch; type C-h C-c to see the conditions.
 Type C-h C-d for information on getting the latest version.
