@@ -20,15 +20,26 @@
 ;;; It should be in a file named COPYING.  Among other things, the
 ;;; copyright notice and this notice must be preserved on all copies. 
 ;;;
-;;; $Revision: 1.2 $
-;;; $Source: /import/kaplan/stable/distrib/epoch-4.0/epoch-lisp/RCS/wrapper.el,v $
-;;; $Date: 91/05/07 23:14:10 $
+;;; $Revision: 1.12 $
+;;; $Source: /import/kaplan/stable/distrib/epoch-4.2/epoch-lisp/RCS/wrapper.el,v $
+;;; $Date: 92/04/10 10:45:01 $
 ;;; $Author: love $
 ;;;
 ;;; wrapper.el - define "wrappers" for all epoch:: functions
 ;;;
 (provide 'wrapper)
 
+(defun functionp (thing)
+"Returns t if the argument is a function, nil otherwise"
+  (cond
+    ((symbolp thing) (fboundp thing))
+    ((consp thing)
+      (and (eq (car thing) 'lambda) (listp (cadr thing)))
+    )
+    (t nil)
+  )
+)
+;;;
 (fset 'screen-height (symbol-function 'epoch::screen-height))
 (fset 'screen-width (symbol-function 'epoch::screen-width))
 (fset 'redisplay-screen (symbol-function 'epoch::redisplay-screen))
@@ -42,7 +53,10 @@
 (fset 'map-screen (symbol-function 'epoch::map-screen))
 (fset 'background (symbol-function 'epoch::background))
 (fset 'foreground (symbol-function 'epoch::foreground))
-(fset 'cursor-color (symbol-function 'epoch::cursor-color))
+(fset 'cursor-foreground (symbol-function 'epoch::cursor-foreground))
+(fset 'cursor-background (symbol-function 'epoch::cursor-background))
+; for compatibility (next 1)
+(fset 'cursor-color (symbol-function 'epoch::cursor-background))
 (fset 'cursor-glyph (symbol-function 'epoch::cursor-glyph))
 (fset 'set-update (symbol-function 'epoch::set-update))
 (fset 'update-p (symbol-function 'epoch::update-p))
@@ -62,6 +76,7 @@
 (fset 'query-mouse (symbol-function 'epoch::query-mouse))
 (fset 'query-pointer (symbol-function 'epoch::query-pointer))
 (fset 'query-cursor (symbol-function 'epoch::query-cursor))
+(fset 'query-cursor-pixels (symbol-function 'epoch::query-cursor-pixels))
 (fset 'ungrab-pointer (symbol-function 'epoch::ungrab-pointer))
 (fset 'warp-mouse (symbol-function 'epoch::warp-mouse))
 (fset 'warp-pointer (symbol-function 'epoch::warp-pointer))
@@ -96,30 +111,36 @@
 (fset 'flash-screen (symbol-function 'epoch::flash-screen))
 (fset 'set-bell (symbol-function 'epoch::set-bell))
 
-(fset 'move-button (symbol-function 'epoch::move-button))
-(fset 'make-button (symbol-function 'epoch::make-button))
-(fset 'button-style (symbol-function 'epoch::button-style))
-(fset 'button-data (symbol-function 'epoch::button-data))
-(fset 'button-start (symbol-function 'epoch::button-start))
-(fset 'button-end (symbol-function 'epoch::button-end))
-(fset 'button-at (symbol-function 'epoch::button-at))
-(fset 'buttons-at (symbol-function 'epoch::buttons-at))
-(fset 'delete-button (symbol-function 'epoch::delete-button))
-(fset 'delete-button-at (symbol-function 'epoch::delete-button-at))
-(fset 'buttonp (symbol-function 'epoch::buttonp))
-(fset 'button-list (symbol-function 'epoch::button-list))
+(fset 'move-zone (symbol-function 'epoch::move-zone))
+(fset 'make-zone (symbol-function 'epoch::make-zone))
+(fset 'zone-style (symbol-function 'epoch::zone-style))
+(fset 'zone-data (symbol-function 'epoch::zone-data))
+(fset 'zone-start (symbol-function 'epoch::zone-start))
+(fset 'zone-end (symbol-function 'epoch::zone-end))
+(fset 'zone-at (symbol-function 'epoch::zone-at))
+(fset 'zones-at (symbol-function 'epoch::zones-at))
+(fset 'delete-zone (symbol-function 'epoch::delete-zone))
+(fset 'delete-zone-at (symbol-function 'epoch::delete-zone-at))
+(fset 'zonep (symbol-function 'epoch::zonep))
+(fset 'zone-list (symbol-function 'epoch::zone-list))
+(fset 'zones-in-region (symbol-function 'epoch::zones-in-region))
 (fset 'read-only-region-p (symbol-function 'epoch::read-only-region-p))
-(fset 'button-buffer (symbol-function 'epoch::button-buffer))
-(fset 'set-button-style (symbol-function 'epoch::set-button-style))
-(fset 'set-button-data (symbol-function 'epoch::set-button-data))
-(fset 'button-read-only-p (symbol-function 'epoch::button-read-only-p))
-(fset 'set-button-read-only (symbol-function 'epoch::set-button-read-only))
-(fset 'clear-buttons (symbol-function 'epoch::clear-buttons))
+(fset 'zone-buffer (symbol-function 'epoch::zone-buffer))
+(fset 'set-zone-style (symbol-function 'epoch::set-zone-style))
+(fset 'set-zone-data (symbol-function 'epoch::set-zone-data))
+(fset 'zone-read-only-p (symbol-function 'epoch::zone-read-only-p))
+(fset 'set-zone-read-only (symbol-function 'epoch::set-zone-read-only))
+(fset 'zone-transient-p (symbol-function 'epoch::zone-transient-p))
+(fset 'set-zone-transient (symbol-function 'epoch::set-zone-transient))
+(fset 'clear-zones (symbol-function 'epoch::clear-zones))
 (fset 'number-of-colors (symbol-function 'epoch::number-of-colors))
 (fset 'color-components (symbol-function 'epoch::color-components))
 (fset 'get-color (symbol-function 'epoch::get-color))
 (fset 'free-color (symbol-function 'epoch::free-color))
+(fset 'get-color-rw (symbol-function 'epoch::get-color-rw))
+(fset 'set-color-rw (symbol-function 'epoch::set-color-rw))
 ;;;
+(fset 'stylep (symbol-function 'epoch::stylep))
 (fset 'make-style (symbol-function 'epoch::make-style))
 (fset 'style-foreground (symbol-function 'epoch::style-foreground))
 (fset 'style-background (symbol-function 'epoch::style-background))
@@ -139,8 +160,11 @@
 (fset 'set-style-stipple (symbol-function 'epoch::set-style-stipple))
 (fset 'set-style-cursor-stipple (symbol-function 'epoch::set-style-cursor-stipple))
 (fset 'set-style-background-stipple (symbol-function 'epoch::set-style-background-stipple))
+(fset 'set-style-tag (symbol-function 'epoch::set-style-tag))
+(fset 'style-tag (symbol-function 'epoch::style-tag))
 ;;;
 (fset 'make-bitmap (symbol-function 'epoch::make-bitmap))
+(fset 'load-bitmap (symbol-function 'epoch::load-bitmap))
 (fset 'free-bitmap (symbol-function 'epoch::free-bitmap))
 
 (fset 'acquire-selection (symbol-function 'epoch::acquire-selection))
@@ -148,3 +172,13 @@
 (fset 'convert-selection (symbol-function 'epoch::convert-selection))
 (fset 'get-default (symbol-function 'epoch::get-default))
 (fset 'reparent-screen (symbol-function 'epoch::reparent-screen))
+
+;; Graphical zone primitives
+(if (functionp 'epoch::query-pixmap)
+  (progn
+    (fset 'query-pixmap (symbol-function 'epoch::query-pixmap))
+    (fset 'define-opaque-font (symbol-function 'epoch::define-opaque-font))
+    (fset 'read-pixmap-file (symbol-function 'epoch::read-pixmap-file))
+    (fset 'set-style-pixmap (symbol-function 'epoch::set-style-pixmap))
+    (fset 'style-pixmap (symbol-function 'epoch::style-pixmap))
+))
\ No newline at end of file
