@@ -20,16 +20,19 @@
 ;;; It should be in a file named COPYING.  Among other things, the
 ;;; copyright notice and this notice must be preserved on all copies. 
 ;;;
-;;; $Revision: 1.2 $
-;;; $Source: /import/kaplan/stable/distrib/epoch-4.0p0/epoch-lisp/RCS/zone.el,v $
-;;; $Date: 92/02/28 10:57:00 $
-;;; $Author: love $
+;;; $Revision: 1.3 $
+;;; $Source: /import/kaplan/stable/distrib/epoch-4.2/epoch-lisp/RCS/zone.el,v $
+;;; $Date: 92/08/05 17:00:22 $
+;;; $Author: marca $
 ;;;
 (require 'mini-cl)
 (provide 'zone)
 
 (defun epoch::add-zone (start end style &optional data buffer)
-  "Adds a zone to from START to END having STYLE and DATA in an optional BUFFER. START and END should be markers or integers. STYLE must be a style or nil.  DATA can be any lisp object. Returns the zone if successful."
+  "Adds a zone to from START to END having STYLE and DATA in an
+optional BUFFER. START and END should be markers or integers. STYLE
+must be a style or nil.  DATA can be any lisp object. Returns the
+zone if successful."  
   (let
     (
       (zone (epoch::move-zone (epoch::make-zone) start end buffer))
@@ -46,7 +49,10 @@
 ;;; some utility functions
 ;;;
 (defun add-read-only-zone (start end style &optional data buffer)
-  "Adds a read-only-zone from START to END having STYLE and DATA in an optional BUFFER.  START and END should be markers or integers.  STYLE must be a style or nil.  DATA can be any lisp object.  Returns zone if successful"
+  "Adds a read-only-zone from START to END having STYLE and DATA in an
+optional BUFFER.  START and END should be markers or integers.
+STYLE must be a style or nil.  DATA can be any lisp object.  Returns
+zone if successful" 
   (set-zone-read-only
     (add-zone start end style data buffer)
     t
@@ -54,9 +60,13 @@
 )
 
 (if (functionp 'epoch::query-pixmap)
-  (defun add-graphic-zone (name start end &optional offset data buffer ro)
-    "Adds a graphical zone using pixmap NAME from START to END having DATA in an optional BUFFER.  START and END should be markers or integers.  OFFSET can be a percentage value (0-100) for vertical offset of pixmap.  DATA can be any lisp object.  Returns zone if successful"
-    (let
+    (defun add-graphic-zone (name start end &optional offset data buffer ro)
+      "Adds a graphical zone using pixmap NAME from START to END having
+    DATA in an optional BUFFER.  START and END should be markers or
+    integers.  OFFSET can be a percentage value (0-100) for vertical
+    offset of pixmap.  DATA can be any lisp object.  Returns zone if
+    successful" 
+     (let
       (
 	(pix (read-pixmap-file name))
 	dim
@@ -91,11 +101,6 @@
 	))
   )))
 )
-	     
-
-	  
-	  
-	  
 
 ;;;
 (defun zone-text (&optional zone)
