@@ -18,13 +18,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.3 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/alloc.c,v $
- * $Date: 92/02/23 11:03:00 $
- * $Author: love $
+ * $Revision: 1.4 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/alloc.c,v $
+ * $Date: 92/08/05 04:15:51 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:03:00 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/alloc.c,v $ $Revision: 1.3 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/05 04:15:51 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/alloc.c,v $ $Revision: 1.4 $";
 #endif
 
 #include "config.h"
@@ -922,7 +922,7 @@ gc-cons-threshold  bytes of Lisp data since previous garbage collection.")
   if (gc_cons_threshold < 10000)
     gc_cons_threshold = 10000;
 
-  if (omessage)
+  if (omessage || minibuf_prompt)
     message1 (omessage);
   else if (!noninteractive)
     message1 ("Garbage collecting...done");
