@@ -18,13 +18,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.11 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/buffer.c,v $
- * $Date: 92/02/23 11:02:56 $
- * $Author: love $
+ * $Revision: 1.12 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/buffer.c,v $
+ * $Date: 92/07/12 03:28:25 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:02:56 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/buffer.c,v $ $Revision: 1.11 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/07/12 03:28:25 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/buffer.c,v $ $Revision: 1.12 $";
 #endif
 
 #include <sys/param.h>
@@ -790,7 +790,19 @@ DEFUN ("barf-if-buffer-read-only", Fbarf_if_buffer_read_only,
   ()
 {
   while (!NULL (current_buffer->read_only))
-    Fsignal (Qbuffer_read_only, (Fcons (Fcurrent_buffer (), Qnil)));
+    {
+      extern int waiting_for_input;
+
+      /* Avoid coredump in Fsignal by making sure waiting_for_input
+         isn't nonzero.  This should be caught in read_command_char
+         but someone says that doesn't fix the problem. */
+      int save_wait = waiting_for_input;
+      waiting_for_input = 0;
+
+      Fsignal (Qbuffer_read_only, (Fcons (Fcurrent_buffer (), Qnil)));
+
+      waiting_for_input = save_wait;
+    }      
   return Qnil;
 }
 
@@ -1116,6 +1128,7 @@ init_buffer_once ()
   XFASTINT (buffer_defaults.tab_width) = 8;
   buffer_defaults.truncate_lines = Qnil;
   buffer_defaults.ctl_arrow = Qt;
+  buffer_defaults.meta_printable = Qt;
 
   XFASTINT (buffer_defaults.fill_column) = 70;
   XFASTINT (buffer_defaults.fill_pixel) = 0; /* Disable */
@@ -1160,6 +1173,7 @@ init_buffer_once ()
   XFASTINT (buffer_local_flags.fill_pixel) = 0x4000;
   XFASTINT (buffer_local_flags.line_fill_hook) = 0x8000;
   XFASTINT (buffer_local_flags.undo_buttons) = 0x10000;
+  XFASTINT (buffer_local_flags.meta_printable) = 0x20000;
 
   Vbuffer_alist = Qnil;
   current_buffer = 0;
@@ -1230,6 +1244,10 @@ This is the same as (default-value 'abbrev-mode).");
 	      &buffer_defaults.ctl_arrow,
     "Default ctl-arrow for buffers that do not override it.\n\
 This is the same as (default-value 'ctl-arrow).");
+  DEFVAR_LISP_NOPRO ("default-meta-printable",
+ 	      &buffer_defaults.meta_printable,
+    "Default meta-printable for buffers that do not override it.\n\
+This is the same as (default-value 'meta-printable).");
 
   DEFVAR_LISP_NOPRO ("default-truncate-lines",
 	      &buffer_defaults.truncate_lines,
@@ -1337,9 +1355,14 @@ Automatically becomes local when set in any fashion.");
   DEFVAR_PER_BUFFER ("ctl-arrow", &current_buffer->ctl_arrow,
     "*t means display control chars with uparrow.\n\
 Nil means use backslash and octal digits.\n\
-Otherwise assume that control and meta characters are printable.\n\
 Automatically becomes local when set in any fashion.");
 
+  DEFVAR_PER_BUFFER ("meta-printable", &current_buffer->meta_printable,
+    "*Non-nil means meta characters, that is characters in the range \n\
+0240..0377, are printable.\n\
+Nil means display meta chars with backslash and octal digits.\n\
+Automatically becomes local when set in any fashion.");
+  
   DEFVAR_PER_BUFFER ("truncate-lines", &current_buffer->truncate_lines,
     "*Non-nil means do not display continuation lines;\n\
 give each line of text one screen line.\n\
