@@ -18,10 +18,10 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.3 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/buffer.h,v $
- * $Date: 91/05/07 22:58:28 $
- * $Author: love $
+ * $Revision: 1.8 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/buffer.h,v $
+ * $Date: 92/07/12 03:28:33 $
+ * $Author: marca $
  */
 #define SET_PT  PT =
 
@@ -144,6 +144,9 @@ struct buffer
        the last time this buffer was displayed */
     int last_window_start;
 
+    /* Count of the number of read-only buttons in this buffer. */
+    int read_only_buttons;
+
     /* This is a special exception -- as this slot should not be
        marked by gc_sweep, and as it is not lisp-accessible as
        a local variable -- so re regard it as not really being of type
@@ -213,6 +216,8 @@ struct buffer
     Lisp_Object truncate_lines;
     /* Non-nil means display ctl chars with uparrow */
     Lisp_Object ctl_arrow;
+    /* Non-nil means meta chars are printable. */
+    Lisp_Object meta_printable;
     /* Non-nil means do selective display;
        See doc string in syms_of_buffer (buffer.c) for details.  */
     Lisp_Object selective_display;
@@ -229,6 +234,9 @@ struct buffer
     Lisp_Object force_redisplay; /* buffer should be redisplayed ASAP */
     Lisp_Object allowed_screens; /* screens buffer can be default for */
     Lisp_Object style;		/* normal style for this buffer. */
+    Lisp_Object fill_pixel;	/* for crude kerning mode */
+    Lisp_Object line_fill_hook;
+    Lisp_Object undo_buttons;	/* Undo restores buttons ? */
 };
 
 extern struct buffer *current_buffer;		/* points to the current buffer */
@@ -276,4 +284,7 @@ extern struct buffer buffer_local_symbols;
 
 #define BufferSafeFloor(n) (BEGV <= GPT && GPT <= (n) ? GPT : BEGV)
 
+#define is_control_char(c)   (((c) >= 0 && (c) < 040) || (c) == 0177)
+#define is_meta_char(c) ((c) > 0240 && (c) <= 0377)
+
 extern void reset_buffer ();
