@@ -25,13 +25,13 @@ file named COPYING.  Among other things, the copyright notice
 and this notice must be preserved on all copies.  */
 
 /*
- * $Revision: 1.48 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/button.c,v $
- * $Date: 92/03/01 17:20:04 $
- * $Author: love $
+ * $Revision: 1.61 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/button.c,v $
+ * $Date: 92/08/01 22:35:11 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/03/01 17:20:04 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/button.c,v $ $Revision: 1.48 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/01 22:35:11 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/button.c,v $ $Revision: 1.61 $";
 #endif
 
 #include <stdio.h>
@@ -52,7 +52,7 @@ static char rcsid[] = "$Author: love $ $Date: 92/03/01 17:20:04 $ $Source: /impo
 #include "xresource.h"		/* Epoch */
 
 #ifdef HAVE_GRAPHIC_ZONES
-#include <X11/xpm.h>
+#include "xpm.h"
 #endif
 
 #define min(a,b) ((a)<(b) ? (a) : (b))
@@ -68,6 +68,8 @@ extern short text_width();
 extern struct X_font *font_list;
 Lisp_Object Vbuttons_modify_buffer;
 
+Lisp_Object Vmotion_style_overrides;
+
 /* this signals a read-only error if buttons-modify-buffer is set, and the
  * buffer is read-only. b is a buffer pointer, and buf should be the
  * Lisp_Object buffer corresponding to b
@@ -148,6 +150,12 @@ Return an Xresource of type Pixmap, or nil of errors encountered")
   attributes.colorsymbols = symbols;
   attributes.numsymbols = 0;
 
+  attributes.colorTable = (char***)0;
+  attributes.hints_cmt  = (char*)0;
+  attributes.colors_cmt = (char*)0;
+  attributes.pixels_cmt = (char*)0;
+  attributes.pixels     = (Pixel*)0;
+
   BLOCK_INPUT();  
   status = XpmReadFileToPixmap(XD_display,cur_Xscreen->xid,
 			       pixname,
@@ -617,65 +625,48 @@ Lisp_Object get_start_button(pos,w)
      struct window *w;
 {
   register struct buffer *b = XBUFFER(w->buffer);
-  register int start = marker_position(w->start);
-  register Lisp_Object buttons,ret_button;  
+  register int start;
+  register int end;
+  register Lisp_Object buttons,ret_button;
 
-  if (NIL(w->start_button) && NIL(b->buttons))
+  if (NIL(b->buttons))
     {
       /* Window's buffer starting pt unchanged */
-      return w->start_button;
+      return Qnil;
     }
-  if (NIL(w->start_button) ||
-      XMARKER(XBUTTON(w->start_button)->start)->buffer != b)
+
+  ret_button = buttons = b->buttons;
+  while (!NIL(buttons))
     {
-      buttons = b->buttons;
-      ret_button = b->buttons;	  
+      register struct Lisp_Button *button = XBUTTON(buttons);
+
+      start = marker_position(button->start);
+      end = marker_position(button->end);
+      ret_button = buttons;
+      
+      if (end > pos) break;
+
+      /* Consider the next one */
+
+      buttons = button->next;
     }
+
+  /* Take the previous one */
+  ret_button = XBUTTON(ret_button)->prev;
+
+  if (!NIL(ret_button))
+    {
+      start = marker_position(XBUTTON(ret_button)->start);
+      end = marker_position(XBUTTON(ret_button)->end);
+    }
+#if 0
+  if (NIL(ret_button))
+    fprintf(stderr,"Start button nil for window start %d\n",pos);
   else
-    {
-      buttons = w->start_button;
-      ret_button = w->start_button;
-    }  
-  if (pos < start)
-    {
-      /* Starting earlier in buffer; scroll backward in button list to
-       * find appropriate starting point.
-       * First check if button is null or for a different buffer; then
-       * return a worst case search of the list
-       */
-      while (!NIL(buttons))
-	{
-	  register struct Lisp_Button *button = XBUTTON(buttons);
-	  register int end = marker_position(button->end);
-
- 	  ret_button = buttons;
-	  if (end < pos)
-	    break;
-
-	  buttons = XBUTTON(buttons)->prev;
-	}
-      return EQ(ret_button,b->buttons) ? Qnil : ret_button;
-    }
-  else
-    {
-      /* Starting later in buffer; scroll forward in button list to
-       * find appropriate starting point.
-       * First check if button is null or for a different buffer; then
-       * return a worst case search of the list
-       */
-      while (!NIL(buttons))
-	{
-	  register struct Lisp_Button *button = XBUTTON (buttons);
-	  register int start = marker_position (button->start);
-
-	  if (start > pos)
-	    break;
-
-	  ret_button = buttons;
-	  buttons = XBUTTON(buttons)->next;
-	}
-      return EQ(ret_button,b->buttons) ? Qnil : ret_button;
-    }
+    fprintf(stderr,"Start button from %d to %d for window start %d\n",
+	    start,end,pos);
+#endif
+  return ret_button;
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* given a buffer and a location, figure out the style value
@@ -693,6 +684,15 @@ get_button(b,loc,b_start,b_end,b_first,b_return)
   register struct Lisp_Button *button;
   register Lisp_Object buttons, next_b = Qnil;
   register int first = -1,last = -1;
+  struct Lisp_Button *drag_button;
+  Lisp_Object drag_button_value;
+
+  if (!NIL(Vmotion_style_overrides))
+    {
+      drag_button_value = XSYMBOL(intern("drag-zone"))->value;
+      drag_button = 
+        XBUTTON(BUTTONP(drag_button_value) ? drag_button_value : Qnil);
+    }
 
   /* Possible start in list */
   if (b_first) next_b = *b_first;
@@ -700,7 +700,7 @@ get_button(b,loc,b_start,b_end,b_first,b_return)
   /* End of buffer - no more buttons in buffer after loc */
   if (b_start)  *b_start  = ZV;	
   if (b_end)    *b_end    = -1;
-  if (b_first)  *b_first = Qnil;
+/*  if (b_first)  *b_first = Qnil; */
   if (b_return) *b_return = Qnil;
 
   for (buttons = next_b; !NIL(buttons); buttons = button->next)
@@ -720,7 +720,13 @@ get_button(b,loc,b_start,b_end,b_first,b_return)
           if (b_start)  *b_start  = first;
           if (b_end)    *b_end    = last;
           if (b_return) *b_return = buttons;
-	  if (b_first)  *b_first = buttons;
+/*	  if (b_first)  *b_first = buttons; */
+
+          if (!NIL(Vmotion_style_overrides))
+            {
+              if (button == drag_button)
+                return button->style;
+            }
 
           for (buttons2 = button->next; !NIL(buttons2); buttons2 = button2->next)
             {
@@ -1574,7 +1580,7 @@ PlotTextLine(w,l,start,end,clear)
 {
   struct Root_Block *rb = XROOT(w->root);
   XS_DEF; WS_DEF;
-  char buf[300];		/* Buffer for constructing string. */
+  char buf[1000];		/* Buffer for constructing string. */
   char *pos;			/* Position in buf */
   struct char_block *cb;	/* Current char in line */
   struct Lisp_Style *s,t;	/* Current style for plotting */
@@ -1593,9 +1599,9 @@ PlotTextLine(w,l,start,end,clear)
   cursor = 2 * (l->modeline);
 
   BLOCK_INPUT();
-  while (cb != l->end)
+  while (cb && cb != l->end)
     {
-      if (cb->style == s)
+      if (cb->style == s && n < 999)
 	{
 	  *pos++ = cb->ch;
 	  /* Update attributes */
@@ -1674,7 +1680,7 @@ Lisp_Object Qbuttonp,Qstylep;
 void
 syms_of_button()
 {
-  Qbuttonp = intern ("buttonp"); staticpro(&Qbuttonp);
+  Qbuttonp = intern ("zonep"); staticpro(&Qbuttonp);
   Qstylep = intern ("stylep"); staticpro(&Qstylep);
   QXcardinalp = intern ( "Xcardinalp" ); staticpro(&QXcardinalp);
   QXfontp = intern ( "Xfontp" ); staticpro(&QXfontp);
@@ -1743,4 +1749,8 @@ syms_of_button()
   DEFVAR_LISP("epoch::zones-modify-buffer",&Vbuttons_modify_buffer,
 	      "Flag on whether zones should mark a buffer as modified.");
   Vbuttons_modify_buffer = Qt;
+
+  DEFVAR_LISP("epoch::motion-style-overrides",&Vmotion_style_overrides,
+              "*Flag on whether motion style overrides other styles.");
+  Vmotion_style_overrides = Qt;
 }
