@@ -18,14 +18,14 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.2 $
- * $Source: /home/kaplan/love/epoch/src/RCS/cmds.c,v $
- * $Date: 91/09/29 13:08:50 $
- * $Author: love $
+ * $Revision: 1.9 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/cmds.c,v $
+ * $Date: 92/07/12 03:28:36 $
+ * $Author: marca $
  *
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/09/29 13:08:50 $ $Source: /home/kaplan/love/epoch/src/RCS/cmds.c,v $ $Revision: 1.2 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/07/12 03:28:36 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/cmds.c,v $ $Revision: 1.9 $";
 #endif
 
 #include "config.h"
@@ -34,7 +34,7 @@ static char rcsid[] = "$Author: love $ $Date: 91/09/29 13:08:50 $ $Source: /home
 #include "buffer.h"
 #include "syntax.h"
 
-#ifdef HAVE_LEIF
+#ifdef DEFINE_CHANGE_FUNCTIONS
 extern Lisp_Object Vbefore_change_function;
 extern Lisp_Object Vafter_change_function;
 #endif
@@ -86,8 +86,8 @@ DEFUN ("forward-line", Fforward_line, Sforward_line, 0, 1, "p",
   "If point is on line i, move to the start of line i + ARG.\n\
 If there isn't room, go as far as possible (no error).\n\
 Returns the count of lines left to move.\n\
-With positive ARG, a non-empty line at the end counts as one line\n\
-  successfully moved (for the return value).")
+With positive ARG, a non-empty line traversed at end of buffer \n\
+counts as one line successfully moved (for the return value).")
   (n)
      Lisp_Object n;
 {
@@ -107,7 +107,7 @@ With positive ARG, a non-empty line at the end counts as one line\n\
   pos = scan_buffer ('\n', pos2, count - negp, &shortage);
   if (shortage > 0
       && (negp
-	  || (ZV > BEGV
+	  || (ZV > BEGV && pos != pos2
 	      && FETCH_CHAR (pos - 1) != '\n')))
     shortage--;
   SET_PT (pos);
@@ -272,7 +272,7 @@ self_insert_internal (c1, noautofill)
   register enum syntaxcode synt;
   register int c = c1;
 
-#ifdef HAVE_LEIF
+#ifdef DEFINE_CHANGE_FUNCTIONS
   if (!NULL (Vbefore_change_function) ||
       !NULL(Vafter_change_function))
     {
@@ -284,7 +284,7 @@ self_insert_internal (c1, noautofill)
       && c != '\n' && FETCH_CHAR (point) != '\n'
       && (FETCH_CHAR (point) != '\t'
 	  || XINT (current_buffer->tab_width) <= 0
-	  || !((current_column () + 1) % XFASTINT (current_buffer->tab_width))))
+	  || !((current_column (0) + 1) % XFASTINT (current_buffer->tab_width))))
     {
       del_range (point, point + 1);
       hairy = 1;
@@ -301,7 +301,10 @@ self_insert_internal (c1, noautofill)
   if ((c == ' ' || c == '\n')
       && !noautofill
       && !NULL (current_buffer->auto_fill_hook)
-      && current_column () > XFASTINT (current_buffer->fill_column))
+      && ((!NULL(current_buffer->fill_column) &&
+	   current_column (0) > XFASTINT (current_buffer->fill_column)) ||
+	  (XFASTINT(current_buffer->fill_pixel) &&
+	   current_pixel() > XFASTINT(current_buffer->fill_pixel))))
     {
       if (c1 != '\n')
 	insert (&c1, 1);
@@ -310,6 +313,16 @@ self_insert_internal (c1, noautofill)
 	insert (&c1, 1);
       hairy = 1;
     }
+  else if (!noautofill &&
+	   !NULL (current_buffer->line_fill_hook) &&
+	   NULL(current_buffer->fill_column) &&
+	   INTEGERP (current_buffer->fill_pixel) &&
+	   !pixel_insert_ok(c,XFASTINT(current_buffer->fill_pixel)))
+    {
+      insert(&c1,1);
+      call0(current_buffer->line_fill_hook);
+      hairy = 1;
+    }
   else
     insert (&c1, 1);
   synt = SYNTAX (c);
@@ -357,7 +370,11 @@ keys_of_cmds ()
   ndefkey (Vglobal_map, Ctl('I'), "self-insert-command");
   for (n = 040; n < 0177; n++)
     ndefkey (Vglobal_map, n, "self-insert-command");
-
+#ifdef EIGHTBIT_KEYBOARD
+  for (n = 0240; n <= 0377; n++)
+ 	ndefkey (Vglobal_map, n, "self-insert-command");
+#endif
+  
   ndefkey (Vglobal_map, Ctl ('A'), "beginning-of-line");
   ndefkey (Vglobal_map, Ctl ('B'), "backward-char");
   ndefkey (Vglobal_map, Ctl ('D'), "delete-char");
