@@ -1,3 +1,10 @@
+/*
+    6-11-90 Jim Rees: Incorporate Leonard's change into epoch 3.2.
+
+  Modified on 6-Jan-90 by Leonard N. Zubkoff:
+    Modified to work with Domain/OS SR10.2.
+*/
+
 /* C code startup routine.
    Copyright (C) 1985, 1986 Free Software Foundation, Inc.
 
@@ -20,13 +27,13 @@ You are forbidden to forbid anyone else to use, share and improve
 what you give them.   Help stamp out software-hoarding!  */
 
 /*
- * $Revision: 1.2 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/crt0.c,v $
- * $Author: love $
- * $Date: 92/02/23 11:02:50 $
+ * $Revision: 1.3 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/crt0.c,v $
+ * $Author: marca $
+ * $Date: 92/07/26 03:06:18 $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:02:50 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/crt0.c,v $ $Revision: 1.2 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/07/26 03:06:18 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/crt0.c,v $ $Revision: 1.3 $";
 #endif
 
 /* The standard Vax 4.2 Unix crt0.c cannot be used for Emacs
@@ -83,6 +90,31 @@ int errno;
 char **environ;
 #endif
 
+#ifdef APOLLO
+/* Added by John Vasta, 19-Oct-89
+   Malloc, free, etc. are in shared libraries on the Apollo. Library references
+   to them are indirect so that they can be replaced by the user, just as if
+   they were statically linked. We set the pointer values here. (The regular
+   Apollo crt0.o file isn't used because it has a compressed data section which
+   interferes with the unexec process.) */
+
+extern	char   *malloc(),      *(*_libc_malloc) ();
+extern	void	free(),		(*_libc_free) ();
+extern	char   *realloc(),     *(*_libc_realloc)();
+
+extern	int	main();
+std_$call void	unix_$main();
+
+_start()
+{
+	_libc_malloc = malloc;
+	_libc_realloc = realloc;
+	_libc_free = free;
+
+	unix_$main(main);	/* no return */
+}
+#endif /* APOLLO */
+
 #if defined(orion) || defined(pyramid) || defined(celerity) || defined(ALLIANT) || defined(clipper)
 
 #ifdef ALLIANT
