@@ -18,13 +18,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.4 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/data.c,v $
- * $Date: 92/02/23 11:02:48 $
- * $Author: love $
+ * $Revision: 1.5 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/data.c,v $
+ * $Date: 92/06/30 00:14:53 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:02:48 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/data.c,v $ $Revision: 1.4 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/06/30 00:14:53 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/data.c,v $ $Revision: 1.5 $";
 #endif
 
 #include <signal.h>
@@ -1212,8 +1212,20 @@ arith_driver
 	  break;
 	case Amult: accum *= next; break;
 	case Adiv:
+#if 0
 	  if (!argnum) accum = next;
 	  else accum /= next;
+#endif
+	  if (!argnum) 
+            accum = next;
+	  else 
+            {
+              /* Coredumping on divide by 0 in lisp code is stupid. */
+              if (next == 0)
+                accum = 0;
+              else
+                accum /= next;
+            }
 	  break;
 	case Alogand: accum &= next; break;
 	case Alogior: accum |= next; break;
@@ -1275,7 +1287,13 @@ DEFUN ("%", Frem, Srem, 2, 2, 0,
   CHECK_NUMBER (num1, 0);
   CHECK_NUMBER (num2, 1);
 
-  XSET (val, Lisp_Int, XINT (num1) % XINT (num2));
+  /* Don't allow mod by 0; assume 3 % 0 = 3.
+     Coredumping on % by 0 in lisp code is stupid. */
+  if (XINT (num2) == 0)
+    XSET (val, Lisp_Int, XINT (num1));
+  else
+    XSET (val, Lisp_Int, XINT (num1) % XINT (num2));
+  
   return val;
 }
 
