@@ -1,22 +1,23 @@
 /*
  * Epoch 4.0 display code - Abstract redisplay code.
  *
- * $Revision: 1.22 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/dispepoch.c,v $
- * $Date: 91/09/27 10:58:14 $
- * $Author: love $
+ * $Revision: 1.58 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/dispepoch.c,v $
+ * $Date: 92/08/26 23:14:04 $
+ * $Author: marca $
  */
 
 /* #define CURSOR_DEBUG */
 
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/09/27 10:58:14 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/dispepoch.c,v $ $Revision: 1.22 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/26 23:14:04 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/dispepoch.c,v $ $Revision: 1.58 $";
 #endif
 
 #include <signal.h>
 
 #include "config.h"
 #include <stdio.h>
+#include <errno.h>
 
 #ifdef HAVE_TIMEVAL
 #ifdef HPUX
@@ -56,7 +57,6 @@ static char rcsid[] = "$Author: love $ $Date: 91/09/27 10:58:14 $ $Source: /impo
 
 #include "termchar.h"
 #include "termopts.h"
-#include "cm.h"
 #include "lisp.h"
 #include "buffer.h"
 #include "window.h"
@@ -75,7 +75,7 @@ static char rcsid[] = "$Author: love $ $Date: 91/09/27 10:58:14 $ $Source: /impo
  * This ought to be built into stdio, but it isn't.
  * Some s- files overide this */
 #ifdef __GNU_LIBRARY__
-#define	PENDING_OUTPUT_COUNT(FILE) ((FILE)->__bp - (FILE)->__buf)
+#define	PENDING_OUTPUT_COUNT(FILE) ((FILE)->__bufp - (FILE)->__buffer)
 #else
 #define PENDING_OUTPUT_COUNT(FILE) ((FILE)->_ptr - (FILE)->_base)
 #endif
@@ -112,46 +112,15 @@ Lisp_Object Vwindow_system_version;
 
 int cursor_in_echo_area;
 
-
 /* Stdio stream being used for copy of all terminal output.  */
 
 FILE *termscript;
 
-/* Structure for info on cursor positioning */
-
-struct cm Wcm;
-
 int in_display;		/* 1 if in redisplay: can't handle SIGWINCH now.  */
 
-/* This buffer records the history of display preemption.  */
-
-struct preempt
-{
-  /* Number of keyboard characters read so far at preempt.  */
-  int keyboard_char_count;
-  /* Vertical position at which preemption occurred.  */
-  int vpos;
-};
-
-#define N_PREEMPTIONS 50
-
-/* Circular buffer recording recent display preemptions.  */
-struct preempt preemptions[N_PREEMPTIONS];
-
-/* Index of next element in preemptions.  */
-int preemption_index;
-
-/* Set these variables in the debugger to force a display preemption.  */
-int debug_preemption_vpos = -1;
-int debug_preemption_char_count = -1;
-
-int display_completed;
-
 extern int distinct_minibuffer;
 extern int screen_changed;
 
-extern struct X_font fake_font;
-
 extern short text_width();
 
 extern struct Root_Block *cur_root;
@@ -161,10 +130,128 @@ extern struct Root_Block *root;
 extern struct redisplay_block display_list[300];
 extern int list_ptr;
 
+struct char_block old_cur_char;
+struct line_header old_cur_line;
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+int
+scroll_screen_lines(from,end,amount,w)
+     int from,end,amount;
+     struct window *w;
+{
+  struct Root_Block *rb = XROOT(w->root);
+  WS_DEF;
+  
+  register struct line_header *l,*l1,*l2;
+  register int i;
+
+  if (amount == 0) return 1;
+
+  list_ptr++;
+#if 0
+  old_cur_char.ch = ws->cur_char->ch;
+  old_cur_char.xpos = ws->cur_char->xpos;
+  old_cur_char.style = ws->cur_char->style;
+
+  old_cur_line.chars = ws->cur_line->chars;
+  old_cur_line.ypos = ws->cur_line->ypos;
+  old_cur_line.ascent = ws->cur_line->ascent;
+  old_cur_line.descent = ws->cur_line->descent;
+#endif
+
+  /* Find start and end of region */
+  l = XLINE(w->lines);
+
+  i = from;
+  while (l && i--) l = l->next;
+  l1 = l2 = l;			/* start */
+
+  /* If we're confused, signal to do a regular layout */
+  if (!l)
+    {
+      list_ptr--;
+      return -1;
+    }
+
+  /* Find end block of region; this isn't entirely accurate, and may
+   * be clipped.
+   */
+  while (l->next && i--) l = l->next;
+  l2 = l;			/* end */
+
+  BLOCK_TYPE(list_ptr) = BLIT;
+  BLOCK_BLIT_OLD_TOP(list_ptr) = l1->ypos;
+  l1->prevy = l1->ypos;
+  BLOCK_BLIT_START(list_ptr) = l1;
+  BLOCK_BLIT_END(list_ptr) = l2;
+
+  if (amount > 0)
+    {
+      struct line_header *l3;
+
+      for (i = 0; i < amount; i++)
+	{
+	  l = get_line();
+
+	  if (l1->prev)
+	    {
+	      /* Inserting line in middle of window */
+	      l->prev = l1->prev;
+	      l1->prev->next = l;
+	    }
+	  else
+	    {
+	      /* New first line in window */
+	      XSET(w->lines,Lisp_Raw_Data,l);
+	    }
+	  
+	  l->next = l1;
+	  l1->prev = l;
+	}
+      /* At this point, ypos in block to be shifted is wrong, but can't
+       * be corrected til these new lines are layed out.
+       */
+    }
+  if (amount < 0)
+    {
+      for (i = 0; i < (-amount); i++)
+	{
+	  l = l1->prev;
+
+	  l1->prev = l->prev;	  
+	  if (l->prev)
+	    {
+	      l->prev->next = l1;
+	    }
+	  if (l == XLINE(w->lines))	  
+	    XSET(w->lines,Lisp_Raw_Data,l1);      
+	  free_line(l);
+	}
+      
+      i = l1->prev ? (l1->prev->ypos + l1->prev->descent) : w->pixtop;
+
+      while (l1)
+	{
+	  l1->prevy = l1->ypos;
+	  l1->ypos = i + l1->ascent;
+
+	  i += (l1->ascent + l1->descent);
+
+	  l1 = l1->next;
+	}
+    }
+  return 1;
+}
+     
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* Replot an Emacs window in its screen.  This will also fly for minibuffer
  * window.
  */
+#ifdef DEFINE_CHANGE_FUNCTIONS
+static int cur_buf_modif;
+#endif
+
 void
 update_window(w)
      struct window *w;
@@ -174,12 +261,20 @@ update_window(w)
   struct line_header *l,*start,*end;
   struct char_block *cb,*endc;
   int i = 0;
+  extern int DEBUG_ALL;
 
-  if (scroll_region_ok)
+  /* Running under X.  We want to find any lines which have been
+   * marked "shifted"
+   */
+#if 0  
+  if (DEBUG_ALL && w == XWINDOW(selected_window))
+    fprintf(stderr,"Updating %d\n",list_ptr);
+#endif  
+  if (list_ptr > -1)
     {
-      /* Running under X.  We want to find any lines which have been
-       * marked "shifted"
-       */
+      if (BLOCK_TYPE(0) != AREA)
+	CXTupdate_begin(w);
+
       for (i = 0; i <= list_ptr; i++)
 	if (BLOCK_TYPE(i) == LINE &&
 	    BLOCK_LINE(i)->shifted && !BLOCK_LINE(i)->changed) break;
@@ -198,38 +293,64 @@ update_window(w)
 	    }
 	  shift_region(w,start,end);
 	}
-    }
-  i = 0;
+  
+      i = 0;
 
-  while (i <= list_ptr)
-    {
-      if (!BLOCK_DONE(i))
-	switch (BLOCK_TYPE(i))
-	  {
-	  case LINE:		/* A regular line */
-	    l = BLOCK_LINE(i);
-	    cb = BLOCK_LINE_START(i);
-	    endc = BLOCK_LINE_END(i);
-	    update_line(w,l,cb,endc);	
-	    l->new = l->changed = l->shifted = 0; /* Line is up to date */
-	    break;
-	  case AREA:		/* Region to blank */
-	    clear_window_end(w,BLOCK_AREA_TOP(i),BLOCK_AREA_BOTTOM(i));
-	    break;
-	  default:
-	    break;
-	  }
-      i++;
+      while (i <= list_ptr)
+	{
+	  if (!BLOCK_DONE(i))
+	    switch (BLOCK_TYPE(i))
+	      {
+	      case LINE:	/* A regular line */
+		l = BLOCK_LINE(i);
+		cb = BLOCK_LINE_START(i);
+		endc = BLOCK_LINE_END(i);
+		update_line(w,l,cb,endc,BLOCK_LINE_CLEAR(i));
+		l->new = l->changed = l->shifted = 0; /* Line is up to date */
+		break;
+	      case AREA:	/* Region to blank */
+		clear_window_end(w,BLOCK_AREA_TOP(i),BLOCK_AREA_BOTTOM(i));
+		break;
+	      case BLIT:	/* Region to copy */
+		shift_region(w,BLOCK_BLIT_START(i),BLOCK_BLIT_END(i));
+		break;
+	      default:
+		break;
+	      }
+	  i++;
+	}
+      list_ptr = -1;
+
+      if (BLOCK_TYPE(0) != AREA)
+	CXTupdate_end(w);
     }
-  list_ptr = -1;
+
+  if (rb == cur_root && w == XWINDOW(rb->select))
+    {
+#ifdef DEFINE_CHANGE_FUNCTIONS      
+      if (w != XWINDOW(rb->minibuf_window) &&
+	  (BUF_MODIFF(XBUFFER(w->buffer)) == cur_buf_modif) &&
+	  (point != w->last_point))
+	{
+	  signal_after_movement((NIL(w->last_point)) ? point : w->last_point);
+/*	  fprintf(stderr,"Foobar\n"); */
+	}
+
+      if (w != XWINDOW(rb->minibuf_window))
+	cur_buf_modif = BUF_MODIFF(XBUFFER(w->buffer));
+#endif      
+      update_cursor(rb,(BLOCK_TYPE(0) == BLIT));
+    }
+
   display_completed = 1;
   bzero(display_list,300*sizeof(struct redisplay_block));
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* Clean up at end of redisplay for screen.
  */
-update_cursor(rb)
+update_cursor(rb,blit)
      struct Root_Block *rb;
+     int blit;
 {
   WS_DEF;
 
@@ -258,14 +379,24 @@ update_cursor(rb)
 		  ws->cursor_x,
 		  ws->new_cur_w,rb);
     }
-  else if (ws->new_cur_line != ws->cur_line ||
-	   ws->new_cur_char != ws->cur_char)
+  else 
     {
-      move_cursor(ws->new_cur_line,
-		  ws->new_cur_char,
-		  ws->cursor_y,
-		  ws->cursor_x,
-		  ws->new_cur_w,rb);
+      if (blit)
+	{
+	  ws->cur_char = ws->new_cur_char;
+	  ws->cur_line = ws->new_cur_line;
+	  ws->cur_w = ws->new_cur_w;
+	  ws->vis_ypos = ws->cursor_y;
+	  ws->vis_xpos = ws->cursor_x;
+
+	  Fastmove_cursor(rb);
+	}
+      else
+	move_cursor(ws->new_cur_line,
+		    ws->new_cur_char,
+		    ws->cursor_y,
+		    ws->cursor_x,
+		    ws->new_cur_w,rb);
     }
 #ifdef CURSOR_DEBUG
   if (rb != mini_root)
@@ -275,7 +406,6 @@ update_cursor(rb)
 	      ws->vis_xpos,ws->vis_ypos,ws->cur_char->ch);      
     }
 #endif
-  update_end();
 }
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -285,18 +415,16 @@ int
 output_forward_char(n)
      int n;
 {
-  register struct window *w = XWINDOW(selected_window);
-  struct Root_Block *rb = XROOT(w->root);
-  struct W_Screen *ws = XWSCREEN(rb->win);
-  struct char_block *cb,*cb1;
-  int i = ws->cursor_y;
+  register struct window *w = XWINDOW(selected_window);  
+  struct Root_Block *rb = XROOT(w->root);  
+  register struct W_Screen *ws = XWSCREEN(rb->win);
 
-  if ((n < 0 && ws->cur_char == ws->cur_line->body) ||
-      (n > 0 && ws->cur_char == ws->cur_line->end) ||
-      (ws->vis_xpos > ws->cur_line->chars) ||
-      w->hscroll > 0)
-    return 0;			/* Silently fail at window edges */
-    
+  /* Silently fail at window edges or on horizontally scrolled windows */
+  if ((n < 0 && (ws->vis_xpos == 0 || ws->vis_xpos == ws->cur_line->chars)) ||
+      (n > 0 && ws->vis_xpos >= ws->cur_line->chars - 1) ||
+      w->hscroll)
+    return 0;
+  
   /* Set new cursor position.  Moving cursor will set this equal to the
    * prev position.
    */
@@ -305,22 +433,28 @@ output_forward_char(n)
   ws->new_cur_char = (n > 0) ? ws->cur_char->next : ws->cur_char->prev;
   ws->new_cur_w = w;
 
+#ifdef DEFINE_CHANGE_FUNCTIONS
+  if (w != XWINDOW(rb->minibuf_window))
+    {
+      signal_after_movement((NIL(w->last_point)) ? point : w->last_point);
+/*      fprintf(stderr,"Foobar\n"); */
+    }
+#endif  
+  
   XFASTINT(w->last_point_x) = ws->cursor_x;
   XFASTINT(w->last_point) = point;  
 
   move_cursor(ws->new_cur_line,ws->new_cur_char,
 	      ws->cursor_y,ws->cursor_x,ws->new_cur_w,rb);
   
-  if (inhibit_window_system)
-    fflush(stdout);
   return 1;
-
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* Handle insertion of a character at 'cur_line,cur_char'
  * Assume that redisplay() is atomic for each screen and won't be preempted.
- * If much is waiting (i.e. fast typing), caller (command_loop_1()) will call
- * something with remaining chars.
+ * Do fast insertion of single char on current line, assuming display
+ * changes will be bounded by current line.  If insertion is in middle of
+ * line, utilize bit blt'ing under X/terminal insert-char operation.
  */
 int
 output_for_insert(c)
@@ -330,52 +464,69 @@ output_for_insert(c)
   struct Root_Block *rb = XROOT(w->root);
   register struct W_Screen *ws = XWSCREEN(rb->win);
   register struct line_header *l = ws->cur_line;
-  register struct char_block *new,*cb = ws->cur_char,*prev;
+  register struct char_block *new,*cb = ws->cur_char;
   register struct buffer *b = current_buffer;
-  char a[1];			/* CRUDE */
+  char a[2];			/* CRUDE */
   int pixright;
-  Lisp_Object ob;
   struct X_font *font;
-
+  extern Lisp_Object Voverlay_arrow_position;
+  extern Lisp_Object Voverlay_arrow_string;
+  
   if (
       /* Selected window is minibuffer & msg is displayed */
       (EQ(selected_window,minibuf_window) && echo_area_contents) ||
-      (w->hscroll > 0) || 
+      /* Window is horizontally scrolled */
+      (w->hscroll > 0) ||
+      /* Cursor on first character on line */
       (cb == l->body) ||
-      ws->cursor_y < 0 ||
-      ws->cursor_y > window_char_height(w) ||
+      /* Invalid cursor position info */
+      (ws->cursor_y < 0) ||
+      (ws->cursor_y > w->height) ||
       !display_completed ||
-      buffer_shared > 1      
+      /* Buffer is shared */
+      buffer_shared > 1 ||
+      /* Line contains tabs */
+      l->tabs ||
+      /* Overlay arrow string is in this buffer */
+      (XTYPE(Voverlay_arrow_string) == Lisp_String &&
+       XTYPE(Voverlay_arrow_position) == Lisp_Marker &&
+       XMARKER(Voverlay_arrow_position)->buffer == b)
       )
-      return 0;			/* Silently fail in these cases */
+    return 0;			/* Silently fail in these cases */
 
+  a[1]=0;
   new = get_block();
   new->ch = a[0] = c;
-  ob = get_button(current_buffer,point,0,0,0);
-  new->style = ws->cursor_x > 0 ? cb->style :
-    NULL(ob) ? 0 : XSTYLE(ob);
-
-  if (!NULL(ob) && (XTYPE(XSTYLE(ob)->font) == Lisp_Raw_Data))
-    font = XXFONT(new->style->font);
+  if (cb->style == cb->prev->style)
+    new->style = cb->style;
   else
-    font = XXFONT(STYLE_FIELD(w,font,stylenorm));
+    {
+      Lisp_Object ob;
+      ob = get_button(b,point,0,0,&w->start_button,0);
+      if (!NULL(ob))
+	new->style = XSTYLE(ob);
+    }
+
+  font = XXFONT(STYLE_FIELD(w,font,stylenorm));    
+  if (new->style && (XTYPE(new->style->font) == Lisp_Raw_Data))
+    font = XXFONT(new->style->font);
+
   new->new = 1;			/* Flag -> this is new */
-  new->width = text_width(font->x_font,a,1);
+  new->width = text_width(font,a,1);
   new->xpos = cb->xpos;
   
   a[0] = '\\';
   pixright = XFASTINT(w->pixleft) + XFASTINT(w->pixwidth) -
-    text_width(font->x_font,a,1);
+    text_width(font,a,1);
     
-  if ((l->lwidth + new->width) >= (pixright) ||
-      font->ascent > l->ascent ||
-      font->descent > l->descent)
+  if ((l->lwidth + new->width) >= pixright ||
+      XFASCENT(font) > l->ascent ||
+      XFDESCENT(font) > l->descent)
     {
       /* Fail here if we are going to go over the edge of this window, or
        * the new character's font is a different height than the line's
        * ascent or descent.
        */
-      l->changed = 1;
       free_blocks(new,new);
       return 0;
     }
@@ -388,41 +539,63 @@ output_for_insert(c)
   cb->prev = new;
 
   /* Adjust attributes for rest of line */
-  prev = new;
-  while (cb)
-    {
-      cb->xpos = prev->xpos + prev->width;
-      prev = cb;
-      cb = cb->next;
-    }
-  
+  for ( ; cb; cb->xpos += new->width, cb = cb->next);
+
+  /* Update the display */
+  if (new->next == l->end)
+    update_line(w,l,new,0,0);	/* Don't clear end of line */	
+  else
+    insert_chars(w,l,new,new->next,l->end,0); /* Faster */      
+
   l->lwidth += new->width;
   l->chars++;
-  
-  if (inhibit_window_system)
-    {
-      fflush(stdout);
-    }
-  else
-    {
-      update_line(w,l,new,0);
-      ws->cursor_x += 1;
-      move_cursor(ws->new_cur_line,ws->new_cur_char,
-		  ws->cursor_y,ws->cursor_x,ws->new_cur_w,rb);
 
-#ifdef CURSOR_DEBUG      
-      if (rb != mini_root)
-	{
-	  fprintf(stderr,"Cursor at %d,%d (%c)\tPreviously at %d,%d (%c)\n",
-		  ws->cursor_x,ws->cursor_y,ws->new_cur_char->ch,
-		  ws->vis_xpos,ws->vis_ypos,ws->cur_char->ch);
-	}
-#endif      
-    }
+  /* Update the cursor */
+  ws->cursor_x += 1;
+  XFASTINT(w->last_point_x) = ws->cursor_x;
+  Fastmove_cursor(rb);
 
   return 1;
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+/* For pixel based fill mode, return info about whether inserting the next
+ * character will push end of current line past fill-pixel value.
+ * Return 1 if insertion will succeed, 0 if other action should take place
+ * first
+ */
+int
+pixel_insert_ok(c,fill)
+     register int c;
+     register int fill;
+{
+  register struct line_header *l = cur_Wscreen->cur_line;
+  register struct char_block *cb = cur_Wscreen->cur_char;
+  register struct buffer *b = current_buffer;
+  register struct Lisp_Style *s;
+  register struct X_font *font;
+  
+  char a[2];
+
+  if (l->lwidth == 0) return 1;	/* Empty line */
+
+  if (cb && (!cb->prev || cb->prev->style == cb->style))
+    s = cb->style;
+  else
+    {
+      Lisp_Object ob = get_button(b,point,0,0,
+				  &XWINDOW(selected_window)->start_button,
+				  0);
+      if (!NULL(ob))
+	s = XSTYLE(ob);
+    }
+  font = XXFONT(STYLE_FIELD(XWINDOW(selected_window),font,stylenorm));    
+  if (s && (XTYPE(s->font) == Lisp_Raw_Data))
+    font = XXFONT(s->font);  
+  a[1] = c; a[0] = 0;
+
+  return ((l->lwidth + text_width (font,a,1)) <= fill) ? 1 : 0;
+}
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* Change screen height and/or width.  Note that new height and width are
  * now in pixels.  ASCII behaviour should be unchanged.
  */
@@ -435,6 +608,10 @@ change_screen_size(height,width,pretend,rb)
   
   if (in_display || gc_in_progress)
     {
+#if 0      
+      fprintf(stderr,"Holding a resize for %d\n",rb->seq);
+#endif
+      
       ws->size_change_pending = 1;
       ws->new_width = width;
       ws->new_height = height;
@@ -450,7 +627,10 @@ change_screen_size_1(height,width,pretend,rb)
      struct Root_Block *rb;
 {
   WS_DEF;
-
+#if 0
+  fprintf(stderr,"Resizing %d, %d\n",rb->seq,pretend);
+#endif
+  
   if ((height == 0 || height == ws->pixh) &&
       (width == 0 || width == ws->pixw))
     {
@@ -464,12 +644,16 @@ change_screen_size_1(height,width,pretend,rb)
     {
       if (height > MScreenLength) height = MScreenLength;
       ws->height = height / XFHEIGHT(XXFONT(XSTYLE(rb->stylenorm)->font));
+      /* Window shouldn't become less than 1 line high. */
+      if (ws->height < 1) ws->height = 1;
       ws->pixh = height;
       if (distinct_minibuffer)
 	set_window_height(rb->ewin,height,1);
       else
 	{
 	  /* Fudge around screen's minibuffer */
+	  XWINDOW(rb->minibuf_window)->pixheight =
+	    XFHEIGHT(XXFONT(XSTYLE(rb->stylenorm)->font));
 	  height -= XWINDOW(rb->minibuf_window)->pixheight;
 	  XWINDOW(rb->minibuf_window)->pixtop = ws->in_border + height;
 	  set_window_height(rb->ewin,height,1);
@@ -512,21 +696,20 @@ window_change_signal ()
 }
 #endif /* SIGWINCH */
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-/* Initialization done when Emacs fork is started, before doing stty.
- * Determine terminal type and set terminal_driver
- * Then invoke its decoding routine to set up variables
- * in the terminal package
+/* Initialization done when Epoch fork is started.
  */
 char *terminal_type;
 
 init_display ()
 {
+  extern char **xargv;
   
   Lisp_Object style;  
 #ifdef HAVE_X_WINDOWS
   extern Lisp_Object Vxterm;
   Vxterm = Qnil;
 #endif
+
   
   Vwindow_system = Qnil;
   meta_key = 0;
@@ -534,7 +717,6 @@ init_display ()
   cursor_in_echo_area = 0;
   terminal_type = (char *) 0;
 
-
   if (!inhibit_window_system)
     {
 #ifdef HAVE_X_WINDOWS
@@ -542,62 +724,24 @@ init_display ()
       char *disp = (char *) egetenv ("DISPLAY");
 
       /* Note KSH likes to provide an empty string as an envvar value.  */
-      if (XD_display_name || (disp && *disp))
+      if ((XD_display_name && *XD_display_name) || (disp && *disp))
 	{
-#if OLD
-	  distinct_minibuffer = 1; /* Default? */
-
-	  distinct_minibuffer = 0;
-#endif	  
 	  x_term_init ();
 	  Vxterm = Qt;
 	  Vwindow_system = intern ("x");
 	  Vwindow_system_version = make_number (11);
+
 	  goto term_init_done;
 	}
 #endif /* HAVE_X_WINDOWS */
       ;
     }
-  /* Record we aren't using a window system.  */
-  inhibit_window_system = 1;
-  distinct_minibuffer = 0;	/* Epoch */
-  x_init_roots();		/* Set up root block appropriately */
 
-  /* Look at the TERM variable */
-  terminal_type = (char *) getenv ("TERM");
-  if (!terminal_type)
-    {
-#ifdef VMS
-      fprintf (stderr, "Please specify your terminal type.\n\
-For types defined in VMS, use  set term /device=TYPE.\n\
-For types not defined in VMS, use  define emacs_term \"TYPE\".\n\
-\(The quotation marks are necessary since terminal types are lower case.)\n");
-#else
-      fprintf (stderr, "Please set the environment variable TERM; see tset(1).\n");
-#endif
-      exit (1);
-    }
-  term_init (terminal_type);
-  /* Fake a 1 x 1 pixel font. */
-  fake_font.x_font = 0;
-  fake_font.ascent = 1;
-  fake_font.descent = 0;
-  fake_font.width = 1;
-  /* fake an _ x _ pixel screen = char geometry */
-  style = Fepoch_make_style();
-  XSET(cur_root->stylenorm,Lisp_Style,style);
-  XSET(XSTYLE(style)->font,Lisp_Raw_Data,&fake_font);
+  /* Punt and run ASCII */
+  punt_and_run_emacs(xargv);
 
-  cur_Wscreen->in_border = 1;
-  cur_Wscreen->pixw = cur_Wscreen->width;
-  cur_Wscreen->pixh = cur_Wscreen->height;
-  /* Set hook for finding text widths */
-
- term_init_done:
+  term_init_done:
   ;
-
-/* if (distinct_minibuffer) */
-
 #ifdef SIGWINCH
 #ifndef CANNOT_DUMP
   if (initialized)
@@ -710,7 +854,7 @@ DEFUN ("sleep-for", Fsleep_for, Ssleep_for, 1, 1, 0,
     return Qnil;
 
 #ifdef subprocesses
-  wait_reading_process_input (t, 0, 0);
+  wait_reading_process_input (t, 0, 0,0);
 #else /* No subprocesses */
   immediate_quit = 1;
   QUIT;
@@ -729,13 +873,12 @@ DEFUN ("sleep-for", Fsleep_for, Ssleep_for, 1, 1, 0,
   while (1)
     {
       gettimeofday (&timeout, &garbage1);
-      timeout.tv_sec = end_time.tv_sec - timeout.tv_sec;
-      timeout.tv_usec = end_time.tv_usec - timeout.tv_usec;
-      if (timeout.tv_usec < 0)
-	timeout.tv_usec += 1000000,
-      timeout.tv_sec--;
-      if (timeout.tv_sec < 0)
+
+      /* In effect, timeout = end_time - timeout.
+	 Break if result would be negative. */
+      if (timeval_subtract (&timeout, end_time, timeout))
 	break;
+
       if (!select (1, 0, 0, 0, &timeout))
 	break;
     }
@@ -753,6 +896,40 @@ DEFUN ("sleep-for", Fsleep_for, Ssleep_for, 1, 1, 0,
   return Qnil;
 }
 
+#ifdef HAVE_TIMEVAL
+
+/* Subtract the `struct timeval' values X and Y,
+   storing the result in RESULT.
+   Return 1 if the difference is negative, otherwise 0.  */
+
+int
+timeval_subtract (result, x, y)
+     struct timeval *result, x, y;
+{
+  /* Perform the carry for the later subtraction by updating y.
+     This is safer because on some systems
+     the tv_sec member is unsigned.  */
+  if (x.tv_usec < y.tv_usec)
+    {
+      int nsec = (y.tv_usec - x.tv_usec) / 1000000 + 1;
+      y.tv_usec -= 1000000 * nsec;
+      y.tv_sec += nsec;
+    }
+  if (x.tv_usec - y.tv_usec > 1000000)
+    {
+      int nsec = (y.tv_usec - x.tv_usec) / 1000000;
+      y.tv_usec += 1000000 * nsec;
+      y.tv_sec -= nsec;
+    }
+
+  /* Compute the time remaining to wait.  tv_usec is certainly positive.  */
+  result->tv_sec = x.tv_sec - y.tv_sec;
+  result->tv_usec = x.tv_usec - y.tv_usec;
+   /* Return indication of whether the result should be considered negative. */
+  return x.tv_sec < y.tv_sec;
+}
+#endif /* HAVE_TIMEVAL */
+
 DEFUN ("sit-for", Fsit_for, Ssit_for, 1, 2, 0,
   "Perform redisplay, then wait for ARG seconds or until input is available.\n\
 Optional second arg non-nil means don't redisplay.\n\
@@ -793,7 +970,7 @@ Value is t if waited the full time with no input arriving.")
 
 	do
 	  {
-	    wait_reading_process_input (wait_time, 1, 1);
+	    wait_reading_process_input (wait_time, 1, 1,0);
 	    detect_input_pending();
 	    Fepoch_dispatch_events();
 	    wait_time -= time(0) - start_time;
@@ -841,7 +1018,7 @@ syms_of_display ()
     "*Non-nil means try to flash the screen to represent a bell.");
   DEFVAR_BOOL ("no-redraw-on-reenter", &no_redraw_on_reenter,
     "*Non-nil means no need to redraw entire screen after suspending.\n\
-It is up to you to set this variable to inform Emacs.");
+It is up to you to set this variable to inform Emacs.");  
   DEFVAR_LISP ("window-system", &Vwindow_system,
     "A symbol naming the window-system under which Emacs is running,\n\
 \(such as `x'), or nil if emacs is running on an ordinary terminal.");
