@@ -1,3 +1,7 @@
+/*
+    11-6-90 Jim Rees: Eliminate old Apollo cruft.
+*/
+
 /* Fully extensible Emacs, running on Unix, intended for GNU.
    Copyright (C) 1985, 1986, 1987, 1990 Free Software Foundation, Inc.
 
@@ -18,24 +22,52 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.5 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/emacs.c,v $
- * $Date: 91/09/08 22:08:48 $
- * $Author: love $
+ * $Revision: 1.15 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/emacs.c,v $
+ * $Date: 92/07/26 03:06:20 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/09/08 22:08:48 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/emacs.c,v $ $Revision: 1.5 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/07/26 03:06:20 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/emacs.c,v $ $Revision: 1.15 $";
 #endif
 
 #include <signal.h>
 #include <errno.h>
 
 #include "config.h"
-#include <stdio.h>
+#ifdef NULL
 #undef NULL
+#endif
 #include "lisp.h"
+#undef NULL
+#include "paths.h"
 #include "commands.h"
 
+/* Get FIONREAD, if it is available,
+   just to help decide whether SIGIO should be defined.  */
+#ifdef USG
+#include <termio.h>
+#include <fcntl.h>
+#else /* not USG */
+#ifndef VMS
+#include <sys/ioctl.h>
+#endif /* not VMS */
+#endif /* not USG */
+
+/* Allow m- file to inhibit use of FIONREAD.  */
+#ifdef BROKEN_FIONREAD
+#undef FIONREAD
+#endif
+
+/* We are unable to use interrupts if FIONREAD is not available,
+   so flush SIGIO so we won't try. */
+#ifndef FIONREAD
+#ifdef SIGIO
+#undef SIGIO
+#endif
+#endif
+
+#include <stdio.h>
 #include <sys/types.h>
 #include <sys/file.h>
 
@@ -51,12 +83,6 @@ static char rcsid[] = "$Author: love $ $Date: 91/09/08 22:08:48 $ $Source: /impo
 #include <sys/ioctl.h>
 #endif
 
-#ifdef APOLLO
-#ifndef APOLLO_SR10
-#include <default_acl.h>
-#endif
-#endif
-
 #ifndef O_RDWR
 #define O_RDWR 2
 #endif
@@ -118,6 +144,7 @@ int fatal_error_code;
 int fatal_error_in_progress;
 
 /* Handle bus errors, illegal instruction, etc. */
+VOID
 fatal_error_signal (sig)
      int sig;
 {
@@ -164,6 +191,17 @@ fatal_error_signal (sig)
 #endif /* not VMS */
 }
 
+
+punt_and_run_emacs(argv)
+     char **argv;
+{
+  char *path = PATH_ASCII;
+
+  /* Run ASCII emacs with these cmd line args */
+
+  execv(path,argv);
+}
+
 /* Code for dealing with Lisp access to the Unix command line */
 
 static
@@ -187,7 +225,7 @@ init_cmdargs (argc, argv, skip_args)
 
   for (i = argc - 1; i >= 0; i--)
     {
-      if (i == 0 || i > skip_args)
+      if ((i == 0 || i > skip_args) && strcmp(argv[i],"-ud"))
 	Vcommand_line_args
 	  = Fcons (build_string (argv[i]), Vcommand_line_args);
     }
@@ -196,7 +234,12 @@ init_cmdargs (argc, argv, skip_args)
 #ifdef VMS
 #ifdef LINK_CRTL_SHARE
 #ifdef SHAREABLE_LIB_BUG
+#ifdef __GNUC__
+#define       environ $$PsectAttributes_NOSHR$$environ
+extern char **environ;
+#else
 extern noshare char **environ;
+#endif
 #endif /* SHAREABLE_LIB_BUG */
 #endif /* LINK_CRTL_SHARE */
 #endif /* VMS */
@@ -224,13 +267,13 @@ main (argc, argv, envp)
   if (argc > 1 && !strcmp (argv[1], "-nl"))
     {
       map_in_data (0);
-      /* The shared momory was just restored, which clobbered this.  */
+      /* The shared memory was just restored, which clobbered this.  */
       skip_args = 1;
     }
   else
     {
       map_in_data (1);
-      /* The shared momory was just restored, which clobbered this.  */
+      /* The shared memory was just restored, which clobbered this.  */
       skip_args = 0;
     }
 #endif
@@ -265,15 +308,6 @@ main (argc, argv, envp)
 
   clearerr (stdin);
 
-#ifdef APOLLO
-#ifndef APOLLO_SR10
-  /* If USE_DOMAIN_ACLS environment variable exists,
-     use ACLs rather than UNIX modes. */
-  if (egetenv ("USE_DOMAIN_ACLS"))
-    default_acl (USE_DEFACL);
-#endif
-#endif /* APOLLO */
-
 #ifndef SYSTEM_MALLOC
   /* Arrange for warnings when nearly out of space.  */
   malloc_init (0, malloc_warning);
@@ -298,6 +332,8 @@ main (argc, argv, envp)
       open (argv[skip_args], O_RDWR, 2 );
       dup (0);
       fprintf (stderr, "Using %s\n", argv[skip_args]);
+
+      punt_and_run_emacs(argv);
       inhibit_window_system = 1;	/* -t => -nw */
       distinct_minibuffer = 0;	/* no distinct minibuffer */
     }
@@ -306,32 +342,31 @@ main (argc, argv, envp)
       && (!strcmp (argv[skip_args + 1], "-nw")))
     {
       skip_args += 1;
+      punt_and_run_emacs(argv);
       inhibit_window_system = 1; /* no X window system */
       distinct_minibuffer = 0;	/* No distinct minibuffer */
     }
 
-  if (skip_args + 1 < argc
-      && (!strcmp (argv[skip_args +1], "-nm")))
-    {
-      skip_args += 1;
-      distinct_minibuffer = 0;
-    }
-
 /* Handle the -batch switch, which means don't do interactive display.  */
   noninteractive = 0;
   if (skip_args + 1 < argc && !strcmp (argv[skip_args + 1], "-batch"))
     {
+      if (initialized)
+	punt_and_run_emacs(argv);
       skip_args += 1;
       noninteractive = 1;
     }
 
 #ifdef POSIX_SIGNALS
-  /* initialize some empty and full sigsets - DJB */
-  /* these are declared in sysdep.c */
-  sigemptyset (&empty_mask);
-  sigfillset (&full_mask);
+  init_signals();
 #endif /* POSIX_SIGNALS */
 
+#ifdef HAVE_TZSET
+  /* Reinitialize the time zone if it was initialized before dumping Emacs.  */
+  if (initialized)
+    tzset ();
+#endif
+
   if (
 #ifndef CANNOT_DUMP
       ! noninteractive || initialized
@@ -361,18 +396,28 @@ main (argc, argv, envp)
 #endif
 #ifdef SIGXFSZ
       signal (SIGXFSZ, fatal_error_signal);
-#endif SIGXFSZ
+#endif /* SIGXFSZ */
 
 #ifdef AIX
-      signal (SIGDANGER, fatal_error_signal);
+      /* This used to run fatal_error_signal,
+	 but it isn't fatal.  There's nothing Emacs can usefully do.
+	 Might as well let the system kill us if it insists.  */
+      signal (SIGDANGER, SIG_IGN);
       signal (20, fatal_error_signal);
       signal (21, fatal_error_signal);
       signal (22, fatal_error_signal);
       signal (23, fatal_error_signal);
       signal (24, fatal_error_signal);
+#ifdef SIGIO  
       signal (SIGAIO, fatal_error_signal);
       signal (SIGPTY, fatal_error_signal);
-      signal (SIGIOINT, fatal_error_signal);
+#endif  
+#ifdef SIGURG
+      /* Note that SIGIOINT is the same as SIGIO on some machines,
+         and the same as SIGURG on others.  It seems ore reliable to use the
+         name with a uniform meaning.  */
+      signal (SIGURG, fatal_error_signal);
+#endif
       signal (SIGGRANT, fatal_error_signal);
       signal (SIGRETRACT, fatal_error_signal);
       signal (SIGSOUND, fatal_error_signal);
@@ -534,7 +579,7 @@ return ARG as the exit program code.")
 
   GCPRO1 (arg);
 
-  if (!NULL (Vkill_emacs_hook))
+  if (!EQ(Vkill_emacs_hook, Qnil))
     call0 (Vkill_emacs_hook);
 
   if (feof (stdin))
@@ -581,8 +626,6 @@ return ARG as the exit program code.")
 }
 
 #ifndef CANNOT_DUMP
-/* Nothing like this can be implemented on an Apollo.
-   What a loss!  */
 
 #ifdef HAVE_SHM
 
@@ -629,7 +672,7 @@ Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
 
   CHECK_STRING (intoname, 0);
   intoname = Fexpand_file_name (intoname, Qnil);
-  if (!NULL (symname))
+  if (!EQ (symname, Qnil))
     {
       CHECK_STRING (symname, 0);
       if (XSTRING (symname)->size)
@@ -649,7 +692,7 @@ Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
   malloc_init (&my_edata, malloc_warning);
 #endif
   unexec (XSTRING (intoname)->data,
-	  !NULL (symname) ? XSTRING (symname)->data : 0, &my_edata, 0, 0);
+	  !EQ (symname, Qnil) ? XSTRING (symname)->data : 0, &my_edata, 0, 0);
 #endif /* not VMS */
 
   Vpurify_flag = tem;
