@@ -18,13 +18,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.4 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/eval.c,v $
- * $Date: 92/02/23 11:00:55 $
- * $Author: love $
+ * $Revision: 1.5 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/eval.c,v $
+ * $Date: 92/08/05 04:15:52 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:00:55 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/eval.c,v $ $Revision: 1.4 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/05 04:15:52 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/eval.c,v $ $Revision: 1.5 $";
 #endif
 
 #include "config.h"
@@ -1049,12 +1049,18 @@ See  condition-case.")
   if (gc_in_progress || waiting_for_input)
     abort ();
 
+
+  /* Lockup should no longer occur in this situation, since event
+     handler itself is not responsible for removing events from the
+     queue.  -marca 5 Aug '92 */
+#if 0
   /* if we are getting error conditions while in the coed_hook we need to
    * remove the handler so we don't lock up. Flag reset is handled by an
    * unwind-protect in the handler invocation function.
    */
   if (x_in_handler_hook && !EQ(Vx_event_handler_abort,Qnil))
       Vx_event_handler = Qnil;  
+#endif
   
 
   conditions = Fget (sig, Qerror_conditions);
