@@ -1,3 +1,7 @@
+/*
+    11-6-90 Jim Rees: Eliminate old Apollo cruft.
+*/
+
 /* File IO for GNU Emacs.
    Copyright (C) 1985, 1986, 1987, 1988, 1990 Free Software Foundation, Inc.
 
@@ -18,13 +22,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.6 $
- * $Date: 92/02/23 11:01:30 $ 
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/fileio.c,v $
- * $Author: love $
+ * $Revision: 1.8 $
+ * $Date: 92/07/26 03:06:21 $ 
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/fileio.c,v $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:01:30 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/fileio.c,v $ $Revision: 1.6 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/07/26 03:06:21 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/fileio.c,v $ $Revision: 1.8 $";
 #endif
 
 #include <sys/types.h>
@@ -48,8 +52,6 @@ static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:01:30 $ $Source: /impo
 #include <perror.h>
 #include <stddef.h>
 #include <string.h>
-#else
-#include <sys/dir.h>
 #endif
 #include <errno.h>
 
@@ -73,6 +75,12 @@ extern int sys_nerr;
 #include "buffer.h"
 #include "window.h"
 
+#ifdef SYSV_SYSTEM_DIR
+#include <dirent.h>
+#else
+#include <sys/dir.h>
+#endif
+
 #ifdef VMS
 #include <file.h>
 #include <rmsdef.h>
@@ -1125,9 +1133,6 @@ that the old one has.  (This works on only some systems.)")
 	}
 #endif /* HAVE_TIMEVALS */
 
-#ifdef APOLLO
-      if (!egetenv ("USE_DOMAIN_ACLS"))
-#endif
       chmod (XSTRING (newname)->data, st.st_mode & 07777);
     }
 
@@ -1509,36 +1514,8 @@ Only the 12 low bits of MODE are used.")
   abspath = Fexpand_file_name (filename, current_buffer->directory);
   CHECK_NUMBER (mode, 1);
 
-#ifndef APOLLO
   if (chmod (XSTRING (abspath)->data, XINT (mode)) < 0)
     report_file_error ("Doing chmod", Fcons (abspath, Qnil));
-#else /* APOLLO */
-  if (!egetenv ("USE_DOMAIN_ACLS"))
-    {
-      struct stat st;
-      struct timeval tvp[2];
-
-      /* chmod on apollo also change the file's modtime; need to save the
-	 modtime and then restore it. */
-      if (stat (XSTRING (abspath)->data, &st) < 0)
-	{
-	  report_file_error ("Doing chmod", Fcons (abspath, Qnil));
-	  return (Qnil);
-	}
- 
-      if (chmod (XSTRING (abspath)->data, XINT (mode)) < 0)
- 	report_file_error ("Doing chmod", Fcons (abspath, Qnil));
- 
-      /* reset the old accessed and modified times.  */
-      tvp[0].tv_sec = st.st_atime + 1; /* +1 due to an Apollo roundoff bug */
-      tvp[0].tv_usec = 0;
-      tvp[1].tv_sec = st.st_mtime + 1; /* +1 due to an Apollo roundoff bug */
-      tvp[1].tv_usec = 0;
- 
-      if (utimes (XSTRING (abspath)->data, tvp) < 0)
- 	report_file_error ("Doing utimes", Fcons (abspath, Qnil));
-    }
-#endif /* APOLLO */
 
   return Qnil;
 }
@@ -1688,9 +1665,6 @@ before the error is signaled.")
   if (!NULL (visit))
     {
       current_buffer->undo_list = Qnil;
-#ifdef APOLLO
-      stat (XSTRING (filename)->data, &st);
-#endif
       current_buffer->modtime = st.st_mtime;
       current_buffer->save_modified = MODIFF;
       current_buffer->auto_save_modified = MODIFF;
