@@ -18,13 +18,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.6 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/fns.c,v $
- * $Date: 91/10/18 12:26:51 $
- * $Author: love $
+ * $Revision: 1.7 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/fns.c,v $
+ * $Date: 92/07/11 21:31:57 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/10/18 12:26:51 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/fns.c,v $ $Revision: 1.6 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/07/11 21:31:57 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/fns.c,v $ $Revision: 1.7 $";
 #endif
 
 #include "config.h"
@@ -55,6 +55,10 @@ static char rcsid[] = "$Author: love $ $Date: 91/10/18 12:26:51 $ $Source: /impo
 #ifndef NLIST_STRUCT
 #include <a.out.h> 
 #else /* NLIST_STRUCT */
+#ifdef SUNOS_5
+#include <fcntl.h>
+#include <kvm.h>
+#endif
 #include <nlist.h>
 #endif /* NLIST_STRUCT */
 #endif /* not VMS */
@@ -1167,6 +1171,10 @@ and then turned into integers).")
 #endif /* eunice */
 #endif /* VMS */
 
+#ifdef SUNOS_5
+  static kvm_t *kd;
+#endif /* SUNOS_5 */
+
   /* If this fails for any reason, we can return (0 0 0) */
   load_ave[0] = 0.0; load_ave[1] = 0.0; load_ave[2] = 0.0;
 
@@ -1237,11 +1245,22 @@ and then turned into integers).")
 	    nl[0].n_value &= 0x7fffffff;
 	}
 #else
+#ifdef SUNOS_5
+      if (initialized == 0) 
+        {
+          if ((kd = kvm_open(0, 0, 0, O_RDONLY, 0)) != 0) 
+            {
+              initialized = 1;
+              kvm_nlist(kd, nl);
+            }
+        }
+#else /* SUNOS_5 */
 #ifdef NeXT
       nlist (KERN_FILE, nl);
 #else      
       nlist (KERNEL_FILE, nl);
 #endif /* NeXT */      
+#endif /* SUNOS_5 */
 #endif /* IRIS */
 
 #ifdef FIXUP_KERNEL_SYMBOL_ADDR
@@ -1249,6 +1268,7 @@ and then turned into integers).")
 	nl[0].n_value = (nlp->n_value >> 2) | 0xc0000000;
 #endif /* FIXUP_KERNEL_SYMBOL_ADDR */
     }
+#ifndef SUNOS_5
   /*
    *	Make sure we have /dev/kmem open
    */
@@ -1258,13 +1278,26 @@ and then turned into integers).")
        *	Open /dev/kmem
        */
       channel = open ("/dev/kmem", 0);
-      if (channel >= 0) initialized = 1;
+      if (channel >= 0) 
+        initialized = 1;
     }
+#endif /* !SUNOS_5 */
   /*
    *	If we can, get the load ave values
    */
   if ((nl[0].n_value != 0) && (initialized != 0))
     {
+#ifdef SUNOS_5
+      /*
+       * try to read load
+       */
+      if (kvm_read(kd, nl[0].n_value, (char *)load_ave, sizeof(load_ave))
+	  != sizeof(load_ave))
+        {
+          kvm_close(kd);
+          initialized = 0;
+	}
+#else /* SUNOS_5 */
       /*
        *	Seek to the correct address
        */
@@ -1275,9 +1308,10 @@ and then turned into integers).")
 	  close (channel);
 	  initialized = 0;
 	}
+#endif /* SUNOS_5 */
     }
 #endif /* not VMS */
-
+  
   /*
    *	Return the list of load average values
    */
@@ -1294,11 +1328,11 @@ and then turned into integers).")
   (void) dg_sys_info (&load_info,
 		      DG_SYS_INFO_LOAD_INFO_TYPE,
 		      DG_SYS_INFO_LOAD_VERSION_0);
-
+  
   return Fcons (make_number ((int)(load_info.one_minute * 100.0)),
-	Fcons (make_number ((int)(load_info.five_minute * 100.0)),
-	Fcons (make_number ((int)(load_info.fifteen_minute * 100.0)),
- 		   Qnil)));
+         Fcons (make_number ((int)(load_info.five_minute * 100.0)),
+         Fcons (make_number ((int)(load_info.fifteen_minute * 100.0)), 
+         Qnil)));
 #endif /* DGUX */
 }
 
