@@ -6,6 +6,13 @@
 
 #ifndef HAVE_GETPAGESIZE
 
+#ifdef USG5_4
+
+#include <unistd.h>
+#define getpagesize()	sysconf(_SC_PAGESIZE)
+
+#else	/* USG5_4 */
+
 #ifdef VMS
 #include "param.h"
 #else
@@ -25,5 +32,7 @@
 #endif /* no NBPG */
 #endif /* no EXEC_PAGESIZE */
 
+#endif /* USG5_4 */
+
 #endif /* not HAVE_GETPAGESIZE */
 
