@@ -21,13 +21,13 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
  * Epoch 4.0:  must now consider actual character widths, etc. according
  * to styles (which are window-dependent)
  *
- * $Revision: 1.39 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/indent.c,v $
- * $Date: 92/03/18 16:24:22 $
- * $Author: love $
+ * $Revision: 1.42 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/indent.c,v $
+ * $Date: 92/08/05 04:15:54 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/03/18 16:24:22 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/indent.c,v $ $Revision: 1.39 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/05 04:15:54 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/indent.c,v $ $Revision: 1.42 $";
 #endif
 
 #include "config.h"
@@ -114,7 +114,8 @@ current_column (arg)
   int ctl_arrow = (!NULL (current_buffer->ctl_arrow))
     + (EQ (current_buffer->ctl_arrow,Qt));
   int old_pt = point;
-
+  int meta_printable = !NULL (current_buffer->meta_printable);
+  
   if (arg) SET_PT(arg);
 
   if (point == last_known_column_point
@@ -171,13 +172,14 @@ current_column (arg)
 	  post_tab += col;
 	  col = 0;
 	  tab_seen = 1;
-	}
-      else
-	col += (ctl_arrow == 1 && c >= 0177) ? 1 :
-	  (ctl_arrow && c < 0200) ? 2 : 4;
-
-    }
-
+  } else if (is_control_char(c))
+ 	  col += ctl_arrow ? 2 : 4;
+  else if (is_meta_char(c))
+    col += meta_printable ? 1 : 4;
+  else
+ 	col += 4;
+  }
+  
   if (tab_seen)
     {
       col = ((col + tab_width) / tab_width) * tab_width;
@@ -513,6 +515,7 @@ move_to_pixel(goal)
   register int tab_width = XINT(current_buffer->tab_width);
   register int ctl_arrow = (!NULL (current_buffer->ctl_arrow))
     + (EQ (current_buffer->ctl_arrow, Qt));
+  register int meta_printable = !NULL (current_buffer->meta_printable);
   struct X_font *font;
   struct buffer *b = current_buffer;
 
@@ -551,6 +554,7 @@ move_to_pixel(goal)
 	}
       else if (ctl_arrow == 1 && a >= 040)
 	continue;
+	  /*
       else if (ctl_arrow && (a < 040 || a == 0177))
 	{
 	  col++;
@@ -563,6 +567,21 @@ move_to_pixel(goal)
 	  c[0] = '0';
 	  pix += text_width(font,c,1);
 	}
+	  */
+	  else if (is_control_char(a))
+	{
+ 	  col += ctl_arrow ? 1 : 3;
+ 	  c[0] = '0';
+ 	  pix += text_width(font,c,1);
+    } else  if (is_meta_char(a)) {
+	  col += meta_printable ? 0 : 3;
+	  c[0] = '0';
+	  pix += text_width(font,c,1);
+    } else if (a < 040 || a >= 0177) {
+	  col += 3;
+	  c[0] = '0';
+	  pix += text_width(font,c,1);
+    }
     }      
 
   SET_PT(pos);
@@ -604,7 +623,8 @@ move_to_col(goal)
   register int tab_width = XINT (current_buffer->tab_width);
   register int ctl_arrow = (!NULL (current_buffer->ctl_arrow))
     + (EQ (current_buffer->ctl_arrow, Qt));
-
+  register int meta_printable = !NULL (current_buffer->meta_printable);
+  
   if (tab_width <= 0 || tab_width > 20) tab_width = 8;
 
   if (col > goal)
@@ -631,15 +651,14 @@ move_to_col(goal)
 	}
       else if (ctl_arrow == 1 && c >= 040)
 	continue;
-      else if (ctl_arrow && (c < 040 || c == 0177))
-	{
-	  col++;
-	}
-      else if (c < 040 || c >= 0177)
-	{
-	  col += 3;
-	}
-    }
+	  else if (is_control_char(c)) {
+ 		  col += ctl_arrow ? 1 : 3;
+      } else  if (is_meta_char(c)) {
+ 		  col += meta_printable ? 0 : 3;
+ 	  } else if (c < 040 || c >= 0177) {
+ 		  col += 3;
+ 	  }
+  }
 
   SET_PT (pos);
 
@@ -654,6 +673,7 @@ struct position val_compute_motion;
 
 #define CHECK_NEXT(s) {\
   wid = text_width(font,s,1);\
+  if (HPOS(cpos)<0) cpos++; else\
   if ((cwidth + wid) > cend)\
     {\
       if (hscroll || (truncate_partial_width_windows &&\
@@ -719,6 +739,7 @@ compute_motion (from,fromvpos,fromhpos,to,tovpos,tohpos,
   int button_end, button_start;
   int truncate = hscroll || (truncate_partial_width_windows &&
 			     XFASTINT(w->pixwidth) < XWSCREEN(XROOT(w->root)->win)->pixw);
+  int meta_printable = !NULL (current_buffer->meta_printable);
   int selective = XTYPE (current_buffer->selective_display) == Lisp_Int
     ? XINT (current_buffer->selective_display)
       : !NULL (current_buffer->selective_display) ? -1 : 0;
@@ -857,13 +878,36 @@ compute_motion (from,fromvpos,fromhpos,to,tovpos,tohpos,
 	}
       else
 	{
-	  int i;
-	  int inc = (ctl_arrow == 1 && c >= 040) ? 1 :
-	    (ctl_arrow && c < 0200) ? 2 : 4;
+	  if (is_meta_char(c) && meta_printable)
+	    {
+	      /* Insert control character as itself */
+	      a[0] = c;
+	      CHECK_NEXT(a);
+	    }
+	  else if (is_control_char(c) && ctl_arrow)
+	    {
+	      /* Insert control character as 2-char sequence */
+	      a[0] = '^';
+	      CHECK_NEXT(a);
 
-	  a[0] = '0';
-	  for (i = 0; i < inc; i++)
-	    CHECK_NEXT(a);
+	      a[0] = c ^ 0100;
+	      CHECK_NEXT(a);
+	    }
+	  else
+	    {
+	      /* Insert control character as 4-char sequence */
+	      a[0] = '\\';
+	      CHECK_NEXT(a);
+
+	      a[0] = (c>>6) + '0';
+	      CHECK_NEXT(a);
+
+	      a[0] = (7 & (c >>3)) + '0';
+	      CHECK_NEXT(a);
+
+	      a[0] = (7 & c) + '0';
+	      CHECK_NEXT(a);
+	    }
 	}
     }
 
