@@ -18,13 +18,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.9 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/insdel.c,v $
- * $Date: 92/04/14 20:58:12 $
- * $Author: love $
+ * $Revision: 1.10 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/insdel.c,v $
+ * $Date: 92/08/26 23:14:06 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/04/14 20:58:12 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/insdel.c,v $ $Revision: 1.9 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/26 23:14:06 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/insdel.c,v $ $Revision: 1.10 $";
 #endif
 
 #include "config.h"
@@ -538,7 +538,7 @@ prepare_to_modify_buffer (start,end)
       button = raw_read_only_region(start,end,start_b);
 
       if (!NULL(button))
-	Fsignal(Qerror,Fcons(build_string("Read-only button "),
+	Fsignal(Qerror,Fcons(build_string("Zone is read only: "),
 			     Fcons(button,Qnil)));
     }
 #ifdef CLASH_DETECTION
