@@ -18,13 +18,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.17 $
- * $Source: /home/kaplan/love/epoch/src/RCS/keyboard.c,v $
- * $Date: 92/03/27 10:39:27 $
- * $Author: love $
+ * $Revision: 1.22 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/keyboard.c,v $
+ * $Date: 92/08/26 23:14:08 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/03/27 10:39:27 $ $Source: /home/kaplan/love/epoch/src/RCS/keyboard.c,v $ $Revision: 1.17 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/26 23:14:08 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/keyboard.c,v $ $Revision: 1.22 $";
 #endif
 
 /* Allow config.h to undefine symbols found here.  */
@@ -177,6 +177,10 @@ Lisp_Object Vtop_level;
 /* User-supplied string to translate input characters through */
 Lisp_Object Vkeyboard_translate_table;
 
+/* Variable representing whether or not Epoch was compiled with
+   EIGHTBIT_KEYBOARD enabled. */
+Lisp_Object Veightbit_keyboard;
+
 FILE *dribble;			/* File in which we write all commands we read */
 
 /* Nonzero if input is available */
@@ -213,6 +217,13 @@ int interrupts_deferred;
 
 int flow_control;
 
+#ifdef USG5_4
+#define sigfree() {sigset_t tmp; sigemptyset(&tmp); sigprocmask (SIG_SETMASK, &tmp, 0);}
+#define sigholdx(sig) sighold(sig)
+#define sigblockx(sig) sighold (sig)
+#define sigunblockx(sig) sigrelse (sig)
+#define sigpausex(sig) sigpause (sig)
+#else
 #ifndef BSD4_1
 #define sigfree() sigsetmask (SIGEMPTYMASK)
 #define sigholdx(sig) sigsetmask (sigmask (sig))
@@ -220,6 +231,7 @@ int flow_control;
 #define sigunblockx(sig) sigblock (SIGEMPTYMASK)
 #define sigpausex(sig) sigpause (0)
 #endif /* not BSD4_1 */
+#endif /* USG5_4 */
 
 #ifdef BSD4_1
 #define SIGIO SIGTINT
@@ -911,6 +923,7 @@ read_command_char (commandflag)
   Lisp_Object tem;
   jmp_buf save_jump;
   extern request_echo ();
+  int jump_saved_already=0;  
 
 read_command_char_start_over:	/* Epoch:  this is rude & crude */
 
@@ -934,7 +947,11 @@ read_command_char_start_over:	/* Epoch:  this is rude & crude */
     }
 
   /* Save outer setjmp data, in case called recursively.  */
-  bcopy (getcjmp, save_jump, sizeof getcjmp);
+  if (!jump_saved_already)
+  {
+    bcopy (getcjmp, save_jump, sizeof getcjmp);
+    jump_saved_already=1;
+  }
 
   stop_polling ();
 
@@ -1228,12 +1245,20 @@ input is available") ()
       /* if any extra events on the q, send them off now too */
       while (eqlength) epoch_event_dispatch();
 #ifdef SIGIO
+#ifdef USG5_4
+      sighold(SIGIO);
+#else
       mask = sigblock (sigmask (SIGIO));
+#endif
 #endif
       if (kbd_count == 0 && QLength(XD_display))
 	read_avail_input(0);	/* read it */
 #ifdef SIGIO
+#ifdef USG5_4
+      sigrelse(SIGIO);
+#else
       sigsetmask (mask);
+#endif
 #endif
     }
   input_pending = kbd_count > 0 ;
@@ -1290,10 +1315,19 @@ get_input_pending (addr)
     /* It seems there is a timing error such that a SIGIO can be handled here
        and cause kbd_count to become nonzero even though raising of SIGIO
        has already been turned off.  */
+#ifdef USG5_4
+    sigset_t omask;
+    sigprocmask (0, 0, &omask);
+    sighold (SIGIO);
+    if (kbd_count == 0)
+      read_avail_input (*addr);
+    sigprocmask (SIG_SETMASK, &omask, 0);
+#else
     SIGMASKTYPE mask = sigblock (sigmask (SIGIO));
     if (kbd_count == 0)
       read_avail_input (*addr);
     sigsetmask (mask);
+#endif
   }
 #else
   /* If we can't count the input, read it (if any) and see what we got.  */
@@ -1443,11 +1477,13 @@ input_available_signal (signo)
   extern int select_alarmed;
 #endif
 
+#ifndef SUNOS_5
 #ifdef USG
   /* USG systems forget handlers when they are used;
      must reestablish each time */
   signal (signo, input_available_signal);
 #endif /* USG */
+#endif /* SUNOS_5 */
 
 #ifdef BSD4_1
   sigisheld (SIGIO);
@@ -1546,11 +1582,14 @@ read_key_sequence (keybuf, bufsize, prompt, nodisplay)
 
       if (c < 0)
 	return 0;
+
+#ifndef EIGHTBIT_KEYBOARD
       if (c >= 0200)
 	{
 	  nextc = c & 0177;
 	  c = meta_prefix_char;
 	}
+#endif
 
       keybuf[i] = c;
 
@@ -1870,8 +1909,6 @@ Otherwise, suspend normally and after resumption call\n\
 {
   register Lisp_Object tem;
   int count = specpdl_ptr - specpdl;
-  int old_height, old_width;
-  int width, height;
   struct gcpro gcpro1;
   extern init_sys_modes ();
 
@@ -1889,7 +1926,6 @@ Otherwise, suspend normally and after resumption call\n\
       if (!EQ (tem, Qnil)) return Qnil;
     }
 
-  get_screen_size (&old_width, &old_height);
   reset_sys_modes ();
   /* sys_suspend can get an error if it tries to fork a subshell
      and the system resources aren't available for that.  */
@@ -1898,13 +1934,6 @@ Otherwise, suspend normally and after resumption call\n\
   sys_suspend ();
   unbind_to (count);
 
-  /* Check if terminal/window size has changed.
-     Note that this is not useful when we are running directly
-     with a window system; but suspend should be disabled in that case.  */
-  get_screen_size (&width, &height);
-  if (width != old_width || height != old_height)
-    change_screen_size (height, width, 0,cur_root);
-
   /* Call value of suspend-resume-hook
      if it is bound and value is non-nil.  */
   tem = intern ("suspend-resume-hook");
@@ -2087,7 +2116,7 @@ quit_throw_to_read_command_char ()
   input_pending = 0;
   unread_command_char = -1;
 #ifdef POLL_FOR_INPUT
-  if (poll_suppress_count != 1)
+  if (poll_suppress_count == 0)
     abort ();
 #endif
   _longjmp (getcjmp, 1);
@@ -2152,7 +2181,11 @@ init_keyboard ()
 #endif /* HAVE_TERMIO */
 /* Note SIGIO has been undef'd if FIONREAD is missing.  */
 #ifdef SIGIO
+#ifdef SUNOS_5
+      sigset (SIGIO, input_available_signal);
+#else /* not SUNOS_5 */
       signal (SIGIO, input_available_signal);
+#endif /* not SUNOS_5 */
 #endif /* SIGIO */
     }
 
@@ -2279,6 +2312,15 @@ Useful to set before you dump a modified Emacs.");
 Each character is looked up in this string and the contents used instead.\n\
 If string is of length N, character codes N and up are untranslated."); 
   Vkeyboard_translate_table = Qnil;
+
+  DEFVAR_LISP ("eightbit-keyboard", &Veightbit_keyboard,
+    "Boolean representing whether or not Epoch was compiled with\n\
+EIGHTBIT_KEYBOARD enabled.");
+#ifdef EIGHTBIT_KEYBOARD
+  Veightbit_keyboard = Qt;
+#else /* not EIGHTBIT_KEYBOARD */
+  Veightbit_keyboard = Qnil;
+#endif
 }
 
 keys_of_keyboard ()
