@@ -18,13 +18,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.5 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/keymap.c,v $
- * $Date: 92/02/23 11:01:25 $
- * $Author: love $
+ * $Revision: 1.7 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/keymap.c,v $
+ * $Date: 92/08/26 23:14:11 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:01:25 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/keymap.c,v $ $Revision: 1.5 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/26 23:14:11 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/keymap.c,v $ $Revision: 1.7 $";
 #endif
 
 #include "config.h"
@@ -64,20 +64,22 @@ Lisp_Object Vminibuffer_local_must_match_map;
 
 Lisp_Object Qkeymapp, Qkeymap;
 
+#ifndef EIGHTBIT_KEYBOARD
 /* A char over 0200 in a key sequence
    is equivalent to prefixing with this character.  */
 
 extern int meta_prefix_char;
+#endif /* not EIGHTBIT_KEYBOARD */
 
 static void insert_first_line();
 
 DEFUN ("make-keymap", Fmake_keymap, Smake_keymap, 0, 0, 0,
-  "Construct and return a new keymap, a vector of length 128.\n\
+  "Construct and return a new keymap, a vector of length 256.\n\
 All entries in it are nil, meaning \"command undefined\".")
   ()
 {
   register Lisp_Object val;
-  XFASTINT (val) = 0200;
+  XFASTINT (val) = 0400;
   return Fmake_vector (val, Qnil);
 }
 
@@ -121,7 +123,7 @@ synkey (frommap, fromchar, tomap, tochar)
 
 DEFUN ("keymapp", Fkeymapp, Skeymapp, 1, 1, 0,
   "Return t if ARG is a keymap.\n\
-A keymap is a vector of length 128, or a list (keymap . ALIST),\n\
+A keymap is a vector of length 256, or a list (keymap . ALIST),\n\
 where alist elements look like (CHAR . DEFN).\n\
 A symbol whose function definition is a keymap is itself a keymap.")
   (object)
@@ -137,7 +139,7 @@ A symbol whose function definition is a keymap is itself a keymap.")
       QUIT;
     }
 
-  if ((XTYPE (tem) == Lisp_Vector && XVECTOR (tem)->size == 0200)
+  if ((XTYPE (tem) == Lisp_Vector && XVECTOR (tem)->size == 0400)
       || (CONSP (tem) && EQ (XCONS (tem)->car, Qkeymap)))
     return Qt;
   return Qnil;
@@ -158,7 +160,7 @@ get_keymap_1 (object, error)
 	  tem = XSYMBOL (tem)->function;
 	  QUIT;
 	}
-      if ((XTYPE (tem) == Lisp_Vector && XVECTOR (tem)->size == 0200)
+      if ((XTYPE (tem) == Lisp_Vector && XVECTOR (tem)->size == 0400)
 	  || (CONSP (tem) && EQ (XCONS (tem)->car, Qkeymap)))
 	return tem;
       if (error)
@@ -200,8 +202,8 @@ access_keymap (map, idx)
      register int idx;
 {
   register Lisp_Object val;
-  if (idx < 0 || idx >= 0200)
-    error ("Command key out of range 0-127");
+  if (idx < 0 || idx >= 0400)
+    error ("Command key out of range 0-255");
 
   /* Get definition for character `idx' proper.  */
   if (CONSP (map))
@@ -220,8 +222,8 @@ store_in_keymap (keymap, idx, def)
 {
   register Lisp_Object tem;
 
-  if (idx < 0 || idx >= 0200)
-    error ("Command key out of range 0-127");
+  if (idx < 0 || idx >= 0400)
+    error ("Command key out of range 0-255");
 
   if (CONSP (keymap))
     {
@@ -298,7 +300,9 @@ definition, and may be any of the above (including another symbol).")
   register int c;
   register Lisp_Object tem;
   register Lisp_Object cmd;
+#ifndef EIGHTBIT_KEYBOARD
   int metized = 0;
+#endif /* not EIGHTBIT_KEYBOARD */
 
   keymap = get_keymap (keymap);
 
@@ -310,6 +314,9 @@ definition, and may be any of the above (including another symbol).")
   while (1)
     {
       c = XSTRING (key)->data[idx];
+#ifdef EIGHTBIT_KEYBOARD
+      idx++;
+#else /* not EIGHTBIT_KEYBOARD */
       if (c >= 0200 && !metized)
 	{
 	  c = meta_prefix_char;
@@ -321,6 +328,7 @@ definition, and may be any of the above (including another symbol).")
 	  metized = 0;
 	  idx++;
 	}
+#endif /* not EIGHTBIT_KEYBOARD */
 
       if (idx == XSTRING (key)->size)
 	return store_in_keymap (keymap, c, def);
@@ -358,7 +366,9 @@ it takes to reach a non-prefix command.")
   register Lisp_Object tem;
   register Lisp_Object cmd;
   register int c;
+#ifndef EIGHTBIT_KEYBOARD
   int metized = 0;
+#endif
 
   keymap = get_keymap (keymap);
 
@@ -370,17 +380,21 @@ it takes to reach a non-prefix command.")
   while (1)
     {
       c = XSTRING (key)->data[idx];
+#ifdef EIGHTBIT_KEYBOARD
+      idx++;
+#else /* not EIGHTBIT_KEYBOARD */
       if (c >= 0200 && !metized)
-	{
-	  c = meta_prefix_char;
-	  metized = 1;
-	}
+        {
+          c = meta_prefix_char;
+          metized = 1;
+        }
       else
-	{
-	  c &= 0177;
-	  metized = 0;
-	  idx++;
-	}
+        {
+          c &= 0177;
+          metized = 0;
+          idx++;
+        }
+#endif /* not EIGHTBIT_KEYBOARD */
 
       cmd = get_keyelt (access_keymap (keymap, c));
       if (idx == XSTRING (key)->size)
@@ -586,7 +600,7 @@ so that the KEYS increase in length.  The first element is (\"\" . KEYMAP).")
     {
       thisseq = Fcar (Fcar (tail));
       thismap = Fcdr (Fcar (tail));
-      for (i = 0; i < 0200; i++)
+      for (i = 0; i < 0400; i++)
 	{
 	  cmd = get_keyelt (access_keymap (thismap, i));
 	  if (NULL (cmd)) continue;
@@ -626,14 +640,16 @@ push_key_description (c, p)
      register unsigned int c;
      register char *p;
 {
-  if (c >= 0200)
-    {
-      *p++ = 'M';
-      *p++ = '-';
-      c -= 0200;
-    }
   if (c < 040)
     {
+#ifndef EIGHTBIT_KEYBOARD
+      if (c >= 0200)
+        {
+          *p++ = 'M';
+          *p++ = '-';
+          c -= 0200;
+        }
+#endif /* not EIGHTBIT_KEYBOARD */
       if (c == 033)
 	{
 	  *p++ = 'E';
@@ -707,12 +723,14 @@ push_text_char_description (c, p)
      register unsigned int c;
      register char *p;
 {
+#ifndef EIGHTBIT_KEYBOARD
   if (c >= 0200)
     {
       *p++ = 'M';
       *p++ = '-';
       c -= 0200;
     }
+#endif /* not EIGHTBIT_KEYBOARD */
   if (c < 040)
     {
       *p++ = '^';
@@ -779,7 +797,7 @@ sequence found, rather than a list of all possible key sequences.")
       /* If the MAP is a vector, I increments and eventually reaches 0200.
 	 Otherwise I remains 0; MAP is cdr'd and eventually becomes nil.  */
 
-      while (!NULL (map) && i < 0200)
+      while (!NULL (map) && i < 0400)
 	{
 	  register Lisp_Object elt, dummy;
 
