@@ -18,10 +18,10 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.6 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/lisp.h,v $
- * $Date: 92/02/23 11:03:40 $
- * $Author: love $
+ * $Revision: 1.7 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/lisp.h,v $
+ * $Date: 92/07/11 21:32:01 $
+ * $Author: marca $
  */
 
 /* Define the fundamental Lisp data structures */
@@ -262,7 +262,12 @@ Lisp_Object;
 #define VALMASK ((1<<VALBITS) - 1)
 #endif
 #define GCTYPEMASK ((1<<GCTYPEBITS) - 1)
+#ifdef __STDC__
+/* Need to make sure this deals with unsigneds */
+#define MARKBIT ((1U) << (VALBITS + GCTYPEBITS))
+#else
 #define MARKBIT (1 << (VALBITS + GCTYPEBITS))
+#endif
 
 #endif /* NO_UNION_TYPE */
 
