@@ -1,9 +1,9 @@
 
 /*
- * $Header: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/malloc.c,v 1.3 91/09/28 12:27:52 love Exp $
+ * $Header: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/malloc.c,v 1.5 92/02/23 11:01:21 love Exp $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/09/28 12:27:52 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/malloc.c,v $ $Revision: 1.3 $";
+static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:01:21 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/malloc.c,v $ $Revision: 1.5 $";
 #endif
 /* dynamic memory allocation for GNU.
    Copyright (C) 1985, 1987 Free Software Foundation, Inc.
@@ -222,6 +222,8 @@ char *_malloc_base;
 
 static void getpool ();
 
+char *malloc ();
+
 /* Cause reinitialization based on job parameters;
   also declare where the end of pure storage is. */
 void
@@ -262,7 +264,7 @@ morecore (nu)			/* ask system for more memory */
 
 #ifdef BSD
 #ifndef BSD4_1
-  SIGMASKTYPE newmask = -1;
+  SIGMASKTYPE newmask = SIGFULLMASK;
   /* Blocking these signals interferes with debugging, at least on BSD on
      the HP 9000/300.  */
 #ifdef SIGTRAP
@@ -431,6 +433,10 @@ malloc (n)		/* get a block */
      multiple of 8, then figure out which nestf[] area to use.
      Both the beginning of the header and the beginning of the
      block should be on an eight byte boundary.  */
+#ifdef SUNOS_LOCALTIME_BUG
+  if (n < 16)
+    n = 16;
+#endif
   nbytes = (n + ((sizeof *p + 7) & ~7) + EXTRA + 7) & ~7;
   {
     register unsigned int   shiftr = (nbytes - 1) >> 2;
@@ -621,6 +627,13 @@ calloc (num, size)
     bzero (mem, num);
   return mem;
 }
+/* This is in case something linked with Emacs calls cfree.  */
+
+cfree (mem)
+     char *mem;
+{
+  return free (mem);
+}
 
 #ifndef VMS
 
@@ -816,7 +829,7 @@ get_lim_data ()
 #undef malloc
 int vms_out_initial = 0;
 char vms_initial_buffer[VMS_ALLOCATION_SIZE];
-static char *vms_current_brk = &vms_initial_buffer;
+static char *vms_current_brk = vms_initial_buffer;
 static char *vms_end_brk = &vms_initial_buffer[VMS_ALLOCATION_SIZE-1];
 
 #include <stdio.h>
@@ -830,7 +843,7 @@ sys_sbrk (incr)
   if (vms_out_initial)
     {
       /* out of initial allocation... */
-      if (!(temp = malloc (incr)))
+      if (!(temp = (char *) malloc (incr)))
 	temp = (char *) -1;
     }
   else
@@ -845,7 +858,7 @@ sys_sbrk (incr)
       else
 	{
 	  vms_out_initial = 1;	/* mark as out of initial allocation */
-	  if (!(temp = malloc (incr)))
+	  if (!(temp = (char *) malloc (incr)))
 	    temp = (char *) -1;
 	}
     }
