@@ -23,13 +23,13 @@ and this notice must be preserved on all copies.  */
  * Epoch 4.0:  Mouse & motion event data now propogates to elisp level
  *	       as pixel values.
  *
- * $Revision: 1.13 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/mouse.c,v $
- * $Date: 92/03/18 16:24:31 $
- * $Author: love $
+ * $Revision: 1.15 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/mouse.c,v $
+ * $Date: 92/08/05 04:15:56 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/03/18 16:24:31 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/mouse.c,v $ $Revision: 1.13 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/05 04:15:56 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/mouse.c,v $ $Revision: 1.15 $";
 #endif
 
 #include <stdio.h>
@@ -154,6 +154,7 @@ DEFUN ("epoch::coords-to-point",Fepoch_coords_to_point,Sepoch_coords_to_point,
 
   win = Fepoch_first_window(screen);
 
+#if 0 /* This is reproduced immediately following. */
   while (!NIL(win))
     {
       w = XWINDOW(win);
@@ -170,6 +171,46 @@ DEFUN ("epoch::coords-to-point",Fepoch_coords_to_point,Sepoch_coords_to_point,
       if (distinct_minibuffer &&
 	  EQ(win,XROOT(screen)->minibuf_window)) win = Qnil;
     }
+#endif
+
+  /* This is ugly, but I don't quite know how to fix it yet...
+     -marca */
+  {
+#define SANITY_THRESHOLD 1000
+    static int sanity_check;
+    sanity_check = 0;
+
+    while (!NIL(win) && (sanity_check < SANITY_THRESHOLD))
+      {
+        int delta;
+
+        w = XWINDOW(win);
+
+        delta=(w->pixleft==ws->in_border) ? 0 :                      
+          XFWIDTH(XXFONT(XSTYLE(XROOT(w->root)->stylenorm)->font));  
+        
+        if (!NIL(w->buffer) &&
+            x >= w->pixleft+delta && x <= (w->pixleft + w->pixwidth) 
+            && (y >= w->pixtop && y <= (w->pixtop + w->pixheight ) ||
+                (y < ws->in_border && w->pixtop == ws->in_border)))
+          break;
+        /* Looks stupid, but the problem is that we have to allow the minibuf
+         * as a valid window the _first_ time through here, but not else when,
+         * and we can't go on if the first window was the minibuffer
+         */
+        if (!EQ(win,XROOT(screen)->minibuf_window)) win = Fnext_window(win,Qt);
+        if (distinct_minibuffer &&
+            EQ(win,XROOT(screen)->minibuf_window)) win = Qnil;
+        sanity_check++;
+      }
+    /* If we're punting on a sanity check, return a nil window and let
+       somebody else worry about it. */
+    if (sanity_check >= SANITY_THRESHOLD)
+      {
+        win = Qnil;
+        /* fprintf (stderr, "[epoch::coords-to-point] Sanity check.\n"); */
+      }
+  }
 
   if (NIL(win)) return Qnil;
 
@@ -213,6 +254,8 @@ DEFUN ("epoch::coords-to-point",Fepoch_coords_to_point,Sepoch_coords_to_point,
 		break;
 	      cb = cb->next; cx++;
 	    }
+	  if (XFASTINT(w->pixleft) != ws->in_border)       
+	    cx--;                                          
 	}
       
       startp = marker_position (w->start);
