@@ -18,15 +18,17 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.4 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/print.c,v $
- * $Date: 91/09/08 22:17:06 $
+ * $Revision: 1.9 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/print.c,v $
+ * $Date: 92/03/22 15:45:32 $
  * $Author: love $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/09/08 22:17:06 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/print.c,v $ $Revision: 1.4 $";
+static char rcsid[] = "$Author: love $ $Date: 92/03/22 15:45:32 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/print.c,v $ $Revision: 1.9 $";
 #endif
 
+#include <signal.h>
+
 #include "config.h"
 #include <stdio.h>
 #undef NULL
@@ -41,6 +43,7 @@ static char rcsid[] = "$Author: love $ $Date: 91/09/08 22:17:06 $ $Source: /impo
 #include "termchar.h"
 #include "screen.h"		/* Epoch */
 #include "button.h"
+/* X11 includes used; use NIL rather than NULL from lisp.h */
 #endif /* not standalone */
 
 Lisp_Object Vstandard_output, Qstandard_output;
@@ -62,6 +65,7 @@ int print_escape_newlines;
 
 extern char message_buf[300];
 extern int noninteractive_need_newline;
+extern int message_buf_print;
 #ifdef MAX_PRINT_CHARS
 static int print_chars;
 static int max_print;
@@ -82,7 +86,7 @@ static int max_print;
 
 #define PRINTPREPARE \
    original = printcharfun; \
-   if (NULL (printcharfun)) printcharfun = Qt; \
+   if (NIL (printcharfun)) printcharfun = Qt; \
    if (XTYPE (printcharfun) == Lisp_Buffer) \
      { if (XBUFFER (printcharfun) != current_buffer) Fset_buffer (printcharfun); \
        printcharfun = Qnil;}\
@@ -133,8 +137,12 @@ printchar (ch, fun)
 	  noninteractive_need_newline = 1;
 	  return;
 	}
-      if (echo_area_contents != message_buf)
-	echo_area_contents = message_buf, printbufidx = 0;
+      if (echo_area_contents != message_buf || !message_buf_print)
+	{
+	  echo_area_contents = message_buf, printbufidx = 0;
+	  printbufidx = 0;
+	  message_buf_print = 1;
+	}
       if (printbufidx < sizeof message_buf)
 	message_buf[printbufidx++] = ch;
       message_buf[printbufidx] = 0;
@@ -176,10 +184,14 @@ strout (ptr, size, printcharfun)
 	  noninteractive_need_newline = 1;
 	  return;
 	}
-      if (echo_area_contents != message_buf)
-	echo_area_contents = message_buf, printbufidx = 0;
-      if (i > sizeof message_buf - printbufidx - 1)
-	i = sizeof message_buf - printbufidx - 1;
+      if (echo_area_contents != message_buf || !message_buf_print)
+	{
+	  echo_area_contents = message_buf;
+	  printbufidx = 0;
+	  message_buf_print = 1;
+	}
+      if (i > sizeof (message_buf) - printbufidx)
+	i = sizeof (message_buf) - printbufidx;
       bcopy (ptr, &message_buf[printbufidx], i);
       printbufidx += i;
       message_buf[printbufidx] = 0;
@@ -204,7 +216,7 @@ STREAM defaults to the value of `standard-output' (which see).")
   int start_point;
   Lisp_Object original;
 
-  if (NULL (printcharfun))
+  if (NIL (printcharfun))
     printcharfun = Vstandard_output;
   CHECK_NUMBER (ch, 0);
   PRINTPREPARE;
@@ -332,7 +344,7 @@ DEFUN ("terpri", Fterpri, Sterpri, 0, 1, 0,
   int start_point;
   Lisp_Object original;
 
-  if (NULL (printcharfun))
+  if (NIL (printcharfun))
     printcharfun = Vstandard_output;
   PRINTPREPARE;
   PRINTCHAR ('\n');
@@ -356,7 +368,7 @@ Output stream is STREAM, or value of `standard-output' (which see).")
 #ifdef MAX_PRINT_CHARS
   max_print = 0;
 #endif /* MAX_PRINT_CHARS */
-  if (NULL (printcharfun))
+  if (NIL (printcharfun))
     printcharfun = Vstandard_output;
   PRINTPREPARE;
   print_depth = 0;
@@ -409,7 +421,7 @@ Output stream is STREAM, or value of standard-output (which see).")
   int start_point;
   Lisp_Object original;
 
-  if (NULL (printcharfun))
+  if (NIL (printcharfun))
     printcharfun = Vstandard_output;
   PRINTPREPARE;
   print_depth = 0;
@@ -436,7 +448,7 @@ Output stream is STREAM, or value of `standard-output' (which see).")
   print_chars = 0;
   max_print = MAX_PRINT_CHARS;
 #endif /* MAX_PRINT_CHARS */
-  if (NULL (printcharfun))
+  if (NIL (printcharfun))
     printcharfun = Vstandard_output;
   GCPRO1 (obj);
   PRINTPREPARE;
@@ -584,7 +596,7 @@ print (obj, printcharfun, escapeflag)
 	    obj = Fcdr (obj);
 	  }
       }
-      if (!NULL (obj) && !CONSP (obj))
+      if (!NIL (obj) && !CONSP (obj))
 	{
 	  strout (" . ", 3, printcharfun);
 	  print (obj, printcharfun, escapeflag);
@@ -609,7 +621,7 @@ print (obj, printcharfun, escapeflag)
 
 #ifndef standalone
     case Lisp_Buffer:
-      if (NULL (XBUFFER (obj)->name))
+      if (NIL (XBUFFER (obj)->name))
 	strout ("#<killed buffer>", -1, printcharfun);
       else if (escapeflag)
 	{
@@ -636,7 +648,7 @@ print (obj, printcharfun, escapeflag)
       strout ("#<window ", -1, printcharfun);
       sprintf (buf, "%d", XFASTINT (XWINDOW (obj)->sequence_number));
       strout (buf, -1, printcharfun);
-      if (!NULL (XWINDOW (obj)->buffer))
+      if (!NIL (XWINDOW (obj)->buffer))
 	{
 	  unsigned char *p = XSTRING (XBUFFER (XWINDOW (obj)->buffer)->name)->data;
 	  strout (" on ", -1, printcharfun);
@@ -688,7 +700,7 @@ print (obj, printcharfun, escapeflag)
       PRINTCHAR('>');
       break;
     case Lisp_Button:
-      strout ("#<button ",-1, printcharfun);
+      strout ("#<zone ",-1, printcharfun);
       if (!(XMARKER(XBUTTON(obj)->start)->buffer))
 	strout ("in no buffer", -1, printcharfun);
       else
