@@ -18,13 +18,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.21 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/process.c,v $
- * $Date: 92/03/22 15:15:56 $
- * $Author: love $
+ * $Revision: 1.24 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/process.c,v $
+ * $Date: 92/07/11 21:32:03 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/03/22 15:15:56 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/process.c,v $ $Revision: 1.21 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/07/11 21:32:03 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/process.c,v $ $Revision: 1.24 $";
 #endif
 
 #include <signal.h>
@@ -94,6 +94,10 @@ static dummy () {}
 #endif /* AIX */
 
 #ifdef SYSV_PTYS
+#ifdef titan
+#include <sys/ttyhw.h>
+#include <sys/stream.h>
+#endif /* titan's pty needs ttyhw & stream */
 #include <sys/tty.h>
 #include <sys/pty.h>
 #endif
@@ -198,6 +202,9 @@ extern errno;
 extern sys_nerr;
 extern char *sys_errlist[];
 
+#ifdef SYS_SIGLIST
+#define sys_siglist SYS_SIGLIST
+#else
 #ifndef BSD4_1
 extern char *sys_siglist[];
 #else
@@ -231,6 +238,7 @@ char *sys_siglist[] =
     "exceeded file size limit"
     };
 #endif
+#endif /* SYS_SIGLIST */
 
 #ifdef vipc
 
@@ -1261,7 +1269,9 @@ create_process (process, new_argv)
 	  }
 #endif /* not UNIPLUS and not RTU */
 #ifdef SETUP_SLAVE_PTY
-	SETUP_SLAVE_PTY;
+	if (pty_flag) {
+	  SETUP_SLAVE_PTY;
+	}
 #endif /* SETUP_SLAVE_PTY */
 #ifdef AIX
 	/* On AIX, we've disabled SIGHUP above once we start a child on a pty.
@@ -1819,9 +1829,11 @@ wait_reading_process_input (time_limit, read_kbd, do_display,selection)
 	    error("select error: %s", sys_errlist[xerrno]);
 	}
 #if defined (sun) || defined (APOLLO)
+#ifndef USG5_4
       else if (nfds > 0 && FD_ISSET (0, &Available) && interrupt_input)
 	/* System sometimes fails to deliver SIGIO.  */
 	kill (getpid (), SIGIO);
+#endif
 #endif
 
       /* Check for keyboard input */
@@ -2297,7 +2309,11 @@ process_send_signal (process, signo, current_group, nomsg)
   /* gid may be a pid, or minus a pgrp's number */
 #ifdef TIOCSIGSEND
   if (!NIL (current_group))
+#ifdef TIOCSIGNAL	/* SIGSEND is really SIGNAL */
+    ioctl (XFASTINT (p->infd), TIOCSIGNAL, &signo);
+#else
     ioctl (XFASTINT (p->infd), TIOCSIGSEND, signo);
+#endif /* TIOCSIGNAL */
   else
     {
       gid = - XFASTINT (p->pid);
