@@ -2,13 +2,13 @@
  * Epoch 4.0 : X-window property handling
  */
 /*
- * $Revision: 1.2 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/property.c,v $
- * $Date: 91/04/12 11:34:45 $
+ * $Revision: 1.5 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/property.c,v $
+ * $Date: 92/03/27 14:53:43 $
  * $Author: love $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/04/12 11:34:45 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/property.c,v $ $Revision: 1.2 $";
+static char rcsid[] = "$Author: love $ $Date: 92/03/27 14:53:43 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/property.c,v $ $Revision: 1.5 $";
 #endif
 
 
@@ -18,17 +18,21 @@ static char rcsid[] = "$Author: love $ $Date: 91/04/12 11:34:45 $ $Source: /impo
 #ifndef makedev
 #include <sys/types.h>
 #endif
+#ifdef __convex__
+extern char *memchr();
+#else
 #include <memory.h>
+#endif /* Convex */
 
 #include "config.h"
 #include "lisp.h"
 #include "x11term.h"
-
 #include "screen.h"
 #include "screenX.h"
-
 #include "xresource.h"
 
+/* X11 includes used; use NIL rather than NULL from lisp.h */
+
 extern int interrupt_input;
 
 extern Display *XD_display;
@@ -179,9 +183,9 @@ FormatCardinalProperty(buff,count,format)
 {
   switch (format)
     {
-    case 8 : return FormatCardinal8Property((char *)buff,count);
-    case 16 : return FormatCardinal16Property((int16 *)buff,count);
-    case 32 : return FormatCardinal32Property((long *)buff,count);
+    case 8 : return FormatCardinal8Property((unsigned char *)buff,count);
+    case 16 : return FormatCardinal16Property((uint16 *)buff,count);
+    case 32 : return FormatCardinal32Property((unsigned long *)buff,count);
     default : return Qnil;
     }
 }
@@ -253,7 +257,7 @@ raw_get_property(dpy,win,prop)
   Lisp_Object value = Qnil;
   Atom actual_type;
   int actual_format;
-  char *buffer;
+  unsigned char *buffer;
   unsigned long count, remaining;
   int zret;
   BLOCK_INPUT_DECLARE();
@@ -310,7 +314,7 @@ Returns the value of the property, or nil if the property couldn't be retrieved.
   else
     {
       block = find_block(seq);
-      if (NULL(block)) return Qnil;
+      if (NIL(block)) return Qnil;
       dpy = XXSCREEN(XROOT(block)->x11)->display;
       win = XXSCREEN(XROOT(block)->x11)->xid;
     }
@@ -376,7 +380,7 @@ VerifyListHasConsistentType (list) Lisp_Object list;
   if (XRESOURCEP(temp)) rtype = XXRESOURCE(temp)->type;
   list = Fcdr(list);
 
-  for ( ; !NULL(list) ; list = Fcdr(list))
+  for ( ; !NIL(list) ; list = Fcdr(list))
     {
       QUIT;
       temp = Fcar(list);
@@ -578,8 +582,6 @@ raw_set_property(dpy,win,prop,value)
   unsigned long count;		/* Number of data items */
   VOID * addr;			/* address of data item array */
   int zret;			/* X call return value */
-  long tmp;
-  XID tmpxid;
   int free_storage;		/* set if addr points at non-malloc'd store */
   BLOCK_INPUT_DECLARE();
 
@@ -619,7 +621,7 @@ the current screen.") (name,value,seq) Lisp_Object name,value,seq;
   else
     {
       block = find_block(seq);
-      if (NULL(block)) return Qnil;
+      if (NIL(block)) return Qnil;
       dpy = XXSCREEN(XROOT(block)->x11)->display;
       win = XXSCREEN(XROOT(block)->x11)->xid;
     }
