@@ -1,9 +1,9 @@
 
 /*
- * $Header: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/regex.c,v 1.1 91/02/27 15:44:12 love Exp $
+ * $Header: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/regex.c,v 1.3 92/07/12 03:28:48 marca Exp $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/02/27 15:44:12 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/regex.c,v $ $Revision: 1.1 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/07/12 03:28:48 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/regex.c,v $ $Revision: 1.3 $";
 #endif
 /* Extended regular expression matching and search.
    Copyright (C) 1985 Free Software Foundation, Inc.
@@ -1200,7 +1200,8 @@ re_match_2 (pbufp, string1, size1, string2, size2, pos, regs, mstop)
 	    /* Don't allow matching a register that hasn't been used.
 	       This isn't fully reliable in the current version,
 	       but it is better than crashing.  */
-	    if ((int) regend[regno] <= -1)
+	    /* Convex fix:  only check for equal to -1 */
+	    if ((int) regend[regno] == -1)
 	      goto fail;
 
 	    d2 = regstart[regno];
@@ -1617,11 +1618,11 @@ static char upcase[0400] =
     0300, 0301, 0302, 0303, 0304, 0305, 0306, 0307,
     0310, 0311, 0312, 0313, 0314, 0315, 0316, 0317,
     0320, 0321, 0322, 0323, 0324, 0325, 0326, 0327,
-    0330, 0331, 0332, 0333, 0334, 0335, 0336, 0337,
-    0340, 0341, 0342, 0343, 0344, 0345, 0346, 0347,
-    0350, 0351, 0352, 0353, 0354, 0355, 0356, 0357,
-    0360, 0361, 0362, 0363, 0364, 0365, 0366, 0367,
-    0370, 0371, 0372, 0373, 0374, 0375, 0376, 0377
+    0300, 0301, 0302, 0303, 0304, 0305, 0306, 0307,
+    0310, 0311, 0312, 0313, 0314, 0315, 0316, 0317,
+    0320, 0321, 0322, 0323, 0324, 0325, 0326, 0327,
+    0330, 0331, 0332, 0333, 0334, 0335, 0336, 0367,
+    0340, 0341, 0342, 0343, 0344, 0345, 0346, 0377
   };
 
 main (argc, argv)
