@@ -23,13 +23,13 @@ file named COPYING.  Among other things, the copyright notice
 and this notice must be preserved on all copies.
 */
 /*
- * $Revision: 1.46 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/screen.c,v $
- * $Date: 92/03/23 09:39:18 $
- * $Author: love $
+ * $Revision: 1.56 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/screen.c,v $
+ * $Date: 92/08/26 23:14:12 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/03/23 09:39:18 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/screen.c,v $ $Revision: 1.46 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/26 23:14:12 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/screen.c,v $ $Revision: 1.56 $";
 #endif
 
 #include <stdio.h>
@@ -68,6 +68,11 @@ Atom XA_screen_id;			/* root block sequence # */
 Atom XA_wm_change_state;	/* change state atom for iconification */
 Atom XA_resource_manager;	/* Resource manager property atom */
 Atom XA_targets;		/* request for list of valid conversions */
+Atom XA_text;			/* request type "TEXT" */
+Atom XA_length;			/* request type "LENGTH" */
+Atom XA_list_length;		/* request type "LIST LENGTH" */
+Atom XA_clipboard;		/* CLIPBOARD selection */
+
 
 /*
  * Default values - set up Lisp Objects later to point into these
@@ -267,10 +272,6 @@ x_set_hints(rb,xd,ugmask,pgmask)
   
   /* set sizing hints */
 #ifdef PBaseSize
-#if 0  
-  size_hints.base_width = xs->pixwidth; /* 2*ws->in_border; */
-  size_hints.base_height = xs->pixheight; /*  2*ws->in_border; */
-#endif  
   size_hints.base_width = 2*ws->in_border;
   size_hints.base_height = 2*ws->in_border;
 #endif
@@ -296,6 +297,7 @@ x_set_hints(rb,xd,ugmask,pgmask)
 #else
   size_hints.flags = PMinSize | PResizeInc;
 #endif
+
   size_hints.flags |= 
     (ugmask & (XValue|YValue)) ? USPosition : 
       (pgmask & (XValue|YValue)) ? PPosition : 
@@ -304,10 +306,27 @@ x_set_hints(rb,xd,ugmask,pgmask)
     (ugmask & (WidthValue|HeightValue)) ? USSize : 
       (pgmask & (XValue|YValue)) ? PSize :
 	0;
-#if 0  
-  size_hints.flags |= (gmask & (XValue|YValue)) ? USPosition : PPosition;
-  size_hints.flags |= (gmask & (WidthValue|HeightValue)) ? USSize : PSize;
-#endif  
+
+  if (size_hints.flags & USPosition)
+    {
+      /* If user-specified size, also set gravity */
+      switch (ugmask & (XNegative | YNegative))
+	{
+	case 0:
+	  size_hints.win_gravity = NorthWestGravity;
+	  break;
+	case XNegative:
+	  size_hints.win_gravity = NorthEastGravity;
+	  break;
+	case YNegative:
+	  size_hints.win_gravity = SouthWestGravity;
+	  break;
+	default:
+	  size_hints.win_gravity = SouthEastGravity;
+	  break;
+	}
+      size_hints.flags |= PWinGravity;
+    }
 #ifdef PBaseSize
   HOLD_INPUT(XSetWMNormalHints(xs->display,xs->xid,&size_hints));
 #else
@@ -330,8 +349,8 @@ x_create_screen(root,xdbase,alist)
      Lisp_Object alist;
 {
   unsigned int ugmask,pgmask;	/* geometry parse return user/program */
-  int x,y;
-  unsigned int dx,dy;		/* base and size */
+  int x = 0,y = 0;
+  unsigned int dx = 0,dy = 0;	/* base and size */
   unsigned long xcursor_color;	/* X cursor color */
   struct X_Screen *xs = XXSCREEN(root->x11);
   struct W_Screen *ws = XWSCREEN(root->win);
@@ -390,10 +409,6 @@ x_create_screen(root,xdbase,alist)
   /* XGeometry is not used because it assumes the specs are in pixels,
    * not in characters.
    */
-#if 0  
-  gmask = XParseGeometry(xd.default_geometry,&x,&y,&dx,&dy);
-  gmask = XParseGeometry(xd.requested_geometry,&x,&y,&dx,&dy);
-#endif
   pgmask = XParseGeometry(xd.default_geometry,&x,&y,&dx,&dy);
   ugmask = XParseGeometry(xd.requested_geometry,&x,&y,&dx,&dy);
   
@@ -645,6 +660,7 @@ x_init_roots()
       w->root = rootblock;
       cur_root->ewin = XWINDOW(minibuf_window)->prev;
       cur_root->select = cur_root->ewin;
+      cur_root->prev_select = cur_root->ewin;
       cur_root->minibuf_window = minibuf_window;      
       w = XWINDOW(minibuf_window);
       w->root = rootblock;
@@ -721,6 +737,10 @@ x_init_display()
   XA_wm_change_state = XInternAtom(XD_display,"WM_CHANGE_STATE",False);
   XA_resource_manager = XInternAtom(XD_display,"RESOURCE_MANAGER",False);
   XA_targets = XInternAtom(XD_display,"TARGETS",False);
+  XA_text = XInternAtom(XD_display,"TEXT",False);
+  XA_length = XInternAtom(XD_display,"LENGTH",False);
+  XA_list_length = XInternAtom(XD_display,"LIST_LENGTH",False);
+  XA_clipboard = XInternAtom(XD_display,"CLIPBOARD",False);
 
   /* set up the default values, including minibuffer mode -> local or not */
   x_init_defaults();
@@ -1003,8 +1023,8 @@ Defaults to current screen; optional fourth argument specifies a screen.")
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::create-screen",Fcreate_screen,Screate_screen,0,2,0,
-      "Create a new X11 screen using association list ALIST.\
-Optional argument is buffer to put in the screen.\
+      "Create a new X11 screen using association list ALIST.\n\
+Optional argument is buffer to put in the screen.\n\
 If non-existent or nil, a new one is created.")
 	(buff,alist) Lisp_Object buff,alist;
 {
@@ -1063,15 +1083,12 @@ If non-existent or nil, a new one is created.")
        * Create the screen's minibuffer window.  Link it to same buffer
        * as "global" minibuf_window.
        */
-      w = XWINDOW(minibuf_window);
-      b = XBUFFER(w->buffer);      
-      font = XXFONT(STYLE_FIELD(w,font,stylenorm));
-      
       rb->minibuf_window = make_window();
       w = XWINDOW(rb->minibuf_window);
       w->buffer = XWINDOW(minibuf_window)->buffer;
-      b = XBUFFER(w->buffer);
+      b = XBUFFER(w->buffer);	/* Same as minibuf window */
       w->root = new_root;
+      font = XXFONT(STYLE_FIELD(w,font,stylenorm));
       
       XFASTINT(w->pixtop) = ws->in_border + ws->pixh - XFHEIGHT(font); 
       XFASTINT(w->pixleft) = ws->in_border;
@@ -1147,23 +1164,25 @@ x_another_screen(seq,map,dir)
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::next-screen",Fnext_screen,Snext_screen,0,2,0,
-      "return the ID of the \"next\" screen. The minibuf is excepted, and\
-unmapped windows are also. The first optional argument is the ID of the screen\
-to start at - if omitted, the current screen is assumed. The second optional\
-argument if not nil indicates that unmapped screens should be found.")
+      "Return the ID of the \"next\" screen. The minibuf is excepted, and\n\
+unmapped windows are also. The first optional argument is the ID of\n\
+the screen to start at - if omitted, the current screen is assumed.\n\
+The second optional argument if not nil indicates that unmapped\n\
+screens should be found.")
 	(seq,map) Lisp_Object seq,map; /* next from this one */
 { return x_another_screen(seq,map,1); }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::prev-screen",Fprev_screen,Sprev_screen,0,2,0,
-      "return the ID of the \"previous\" screen. The minibuf is excepted, and\
-unmapped windows are also. The first optional argument is the ID of the screen\
-to start at - if omitted, the current screen is assumed. The second optional\
-argument if not nil indicates that unmapped screens should be found.")
+      "Return the ID of the \"previous\" screen. The minibuf is excepted,\n\
+and unmapped windows are also. The first optional argument is the ID\n\
+of the screen to start at - if omitted, the current screen is\n\
+assumed.  The second optional argument if not nil indicates that\n\
+unmapped screens should be found.")
 	(seq,map) Lisp_Object seq,map; /* next from this one */
 { return x_another_screen(seq,map,0); }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::select-screen", Fselect_screen, Sselect_screen, 0, 1, 0,
-      "Change the selected screen to the screen whose ID is passed as the\
+      "Change the selected screen to the screen whose ID is passed as the\n\
 first argument. If omitted, the \"next\" screen is selected.")
 	(seq) Lisp_Object seq;
 {
@@ -1184,20 +1203,34 @@ first argument. If omitted, the \"next\" screen is selected.")
   if (EQ(block,minibuf_rootblock)) return Qnil;
 
   if (XTYPE(XROOT(block)->select) != Lisp_Window)
-    { write(2,msg2,strlen(msg2)); abort(); }
+    {
+      write(2,msg2,strlen(msg2));
+      fprintf 
+        (stderr, "  ** Window type is %d\n", XTYPE(XROOT(block)->select));
+      abort();
+    }
 
   if (XTYPE(XWINDOW(XROOT(block)->select)->buffer) != Lisp_Buffer)
-    { write(2,msg,strlen(msg)); abort(); }
-
+    {
+      write(2,msg,strlen(msg));
+      fprintf 
+        (stderr, "  ** Buffer type is %d\n", 
+         XTYPE(XWINDOW(XROOT(block)->select)->buffer));
+      abort();
+    }
+  
   /*selecting the window fixes everything */
   if (selected_window == minibuf_window)
     {
-      if (!distinct_minibuffer && NIL(XROOT(block)->prev_select))
+      if (!distinct_minibuffer && (XROOT(block)->select != minibuf_window))
 	{
 	  XROOT(block)->prev_select = XROOT(block)->select;
 	}
       /* need to get back to the minibuf in this case */
-      Fselect_window(XROOT(block)->select);
+      if (distinct_minibuffer)
+	Fselect_window(XROOT(block)->select);
+      else
+	Fselect_window(XROOT(block)->prev_select);
       Fselect_window(minibuf_window);
     }
   else
@@ -1266,7 +1299,6 @@ x_select_screen(screen)
   old_screen = EDIT_SCREEN;
   oldrb = XROOT(old_screen);
 
-  oldrb->prev_select = Qnil;
   /* tell WM that it's not current anymore */
   BLOCK_INPUT();
   XChangeProperty(XXSCREEN(oldrb->x11)->display,
@@ -1297,6 +1329,8 @@ x_select_screen(screen)
       /* If minibuffer window is selected window on old screen, make it
        * the selected window on new screen.
        */
+      if (!NIL(oldrb->prev_select))
+	oldrb->select = oldrb->prev_select;
       /* UGLY:  *the* minibuffer window is attached to the window
        * hierarchy of the current screen.  Screen being selected has its
        * own "fake" minibuffer window.  We must swap these two windows
@@ -1881,7 +1915,7 @@ void syms_of_screen()
 
       DEFVAR_LISP("epoch::version",&Vepoch_version,
 		  "Version number of Epoch");
-      Vepoch_version = build_string("Epoch 4.0 Patchlevel 0");
+      Vepoch_version = build_string("Epoch 4.2");
 
       DEFVAR_LISP("epoch::nonlocal-minibuffer",&Vdistinct_minibuffers,
 		  "Set to t if minibuffer is in distinct screen");
