@@ -18,13 +18,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.2 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/search.c,v $
- * $Author: love $
- * $Date: 92/02/23 11:01:14 $
+ * $Revision: 1.3 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/search.c,v $
+ * $Author: marca $
+ * $Date: 92/07/12 03:28:49 $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:01:14 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/search.c,v $ $Revision: 1.2 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/07/12 03:28:49 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/search.c,v $ $Revision: 1.3 $";
 #endif
 
 #include "config.h"
@@ -1283,15 +1283,34 @@ syms_of_search ()
 
   for (i = 0; i < 0400; i++)
     {
-      downcase_table[i] = (i >= 'A' && i <= 'Z') ? i + 040 : i;
+        /*
+         * Table updated to handle extended character set.
+         */
+        if (i >= 'A' && i <= 'Z')
+  	downcase_table[i] = i + 'a' - 'A';
+        else if (i >= 0300 && i <= 0326)
+  	downcase_table[i] = i + 040;
+        else if (i >= 0330 && i <= 0336)
+  	downcase_table[i] = i + 040;
+        else
+  	downcase_table[i] = i;
 /* We do this instead of using compute_trt_inverse to save space. */
- /* Does it? */
-      downcase_table[0400+i]
-	= ((i >= 'A' && i <= 'Z')
-	   ? i + ('a' - 'A')
-	   : ((i >= 'a' && i <= 'z')
-	      ? i + ('A' - 'a')
-	      : i));
+/* Does it? */
+  
+ 	   if (i >= 'A' && i <= 'Z')
+ 	downcase_table[0400+i] = i + 'a' - 'A';
+        else if (i >= 0300 && i <= 0326)
+  	downcase_table[0400+i] = i + 040;
+        else if (i >= 0330 && i <= 0336)
+  	downcase_table[0400+i] = i + 040;
+        else if (i >= 'a' && i <= 'z')
+  	downcase_table[0400+i] = i + 'A' - 'a';
+       else if (i >= 0340 && i <= 0366)
+  	downcase_table[0400+i] = i - 040;
+        else if (i >= 0370 && i <= 0376)
+  	downcase_table[0400+i] = i - 040;
+        else
+  	downcase_table[0400+i] = i;
     }
 /* Use this instead when there come to be multiple translation tables. 
   compute_trt_inverse (downcase_table);    */
