@@ -1,9 +1,9 @@
 
 /*
- * $Header: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/syntax.c,v 1.1 91/02/27 15:44:21 love Exp $
+ * $Header: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/syntax.c,v 1.3 92/07/12 03:28:51 marca Exp $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/02/27 15:44:21 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/syntax.c,v $ $Revision: 1.1 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/07/12 03:28:51 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/syntax.c,v $ $Revision: 1.3 $";
 #endif
 /* GNU Emacs routines to deal with syntax tables; also word and list parsing.
    Copyright (C) 1985, 1987, 1990 Free Software Foundation, Inc.
@@ -113,7 +113,7 @@ One argument, a syntax table.")
  into the code it signifies.
  This is used by modify-syntax-entry, and other things. */
 
-char syntax_spec_code[0400] =
+unsigned char syntax_spec_code[0400] =
   { 0377, 0377, 0377, 0377, 0377, 0377, 0377, 0377,
     0377, 0377, 0377, 0377, 0377, 0377, 0377, 0377,
     0377, 0377, 0377, 0377, 0377, 0377, 0377, 0377,
@@ -1106,6 +1106,34 @@ init_syntax_once ()
 
   for (i = 0; i < 12; i++)
     XFASTINT (v->contents[".,;:?!#@~^'`"[i]]) = (int) Spunct;
+ 
+    /*
+     * Syntax table entries for extended character set:
+     */
+  
+    for (i = 0300; i <= 0326; i++)
+      XFASTINT (v->contents[i]) = (int) Sword;
+    for (i = 0330; i <= 0337; i++)
+      XFASTINT (v->contents[i]) = (int) Sword;
+    for (i = 0340; i <= 0366; i++)
+      XFASTINT (v->contents[i]) = (int) Sword;
+    for (i = 0370; i <= 0377; i++)
+      XFASTINT (v->contents[i]) = (int) Sword;
+  
+    XFASTINT (v->contents[0240]) = (int) Swhitespace;
+    
+    for (i = 0242; i <= 0245; i++)
+      XFASTINT (v->contents[i]) = (int) Sword;
+  
+    for (i = 0246; i <= 0276; i++)
+      XFASTINT (v->contents[i]) = (int) Ssymbol;
+  
+    XFASTINT (v->contents[0327]) = (int) Ssymbol;
+    XFASTINT (v->contents[0367]) = (int) Ssymbol;
+  
+    XFASTINT (v->contents[0241]) = (int) Spunct;
+    XFASTINT (v->contents[0277]) = (int) Spunct;
+
 }
 
 syms_of_syntax ()
