@@ -19,30 +19,22 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.4 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/termhooks.h,v $
- * $Date: 91/08/26 15:37:30 $
+ * $Revision: 1.5 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/termhooks.h,v $
+ * $Date: 91/11/26 13:02:28 $
  * $Author: love $
  */
 
 extern int (*move_cursor_hook) ();
-extern int (*raw_move_cursor_hook) ();
 
 extern int (*clear_window_end_hook) ();
 extern int (*clear_to_end_hook) ();
 extern int (*clear_screen_hook) ();
-extern int (*clear_end_of_line_hook) ();
-
-extern int (*ins_del_lines_hook) ();
-extern int (*shift_region_hook) ();
-
-extern int (*change_line_highlight_hook) ();
-extern int (*reassert_line_highlight_hook) ();
 
 extern int (*update_line_hook) ();
 extern int (*insert_chars_hook) ();
-extern int (*output_chars_hook) ();
 extern int (*delete_chars_hook) ();
+extern int (*shift_region_hook) ();
 
 extern int (*ring_bell_hook) ();
 
