@@ -18,29 +18,32 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.2 $
- * $Source: /home/kaplan/love/epoch/src/RCS/undo.c,v $
- * $Date: 91/09/29 13:10:50 $
+ * $Revision: 1.8 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/undo.c,v $
+ * $Date: 92/05/07 09:53:36 $
  * $Author: love $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/09/29 13:10:50 $ $Source: /home/kaplan/love/epoch/src/RCS/undo.c,v $ $Revision: 1.2 $";
+static char rcsid[] = "$Author: love $ $Date: 92/05/07 09:53:36 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/undo.c,v $ $Revision: 1.8 $";
 #endif
 
 #include "config.h"
 #include "lisp.h"
 #include "buffer.h"
+#include "window.h"
+#include "button.h"
 
-#ifdef HAVE_LEIF
+#ifdef DEFINE_CHANGE_FUNCTIONS
 /* Functions to call before and after each text change. */
 Lisp_Object Vbefore_change_function;
 Lisp_Object Vafter_change_function;
+Lisp_Object Vafter_movement_function;
 #endif
 
 /* Last buffer for which undo information was recorded.  */
 Lisp_Object last_undo_buffer;
 
-#ifdef HAVE_LEIF
+#ifdef DEFINE_CHANGE_FUNCTIONS
 /* Record whether before_change_function or after_change_function are active.
  */
 /* This variable could be replaced by a buffer-local variable. */
@@ -87,6 +90,23 @@ signal_after_change (pos, inspos, dellen)
         unbind_to (count);
       }
 }
+
+/* signal after cursor movement (no text changes) */
+signal_after_movement(last_point)
+     int last_point;
+{
+  int count = specpdl_ptr - specpdl;
+
+  if (!NULL(Vafter_movement_function) && (!change_function_active))
+    {
+      change_function_active = 1;
+      record_unwind_protect (change_function_restore,Qnil);
+      call1(Vafter_movement_function,
+	    make_number(last_point));
+      unbind_to(count);
+    }
+}
+
 #endif
 
 /* Record an insertion that just happened or is about to happen,
@@ -130,12 +150,15 @@ record_insert (beg, length)
 }
 
 /* Record that a deletion is about to take place,
-   for LENGTH characters at location BEG.  */
+   for LENGTH characters at location BEG with BTN_LIST affected buttons.  */
 
-record_delete (beg, length)
+record_delete (beg, length,btn_list)
      int beg, length;
+     Lisp_Object btn_list;
 {
   Lisp_Object lbeg, llength, lend, sbeg;
+  Lisp_Object bstart,bend;
+  Lisp_Object button,buttons = Qnil;
 
   if (current_buffer != XBUFFER (last_undo_buffer))
     Fundo_boundary ();
@@ -153,8 +176,31 @@ record_delete (beg, length)
   XFASTINT (lbeg) = beg;
   XFASTINT (llength) = length;
   XFASTINT (lend) = beg + length;
-  current_buffer->undo_list = Fcons (Fcons (Fbuffer_substring (lbeg, lend), sbeg),
-			     current_buffer->undo_list);
+
+  /* Accumulate info regarding affected buttons.
+   * Must store the button object as well as present start and end pos
+   * Button info is a list of elements, each element being a list
+   * (oldstart oldend button)
+   */
+  while (CONSP(btn_list))
+    {
+      button = Fcar(btn_list);
+      
+      buttons =
+	Fcons(Fcons(make_number(marker_position(XBUTTON(button)->start)),
+		    Fcons(make_number(marker_position(XBUTTON(button)->end)),
+			  Fcons(button,Qnil))),
+	      buttons);
+      btn_list = Fcdr(btn_list);
+    }
+
+  /* Add this to undo list for buffer.  Form is:
+   * ("text" pos button-list)
+   */
+  current_buffer->undo_list =
+    Fcons (Fcons (Fbuffer_substring(lbeg,lend),
+		  Fcons (sbeg,buttons)),
+	   current_buffer->undo_list);
 }
 
 /* Record that a replacement is about to take place,
@@ -164,7 +210,7 @@ record_delete (beg, length)
 record_change (beg, length)
      int beg, length;
 {
-  record_delete (beg, length);
+  record_delete (beg, length, Qnil);
   record_insert (beg, length);
 }
 
@@ -266,6 +312,11 @@ Return what remains of the list.")
      Lisp_Object count, list;
 {
   register int arg = XINT (count);
+
+  extern int buffer_shared;
+  extern Lisp_Object selected_window;
+  extern int update_mode_lines;
+  
 #if 0  /* This is a good feature, but would make undo-start
 	  unable to do what is expected.  */
   Lisp_Object tem;
@@ -281,13 +332,15 @@ Return what remains of the list.")
     {
       while (1)
 	{
-	  Lisp_Object next, car, cdr;
+	  Lisp_Object next, car, cdr, cadr, cddr;
 	  next = Fcar (list);
 	  list = Fcdr (list);
 	  if (NULL (next))
 	    break;
 	  car = Fcar (next);
 	  cdr = Fcdr (next);
+	  cadr = CONSP(cdr) ? Fcar(cdr) : Qnil;
+	  cddr = CONSP(cdr) ? Fcdr(cdr) : Qnil;
 	  if (EQ (car, Qt))
 	    {
 	      Lisp_Object high, low;
@@ -304,6 +357,12 @@ Return what remains of the list.")
 	      Funlock_buffer ();
 #endif /* CLASH_DETECTION */
 	      Fset_buffer_modified_p (Qnil);
+	      if (current_buffer == XBUFFER(XWINDOW(selected_window)->buffer)
+		  && buffer_shared == 1)
+		{
+		  update_mode_lines = 0;
+		  XWINDOW(selected_window)->update_mode_line = Qt;
+		}
 	    }
 	  else if (XTYPE (car) == Lisp_Int && XTYPE (cdr) == Lisp_Int)
 	    {
@@ -314,10 +373,10 @@ Return what remains of the list.")
 	      Fdelete_region (car, cdr);
 	      Fgoto_char (car);
 	    }
-	  else if (XTYPE (car) == Lisp_String && XTYPE (cdr) == Lisp_Int)
+	  else if (XTYPE (car) == Lisp_String && XTYPE (cadr) == Lisp_Int)
 	    {
 	      Lisp_Object membuf;
-	      int pos = XINT (cdr);
+	      int pos = XINT (cadr);
 	      membuf = car;
 	      if (pos < 0)
 		{
@@ -331,9 +390,22 @@ Return what remains of the list.")
 		  if (pos < BEGV || pos > ZV)
 		    error ("Changes to be undone are outside visible portion of buffer");
 		  SET_PT (pos);
-		  Finsert (1, &membuf);
+		  Finsert_before_markers (1, &membuf);
 		  SET_PT (pos);
 		}
+	      /* Now restore buttons (if any) */
+	      while (CONSP(cddr))
+		{
+		  Lisp_Object binfo = Fcar(cddr);
+		  Lisp_Object button = Fcar(Fcdr(Fcdr(binfo)));
+
+		  if (BUTTONP(button) && !NULL(Fbutton_buffer(button)))
+		    {
+		      Fmove_button(button,Fcar(binfo),Fcar(Fcdr(binfo)),Qnil);
+		    }
+
+		  cddr = Fcdr(cddr);
+		}
 	    }
 	}
       arg--;
@@ -346,7 +418,7 @@ syms_of_undo ()
 {
   defsubr (&Sprimitive_undo);
   defsubr (&Sundo_boundary);
-#ifdef HAVE_LEIF
+#ifdef DEFINE_CHANGE_FUNCTIONS
   change_function_active = 0; /* initially inactive */
 
   DEFVAR_LISP ("before-change-function", &Vbefore_change_function,
@@ -358,7 +430,7 @@ syms_of_undo ()
  cause calls to any before-change-function or after-change-function.");
    Vbefore_change_function = Qnil;
 
-   DEFVAR_LISP ("after-change-function", &Vafter_change_function,
+  DEFVAR_LISP ("after-change-function", &Vafter_change_function,
               "Function to call after each text change.\n\
  Three arguments are passed to the function: the position of the change,\n\
  the position of the end of the region deleted, and the position of \n\
@@ -367,5 +439,14 @@ syms_of_undo ()
  While executing the after-change-function, changes to buffers do not\n\
  cause calls to any before-change-function or after-change-function.");
    Vafter_change_function = Qnil;
+
+  DEFVAR_LISP ("after-movement-function", &Vafter_movement_function,
+	       "Function to call after point movement (with no text change).\n\
+ No arguments are passed to this function.\n\
+ \n\
+ While executing the after-movement-function, changes to buffers do not\n\
+ cause calls to any before-change or after-change function.");
+  Vafter_movement_function = Qnil;
+
 #endif
 }
