@@ -19,13 +19,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.34 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/window.c,v $
- * $Date: 92/04/11 19:47:08 $
- * $Author: love $
+ * $Revision: 1.36 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/window.c,v $
+ * $Date: 92/08/05 04:15:57 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/04/11 19:47:08 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/window.c,v $ $Revision: 1.34 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/05 04:15:57 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/window.c,v $ $Revision: 1.36 $";
 #endif
 
 #include "config.h"
@@ -315,6 +315,27 @@ and BOTTOM is one more than the bottommost row used by WINDOW\n\
 and its mode-line.")
   (window)
 {
+  register struct window *w = decode_window(window);
+  register struct Root_Block *rb = XROOT(w->root);
+  WS_DEF;
+  register int l = XFASTINT(w->pixleft) - ws->in_border;
+  register int t = XFASTINT(w->pixtop) - ws->in_border;
+  int r, b;
+  struct X_font *font  = XXFONT(XSTYLE(rb->stylenorm)->font);
+  
+  l = l / XFWIDTH(font);
+  t = t / XFHEIGHT(font);
+
+  b=window_char_height(w)+t;
+  r=window_char_width(w)+l;
+  if (XFASTINT(w->pixleft) != ws->in_border) l++;
+  return Fcons(make_number(l),
+	       Fcons(make_number(t),
+		     Fcons(make_number(r),
+			   Fcons(make_number(b),
+				 Qnil))));
+
+#if 0 /* old code */
   register struct window *w = decode_window(window);
   register struct Root_Block *rb = XROOT(w->root);
   WS_DEF;
@@ -330,6 +351,7 @@ and its mode-line.")
 		       Fcons(window_char_width(w),
 			     Fcons(window_char_height(w),
 				   Qnil))));
+#endif
 }
 
 DEFUN ("window-pixedges", Fwindow_pixedges, Swindow_pixedges, 0, 1, 0,
@@ -345,7 +367,12 @@ and its mode-line.  MODE is pixel top of modeline")
   register int mode = !NULL(w->modeline) ? XLINE(w->modeline)->ascent +
     XLINE(w->modeline)->descent : 0;
   
-  return Fcons (w->pixleft, Fcons (w->pixtop,
+  register struct Root_Block *rb = XROOT(w->root);                
+  WS_DEF;                                                         
+  int delta=(w->pixleft==ws->in_border) ?                         
+    0 : XFWIDTH(XXFONT(XSTYLE(XROOT(w->root)->stylenorm)->font)); 
+
+  return Fcons (w->pixleft+delta, Fcons (w->pixtop,               
            Fcons (make_number (XFASTINT (w->pixleft) + XFASTINT (w->pixwidth)),
 		  Fcons (make_number (XFASTINT (w->pixtop)
 				      + XFASTINT (w->pixheight)),
@@ -1140,6 +1167,18 @@ before each command.")
   windows_or_buffers_changed++;
   screen_changed++;
 
+  {
+    /* blk@mitre.org  Sat Jun  6 19:11:01 1992
+     * Call value of window-select-hook if it is bound and non-nil.
+     * Derived from epoch code for set-window-buffer-hook.
+     */
+    register Lisp_Object tem;
+    tem = intern("window-select-hook");
+    tem = XSYMBOL (tem)->value;	/* convert to value */
+    if (! EQ (tem, Qunbound) && ! EQ (tem, Qnil))
+      tem = call1 (tem,window);
+  }
+  
   return window;
 }
 
