@@ -18,13 +18,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * $Revision: 1.22 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/x11term.c,v $
- * $Date: 91/09/25 20:40:05 $
- * $Author: love $
+ * $Revision: 1.73 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/x11term.c,v $
+ * $Date: 92/08/26 23:14:16 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/09/25 20:40:05 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/x11term.c,v $ $Revision: 1.22 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/08/26 23:14:16 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/x11term.c,v $ $Revision: 1.73 $";
 #endif
 
 /* On 4.3 this loses if it comes after x11term.h.
@@ -47,9 +47,10 @@ static char rcsid[] = "$Author: love $ $Date: 91/09/25 20:40:05 $ $Source: /impo
    but it was moved up above to avoid problems.  */
 #ifdef USG
 #include <termio.h>
-#include <fcntl.h>
 #endif /* USG */
+#include <fcntl.h>
 
+#include "lisp.h"
 #undef NULL
 
 /* Allow m- file to inhibit use of interrupt-driven input.  */
@@ -71,39 +72,13 @@ static char rcsid[] = "$Author: love $ $Date: 91/09/25 20:40:05 $ $Source: /impo
 
 #include "x11term.h"
 
-#ifdef IRIS
-#include <sys/sysmacros.h>	/* for "minor" */
-#include <sys/time.h>
-#else
-#ifdef UNIPLUS
-#include <sys/time.h>
-
-#else /* not IRIS, not UNIPLUS */
-/* Use socket.h just to control whether to use time.h or sys/time.h.
-   This works like the code in process.c.  */
 #ifdef HAVE_SOCKETS
-#include <sys/socket.h>
-#endif
-#ifdef HAVE_TIMEVAL
-/* _h_BSDTYPES is checked because on ISC unix, socket.h includes
-   both time.h and sys/time.h, and the latter file is protected
-   from repeated inclusion.  */
-#if defined(USG) && !defined(AIX) && !defined(_h_BSDTYPES) && !defined(USG_SYS_TIME)
-#include <time.h>
-#else /* _h_BSDTYPES or not USG */
-#include <sys/time.h>
-#endif /* _h_BSDTYPES or not USG */
-#endif /* HAVE_TIMEVAL */
-#endif /* not UNIPLUS */
-#endif /* not IRIS */
-
-#ifdef BAT68K
-#include <sys/time.h>   /* In addition to time.h.  */
+#include <sys/socket.h>		/* Must be done before gettime.h */
 #endif
+/* Include time.h, sys/time.h, or both. */
+#include "gettime.h"
 
 #ifdef AIX
-#include <sys/time.h>   /* In addition to time.h.  */
-
 static KeySym XMOD_Alt[] = { XK_Alt_L };
 static KeySym XMOD_Shift[] = { XK_Shift_L };
 static KeySym XMOD_ShiftAlt[] = { XK_Alt_L, XK_Shift_L };
@@ -113,7 +88,6 @@ static KeySym XMOD_CtrlShift[] = { XK_Control_L, XK_Shift_L };
 static KeySym XMOD_ShiftCtrlAlt[] = { XK_Control_L, XK_Alt_L, XK_Shift_L };
 #endif
 
-#include <fcntl.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
@@ -122,29 +96,37 @@ static KeySym XMOD_ShiftCtrlAlt[] = { XK_Control_L, XK_Alt_L, XK_Shift_L };
 #endif
 #include <sys/stat.h>
 
+#ifndef NULL
+#define NULL 0
+#endif
+
+#include "dispepoch.h"		/* Epoch */
 #include "termhooks.h"
 #include "termopts.h"
 #include "termchar.h"
 
-#include "sink11.h"
-#include "sink11mask.h"
-#include "lisp.h"
 #include "window.h"		/* Epoch */
 #include "buffer.h"
 #include "screen.h"		/* Epoch */
 #include "screenW.h"		/* Epoch */
 #include "screenX.h"		/* Epoch */
-#include "dispepoch.h"		/* Epoch */
 #include "button.h"		/* Epoch */
 #include "xresource.h"		/* Epoch */
 
+/* X11 includes used; use NIL rather than NULL from lisp.h */
+
 #define min(a,b) ((a)<(b) ? (a) : (b))
 #define max(a,b) ((a)>(b) ? (a) : (b))
 #define MAPPINGBUFLEN 256
 extern int errno;
 
+#ifdef USG5_4
+#define sigunblockx(sig) sigrelse(sig)
+#define sigblockx(sig) sighold(sig)
+#else
 #define sigunblockx(sig) sigblock (SIGEMPTYMASK)
-#define sigblockx(sig) sigblock (1 << ((sig) - 1))
+#define sigblockx(sig) sigblock (sigmask ((sig)))
+#endif
 
 #define METABIT 0200
 #define MINWIDTH 12	/* In pixels */
@@ -153,7 +135,6 @@ extern int errno;
 
 char *progname;
 
-
 XButtonEvent *XXm_queue[XMOUSEBUFSIZE];
 int XXm_queue_num, XXm_queue_in, XXm_queue_out;
 extern Lisp_Object Vx_mouse_events;
@@ -179,14 +160,14 @@ extern int xargc;
 extern char **xargv;
 
 int XXpid;
+  
+static XComposeStatus status;
 
 static int InUpdate;		/* many of functions here may be invoked
 				 * even if no update in progress; when
 				 * no update is in progress the action
 				 * can be slightly different */
 
-Pixmap SinkPixmap, SinkMaskPixmap;
-
 extern Atom XA_current;
 
 /* 0 for no focus, 1 for Pointer focus, 2 for Explicit focus */
@@ -196,8 +177,8 @@ int (*handler)();
 
 static void x_init_1 ();
 
-#ifndef NeXT
-char *rindex();
+#if !defined(NeXT) && !defined(AIX)
+extern char *rindex();
 #endif
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -214,8 +195,7 @@ load_font(name)
   XFontStruct *font;
   BLOCK_INPUT_DECLARE();
   struct X_font *f,*new;
-  int val;
-  long wid;
+  unsigned long wid,widi;
 
   f = font_list;
   while (f && strcmp(name,f->font_name))
@@ -231,7 +211,8 @@ load_font(name)
 
   if (!XGetFontProperty(font,XA_QUAD_WIDTH,&wid))
     wid = XTextWidth(font,"M",1);
-    
+  widi = XTextWidth(font,"i",1);
+  
   new = (struct X_font *) malloc(sizeof (struct X_font));
 
   new->font_name = strdup(name);
@@ -240,6 +221,10 @@ load_font(name)
   new->ascent = font->ascent;
   new->descent = font->descent;
   new->width = wid;
+  new->variable = (wid != widi);
+#ifdef HAVE_GRAPHIC_ZONES
+  new->virtual = 0;
+#endif  
   font_list = new;
 
   return new;
@@ -270,62 +255,7 @@ TextWidth(font_struct, string,length)
      char *string;
      int length;
 {
-    return XTextWidth(font_struct,string,length);
-}
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-/* HLmode -- Changes the GX function for output strings.  Could be used to
- * change font.  Check an XText library function call.
- */
-CHLmode (new,w)
-     int new;
-     struct window *w;
-{
-  struct Root_Block *rb = XROOT(w->root);
-
-  XXSCREEN(rb->x11)->highlight = new;
-}
-
-HLmode (new)
-     int new;
-{ return CHLmode(new,XWINDOW(selected_window)); }
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-/* External interface to control of standout mode.
- * Call this when about to modify line at position VPOS
- * and not change whether it is highlighted.
- */
-CXTreassert_line_highlight (highlight, vpos,w)
-     int highlight, vpos;
-     struct window *w;
-{
-  struct Root_Block *rb = XROOT(w->root);
-  
-  CHLmode (highlight,w);
-}
-
-XTreassert_line_highlight (highlight,vpos)
-     int highlight,vpos;
-{
-  return CXTreassert_line_highlight (highlight,XWINDOW(selected_window));
-}
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-/* Call this when about to modify line at position VPOS
- * and change whether it is highlighted.
- */
-CXTchange_line_highlight (new_highlight, vpos, first_unused_hpos,w)
-     int new_highlight, vpos, first_unused_hpos;
-     struct window *w;
-{
-  struct Root_Block *rb = XROOT(w->root);  
-  CHLmode (new_highlight,w);
-  XTmove_cursor (vpos, 0,0,0,rb); /* WRONG */
-  Cx_clear_end_of_line (0,w);
-}
-
-XTchange_line_highlight (new_highlight, vpos, first_unused_hpos)
-     int new_highlight, vpos, first_unused_hpos;
-{
-  return CXTchange_line_highlight (new_highlight,vpos,first_unused_hpos,
-				   XWINDOW(selected_window));
+  return XTextWidth(font_struct,string,length);
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* Used for starting or restarting (after suspension) the X window.  Puts the
@@ -341,6 +271,9 @@ CXTset_terminal_modes (w)
 	fprintf (stderr, "XTset_terminal_modes\n");
 #endif
   InUpdate = 0;
+  status.compose_ptr = 0;
+  status.chars_matched = 0;
+   
   if (!initialized)
     {
       xs->cursor_exists = 0;
@@ -393,7 +326,6 @@ XTshift_region(w,start,end)
 	{
 	  /* Shifted region up */
 	  i = end->ypos + end->descent;
-/*	  i = new_top + length; */
 	  length = old_top - new_top;
 	  XClearArea(xs->display,xs->xid,
 		     XFASTINT(w->pixleft),i,
@@ -402,6 +334,29 @@ XTshift_region(w,start,end)
     }
   UNBLOCK_INPUT();
 }
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+/* Fastmove_cursor - called from output_for_insert().  Note:  in this case,
+  * a character has just been inserted (overwriting past cursor), and cursor
+  * needs to be redrawn in its new spot.
+  */
+Fastmove_cursor(rb)
+     struct Root_Block *rb;
+{
+  XS_DEF;
+  WS_DEF;
+  BLOCK_INPUT_DECLARE();
+  
+  /* Fudge so we are toggling the cursor BACK into existence. */
+
+  xs->cursor_exists = 0;
+  ws->vis_xpos = ws->cursor_x;
+  BLOCK_INPUT();
+  CursorToggle(rb);
+  XFlush(xs->display);
+  UNBLOCK_INPUT();
+  return;
+}
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* XTmove_cursor moves the cursor to the correct location and checks whether an
  * update is in progress in order to toggle it on.
@@ -415,7 +370,8 @@ XTmove_cursor (l,cb,row, col,win,rb)
 {
   XS_DEF; WS_DEF;
   BLOCK_INPUT_DECLARE ();
-#ifdef XDEBUG
+
+#if 0  
   fprintf (stderr, "XTmove_cursor (X %d, Y %d)\n",col,row);
 #endif
 
@@ -423,20 +379,6 @@ XTmove_cursor (l,cb,row, col,win,rb)
 
   if (InUpdate)
     {
-      if (ws->cur_w && (ws->new_cur_w != ws->cur_w &&
-	  ws->cur_w != XWINDOW(rb->minibuf_window)))
-	{
-	  char c = ws->cur_char->ch == -2 ? ' ' : ws->cur_char->ch;	  
-	  /* UGLY:  Cursor's previous position may be in another window;
-	   * this will result in a blank or stray cursor left in that other
-	   * window.  Find the window, and erase the old cursor
-	   */
-
-	  ShipOutTextBlock(&c,1,
-			   ws->cur_char->xpos,ws->cur_line->ypos,
-			   ws->cur_line->ascent,ws->cur_line->descent,0,
-			   ws->cur_char->style,ws->cur_w);
-	}
       /* Cursor is already dead.  Now put it in its place. */
       ws->cur_w = win;
       ws->cur_line = l;
@@ -444,10 +386,7 @@ XTmove_cursor (l,cb,row, col,win,rb)
       ws->vis_xpos = col;
       ws->vis_ypos = row;       
       if (xs->cursor_exists)
-	{
-/*	  fprintf(stderr,"**1**\n"); */
-	  CursorToggle (rb);
-	}
+	CursorToggle (rb);
       UNBLOCK_INPUT ();
       return;
       /* Generally, XTmove_cursor will be invoked
@@ -455,21 +394,44 @@ XTmove_cursor (l,cb,row, col,win,rb)
        * so that wasteful XFlush is not called
        */
     }
-  if ((row == ws->vis_ypos) && (col == ws->vis_xpos))
+  if (ws->cur_w &&
+      (ws->new_cur_w != ws->cur_w &&
+       ws->cur_w != XWINDOW(rb->minibuf_window)) &&
+      find_window(ws->cur_w,rb->ewin))
     {
+      char a[2];
+      struct Lisp_Style *style = find_style(ws->cur_w,0);
+      a[0] = ws->cur_char->ch == 0 ? ' ' : ws->cur_char->ch;
+      a[1] = 0;
+	  
+      /* UGLY:  Cursor's previous position may be in another window
+       * still being displayed; this will result in a blank or stray
+       * cursor left in that other window.  Find the window, and
+       * erase the old cursor
+       */
+      ShipOutTextBlock(a,1,
+		       ws->cur_char->xpos,ws->cur_line->ypos,
+		       ws->cur_line->ascent,ws->cur_line->descent,0,
+		       ws->cur_char->style ? ws->cur_char->style : style,
+		       ws->cur_w);
 
-      fprintf(stderr,"Bam2;\n");
-
+      xs->cursor_exists = 0;
       ws->cur_w = win;
       ws->cur_line = l;
       ws->cur_char = cb;
       ws->vis_xpos = col;
-      ws->vis_ypos = row;       
+      ws->vis_ypos = row;
+      CursorToggle(rb);
+      UNBLOCK_INPUT();
+      return;
+    }     
+  if ((row == ws->vis_ypos) && (col == ws->vis_xpos))
+    {
+      ws->cur_w = win;
+      ws->cur_line = l;
+      ws->cur_char = cb;
       if (!xs->cursor_exists)
-	{
-/*	  fprintf(stderr,"**2**\n"); */
-	  CursorToggle (rb);
-	}
+	CursorToggle (rb);
       XFlush (xs->display);
       UNBLOCK_INPUT ();
       return;
@@ -478,10 +440,7 @@ XTmove_cursor (l,cb,row, col,win,rb)
    * First remove cursor from where it is
    */
   if (xs->cursor_exists)
-    {
-/*      fprintf(stderr,"**3**\n"); */
-      CursorToggle (rb);
-    }
+    CursorToggle (rb);
   /*
    * Now update ptrs to where cursor actually is
    */
@@ -494,12 +453,12 @@ XTmove_cursor (l,cb,row, col,win,rb)
    * Now plot it
    */
   if (!xs->cursor_exists)
-    {
-/*      fprintf(stderr,"**4**\n"); */
-      CursorToggle (rb);
-    }
+    CursorToggle (rb);
   XFlush (xs->display);
   UNBLOCK_INPUT ();
+#if 0
+  fprintf(stderr, "End move cursor\n");
+#endif  
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* Used to get the terminal back to a known state after resets.  Usually
@@ -508,9 +467,7 @@ XTmove_cursor (l,cb,row, col,win,rb)
 Ccleanup (w)
      struct window *w;
 {
-  struct Root_Block *rb = XROOT(w->root);  
   inverse_video = 0;
-  CHLmode (0, w);
 }
 
 cleanup ()
@@ -536,106 +493,6 @@ XTclear_window_end(w,ypos1,ypos2)
   UNBLOCK_INPUT();
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-/* Erase current line from current column to column END.
- * Leave cursor at END.
- */
-CXTclear_end_of_line (end,w)
-     register int end;
-     struct window *w;
-{
-  struct Root_Block *rb = XROOT(w->root);  
-  WS_DEF; XS_DEF;
-  BLOCK_INPUT_DECLARE ();
-
-#ifdef XDEBUG
-  fprintf (stderr, "XTclear_end_of_line (to %d)\n",end);
-#endif
-
-  if (ws->cursor_y < 0 || ws->cursor_y >= WS->height)
-    return;
-
-  if (end <= ws->cursor_x)
-    return;
-  if (end >= ws->width)
-    end = ws->width;
-
-  BLOCK_INPUT ();
-  if (ws->cursor_y == ws->vis_ypos
-      && ws->vis_xpos >= ws->cursor_x
-      && ws->vis_xpos < end
-      && xs->cursor_exists)
-    CursorToggle (rb);
-  if (xs->highlight)
-    XFillRectangle (xs->display, xs->xid, xs->gc_norm,
-		    ws->cur_char->xpos,
-		    ws->cur_line->ypos - ws->cur_line->ascent,
-		    ws->cur_line->lwidth - ws->cur_char->xpos,
-		    ws->cur_line->ascent + ws->cur_line->descent);
-  else
-    XClearArea (xs->display, xs->xid,
-		ws->cur_char->xpos,
-		ws->cur_line->ypos - ws->cur_line->ascent,
-		ws->cur_line->lwidth - ws->cur_char->xpos,
-		ws->cur_line->ascent + ws->cur_line->descent,0);
-
-  XTmove_cursor (ws->cur_line,ws->cur_line->end,ws->cursor_y, end,rb);
-  UNBLOCK_INPUT ();
-}
-
-XTclear_end_of_line (end)
-     register int end;
-{
-  return CXTclear_end_of_line(end,XWINDOW(selected_window));
-}
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-/* Erase current line from column START to right margin.
- * Leave cursor at START.
- */
-Cx_clear_end_of_line (start,w)
-     register int start;
-     struct window *w;
-{
-  struct Root_Block *rb = XROOT(w->root);  
-  XS_DEF; WS_DEF;
-  BLOCK_INPUT_DECLARE ();
-
-#ifdef XDEBUG
-  fprintf (stderr, "x_clear_end_of_line (start %d)\n", start);
-#endif
-
-  if (ws->cursor_y < 0 || ws->cursor_y >= ws->height)
-    return;
-
-  if (start >= ws->width)
-    return;
-  if (start < 0)
-    start = 0;
-
-  BLOCK_INPUT ();
-  if (ws->cursor_y == ws->vis_ypos && ws->vis_xpos >= start)
-    if (xs->cursor_exists) CursorToggle (rb);
-  if (xs->highlight)
-    XFillRectangle (xs->display, xs->xid, xs->gc_norm,
-		    ws->cur_char->xpos, /* WRONG */
-		    ws->cur_line->ypos - ws->cur_line->ascent,
-		    ws->cur_line->lwidth - ws->cur_char->xpos,
-		    ws->cur_line->ascent + ws->cur_line->descent);
-  else
-    XClearArea (xs->display, xs->xid,
-		ws->cur_char->xpos,
-		ws->cur_line->ypos - ws->cur_line->ascent,
-		ws->cur_line->lwidth - ws->cur_char->xpos,
-		ws->cur_line->ascent + ws->cur_line->descent,0);
-  XTmove_cursor (ws->cur_line,ws->cur_char,ws->cursor_y, start,rb);
-  UNBLOCK_INPUT ();
-}
-
-x_clear_end_of_line (start)
-     register int start;
-{
-  return Cx_clear_end_of_line(start,XWINDOW(selected_window));
-}
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 CXTreset_terminal_modes (w)
      struct window *w;
 {
@@ -662,7 +519,6 @@ CXTclear_screen (w)
 #endif
 
   BLOCK_INPUT ();
-/*  CHLmode (0,w); */
   xs->cursor_exists = 0;
 
   ws->cursor_x = ws->cursor_y = 0;
@@ -687,10 +543,10 @@ CXTflash (w)
 {
   struct Root_Block *rb = XROOT(w->root);  
 #ifdef HAVE_TIMEVAL
-#ifdef HAVE_SELECT
+  struct timeval to;
+#endif
   XGCValues gcv;
   GC thegc;
-  struct timeval to;
   BLOCK_INPUT_DECLARE ();
   XS_DEF; WS_DEF;
 
@@ -714,6 +570,13 @@ CXTflash (w)
 
   UNBLOCK_INPUT ();
 
+#ifdef HAVE_TIMEVAL
+  to.tv_sec = 0;
+  to.tv_usec = 250000;
+
+  select(0, 0, 0, 0, &to);
+#endif
+
   BLOCK_INPUT ();
 
   XFillRectangle (xs->display, xs->xid, thegc, 0, 0,
@@ -723,8 +586,6 @@ CXTflash (w)
   XFlush (xs->display);
 
   UNBLOCK_INPUT ();
-#endif
-#endif
 }
 
 XTflash()
@@ -767,20 +628,14 @@ XTfeep ()
 CursorToggle (rb)
      struct Root_Block *rb;
 {
-  int x,y;
+  int y;
   WS_DEF; XS_DEF;
-  char c,a[1];
+  char a[2];
   int wid;
   struct window *w = XWINDOW(rb->select);
   struct buffer *b = XBUFFER(w->buffer);
   struct X_font *font = XXFONT(STYLE_FIELD(w,font,stylenorm));
-#if 0
-  if (rb == cur_root)
-    {
-      fprintf(stderr,"X_focus=%d\n",X_focus);
-      fprintf(stderr,"Cursor_Exists = %d\n",xs->cursor_exists);
-    }
-#endif
+
   if (!xs->mapped)
     {
       xs->cursor_exists = 0;
@@ -791,7 +646,7 @@ CursorToggle (rb)
        */
     }
 
-  if (ws->vis_xpos < 0 || ws->vis_ypos < 0)
+  if (ws->vis_ypos < 0)
     {
       /* Not much can be done */
       XFlush (xs->display);
@@ -803,14 +658,16 @@ CursorToggle (rb)
    * case.
    */
 
-  if (!ws->cur_line || ws->cur_char->xpos < w->pixleft ||
-      ws->cur_char->xpos > w->pixleft + w->pixwidth)
+  if (!ws->cur_line || !ws->cur_char
+      || ws->cur_char->xpos < w->pixleft
+      || ws->cur_char->xpos > w->pixleft + w->pixwidth)
     {
       XFlush(xs->display);
       xs->cursor_exists = 0;
       return 0;
     }
 
+  a[1] = 0;
   /* Epoch:
    * Potential problem:  cursors in minibuffer screen and current edit
    * screen.  If we are toggling cursor on screen other than current screen,
@@ -820,21 +677,24 @@ CursorToggle (rb)
 
   if (ws->vis_xpos <= ws->cur_line->chars)
     {
-      c = ws->cur_char->ch == -2 ? ' ' : ws->cur_char->ch;
-      a[0] = c; wid = text_width(font->x_font,a,1);
+      a[0] = ws->cur_char->ch == 0 ? ' ' : ws->cur_char->ch;
+      if (ws->cur_char->style &&
+	  (XTYPE(ws->cur_char->style->font) == Lisp_Raw_Data))
+	font = XXFONT(ws->cur_char->style->font);
+      wid = text_width(font,a,1);
       if (xs->cursor_exists)
 	{
-	  ShipOutTextBlock(&c,1,
+	  ShipOutTextBlock(a,1,
 			   ws->cur_char->xpos,ws->cur_line->ypos,
 			   ws->cur_line->ascent,ws->cur_line->descent,0,
-			   ws->cur_char->style,w);
+			   ws->cur_char->style,ws->cur_w);
 	}
       else if (! X_focus)
 	{
-	  ShipOutTextBlock(&c,1,
+	  ShipOutTextBlock(a,1,
 			   ws->cur_char->xpos,ws->cur_line->ypos,
 			   ws->cur_line->ascent,ws->cur_line->descent,0,
-			   ws->cur_char->style,w);
+			   ws->cur_char->style,ws->cur_w);
 	  XDrawRectangle(xs->display,xs->xid,xs->gc_norm,
 			 ws->cur_char->xpos,
 			 ws->cur_line->ypos - ws->cur_line->ascent,
@@ -844,24 +704,37 @@ CursorToggle (rb)
       else
 	/* use reverse attribute for plotting cursor */
 	{
-	  ShipOutTextBlock(&c,1,
+	  ShipOutTextBlock(a,1,
 			   ws->cur_char->xpos,ws->cur_line->ypos,
 			   ws->cur_line->ascent,ws->cur_line->descent,1,
-			   ws->cur_char->style,w);
+			   ws->cur_char->style,ws->cur_w);
 	}
     }
   else
     {
-      c = ws->cur_char->ch == -2 ? ' ' : ws->cur_char->ch;
-      a[0] = c; wid = text_width(font->x_font,a,1);
-/*      wid = ws->cur_char->ch == -2 ? XFWIDTH(font) : ws->cur_char->width;       */
+      a[0] = ws->cur_char->ch == 0 ? ' ' : ws->cur_char->ch;
+      if (ws->cur_char->style &&
+	  (XTYPE(ws->cur_char->style->font) == Lisp_Raw_Data))
+	font = XXFONT(ws->cur_char->style->font);
+      wid = text_width(font,a,1);
       if (xs->cursor_exists)
 	{
-	  XClearArea(xs->display,xs->xid,
-		     ws->cur_char->xpos,
-		     ws->cur_line->ypos - ws->cur_line->ascent,
-		     wid,
-		     ws->cur_line->ascent + ws->cur_line->descent,0);
+	  int height = ws->cur_line->ascent ?
+	    (ws->cur_line->ascent + ws->cur_line->descent) : XFHEIGHT(font);
+
+          if (!distinct_minibuffer && ws->cur_w == XWINDOW(rb->minibuf_window))
+	    {
+	      ;
+	    }
+	  else
+	    {
+	      
+	    XClearArea(xs->display,xs->xid,
+		       ws->cur_char->xpos,
+		       ws->cur_line->ypos - ws->cur_line->ascent,
+		       wid,
+		       height,0);
+	    }
 	}
       else if (! X_focus)
 	XDrawRectangle(xs->display,xs->xid,xs->gc_norm,
@@ -871,16 +744,10 @@ CursorToggle (rb)
 		       ws->cur_line->ascent + ws->cur_line->descent -1);
       else
 	{
-	  ShipOutTextBlock(&c,1,
+	  ShipOutTextBlock(a,1,
 			   ws->cur_char->xpos,ws->cur_line->ypos,
 			   ws->cur_line->ascent,ws->cur_line->descent,1,
-			   XSTYLE(rb->stylecursor),w);
-#if 0	  
-	  ShipOutTextBlock(&c,1,
-			   ws->cur_char->xpos,ws->cur_line->ypos,
-			   ws->cur_line->ascent,ws->cur_line->descent,0,
-			   ws->cur_char->style,w);
-#endif	  
+			   XSTYLE(rb->stylecursor),ws->cur_w);
 	}
     }
 
@@ -905,16 +772,11 @@ ClearCursor(rb)
   XS_DEF; WS_DEF;
   BLOCK_INPUT_DECLARE ();
   char a[1];
-  int wid;
+  int wid,height;
   struct window *w = XWINDOW(rb->select);
   struct buffer *b = XBUFFER(w->buffer);
   struct X_font *font = XXFONT(STYLE_FIELD(w,font,stylenorm));
   
-#ifdef CURSOR_DEBUG
-  fprintf(stderr,"Clear Cursor\n");
-  fprintf(stderr,"Cursor_exists: %d\n",xs->cursor_exists);
-#endif   
-
   BLOCK_INPUT ();
   if (!xs->mapped)
     {
@@ -923,8 +785,8 @@ ClearCursor(rb)
       return;
     }
 	
-  if (ws->vis_xpos < 0 || ws->vis_xpos >= ws->width ||
-      ws->vis_ypos < 0 || ws->vis_ypos >= ws->height)
+  if (ws->vis_xpos >= ws->width || ws->vis_ypos < 0 ||
+      ws->vis_ypos >= ws->height || !ws->new_cur_char || !ws->new_cur_line)
     {
       /* Not much can be done */
       xs->cursor_exists = 0;
@@ -933,13 +795,18 @@ ClearCursor(rb)
     }
 
   a[0] = ' ';
-  wid = ws->new_cur_char->ch == -2 ? text_width (font->x_font,a,1) : 
+  if (ws->new_cur_char->style &&
+      (XTYPE(ws->new_cur_char->style->font) == Lisp_Raw_Data))
+    font = XXFONT(ws->new_cur_char->style->font);
+  wid = ws->new_cur_char->ch == 0 ? text_width (font,a,1) : 
     ws->new_cur_char->width;
+  height = ws->new_cur_line->ascent ?
+    (ws->new_cur_line->ascent + ws->new_cur_line->descent) : XFHEIGHT(font);
 
   XClearArea (xs->display, xs->xid,
 	      ws->new_cur_char->xpos,
 	      ws->new_cur_line->ypos - ws->new_cur_line->ascent,wid,
-	      ws->new_cur_line->ascent + ws->new_cur_line->descent,0);
+	      height,0);
   xs->cursor_exists = 0;
   UNBLOCK_INPUT ();
 }
@@ -950,7 +817,6 @@ CXTupdate_begin(w)
   struct Root_Block *rb = XROOT(w->root);  
   XS_DEF; WS_DEF;
   BLOCK_INPUT_DECLARE ();
-  register int i;
 	
 #ifdef XDEBUG
   fprintf (stderr, "XTupdate_begin\n");
@@ -958,9 +824,11 @@ CXTupdate_begin(w)
 
   BLOCK_INPUT ();
   InUpdate = 1;
-  if (xs->cursor_exists)
+  if (xs->cursor_exists && w == XWINDOW(rb->select))
     {
-/*      fprintf(stderr,"**5**\n"); */
+#if 0      
+      fprintf(stderr,"Blink1\n");
+#endif      
       CursorToggle (rb);
     }
   UNBLOCK_INPUT ();
@@ -981,17 +849,35 @@ CXTupdate_end (w)
 #endif
 
   BLOCK_INPUT ();
-  ws->vis_xpos = ws->cursor_x;
-  ws->vis_ypos = ws->cursor_y;  
-  if (xs->cursor_exists)
+  
+  if (xs->cursor_exists && rb == cur_root && w == XWINDOW(rb->select))
     {
-/*      fprintf(stderr,"**6**\n"); */
+#if 0      
+      fprintf(stderr,"Blink2\n");
+#endif      
       CursorToggle (rb);
     }
-  
+
   InUpdate = 0;
+#if 0  
+  if (rb == cur_root)
+    update_cursor(rb);
+#endif  
+
   XFlush (xs->display);
   UNBLOCK_INPUT ();
+  
+  {
+    /* B<   Brian Kahn   blk@mitre.org   Tue Aug 11 15:36:54 1992
+     * Call value of after-update-hook if it is bound and non-nil
+     * derived from epoch code for set-window-buffer-hook
+     */
+    register Lisp_Object tem;
+    tem = intern("after-update-hook");
+    tem = XSYMBOL (tem)->value;       /* convert to value */
+    if (! EQ (tem, Qunbound) && ! EQ (tem, Qnil))
+      tem = call0 (tem);
+  }
 }
 
 XTupdate_end ()
@@ -1005,22 +891,26 @@ Cdumprectangle (top, left, rows, cols,rb)
      struct Root_Block *rb;
 {
   XS_DEF;
-#if 0
-  fprintf(stderr,"Dumping rectangle\n");
-#endif
-  
-  if (rb == cur_root) ClearCursor(rb);
+  WS_DEF;
+#if 0  
+  int cursor = (!ws->cur_line || !ws->cur_char) ? 0 :
+    ((ws->cur_char->xpos > left + cols) |
+     ((ws->cur_char->xpos + ws->cur_char->width) < left) |
+     ((ws->cur_line->ypos - ws->cur_line->ascent)  > top + rows) |
+     ((ws->cur_line->ypos + ws->cur_line->descent) < top));
 
+  if (rb == cur_root && !cursor) ClearCursor(rb);
+#endif
+  if (rb == cur_root) ClearCursor(rb);
+  
   dump_windows(rb->ewin,top,left,rows,cols);
 
+#if 0  
+  if (!cursor && !InUpdate && rb == cur_root && !xs->cursor_exists)
+#endif
   if (!InUpdate && rb == cur_root && !xs->cursor_exists)
-    {
-/*      fprintf(stderr,"**7**\n"); */
-      CursorToggle(rb);
-    }
-#if 0
-  fprintf(stderr,"Done Dumping rectangle\n");
-#endif  
+    CursorToggle(rb);
+
   return;
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -1028,12 +918,10 @@ dump_windows(window,top,left,rows,cols)
      Lisp_Object window;
      register int top,left,rows,cols;
 {
-  for (; !NULL(window); window = XWINDOW(window)->next)
+  for (; !NIL(window); window = XWINDOW(window)->next)
     dump_window(window,top,left,rows,cols);
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-#define NO_REGION_OVERLAP(x1,x2,y1,y2) ( (x2) <= (y1) || (y2) <= (x1) )
-
 dump_window(window,top,left,rows,cols)
      Lisp_Object window;
      register int top,left,rows,cols;
@@ -1047,17 +935,17 @@ dump_window(window,top,left,rows,cols)
   register int pixbot = XFASTINT(w->pixtop) + XFASTINT(w->pixheight);
   register startx,starty,endx,endy;
 
-  if (!NULL(w->vchild))
+  if (!NIL(w->vchild))
     {
       dump_windows(w->vchild,top,left,rows,cols);
       return;
     }
-  if (!NULL(w->hchild))
+  if (!NIL(w->hchild))
     {
       dump_windows(w->hchild,top,left,rows,cols);
       return;
     }
-  if (NULL(w->buffer))
+  if (NIL(w->buffer))
     abort();			/* No buffer...we're screwed */
 
   if (distinct_minibuffer && window == rb->minibuf_window && rb != mini_root)
@@ -1085,7 +973,7 @@ dump_window(window,top,left,rows,cols)
   while (l && (l->ypos + l->descent) < starty)
     l = l->next;
 
-  while (l && (l->ypos - l->ascent) < endy)
+  while (l && (l->ypos - l->ascent) <= endy)
     {
       if (l->in_display == -1)
 	{
@@ -1094,7 +982,7 @@ dump_window(window,top,left,rows,cols)
 	   */
 	  l->in_display = startx;
 	}
-      else
+      else if (l->ypos)		/* Does line have vertical position? */
 	{
 	  cb = l->body;
 	  while (cb && cb != l->end && (cb->xpos + cb->width) < startx)
@@ -1104,7 +992,13 @@ dump_window(window,top,left,rows,cols)
 	      end = cb;
 	      while (end && end != l->end && (end->xpos) <= endx)
 		end = end->next;
-	    PlotTextLine(w,l,cb,end); /* Replot the line */
+
+	      /*
+	       * Note we will not clear to the end of line here.  If
+	       * expose event doesn't cover end of line, no need to plot
+	       * (clear) it; if it does, it is clear anyway.
+	       */
+	      PlotTextLine(w,l,cb,end,0); /* Replot the line */
 	    }
 	}
       l = l->next;
@@ -1112,12 +1006,11 @@ dump_window(window,top,left,rows,cols)
   /*
    * Check modeline for window
    */
-  if (window != rb->minibuf_window)
+  if (window != rb->minibuf_window && !NIL(w->modeline))
     {
       l = XLINE(w->modeline);
-      if (l && 
-	  !NO_REGION_OVERLAP(l->ypos - l->ascent,l->ypos + l->descent,
-			     starty,endy))
+
+      if (l)
 	{
 	  cb = l->body;
 	  while (cb && cb != l->end && (cb->xpos + cb->width) < startx)
@@ -1127,27 +1020,12 @@ dump_window(window,top,left,rows,cols)
 	      end = cb;
 	      while (end && end != l->end && (end->xpos) <= endx)
 		end = end->next;
-	    PlotTextLine(w,l,cb,end); /* Replot the mode line */
+	      PlotTextLine(w,l,cb,end,0); /* Replot the mode line */
 	    }
 	}
     }
   return;
 }
-#undef NO_REGION_OVERLAP
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-/* Estimate the cost of scrolling as equal to drawing one fifth
- * of the character cells copied if black and white,
- * or half of those characters if color.
- */
-static
-XTcalculate_costs (extra, costvec, ncostvec,w)
-     int extra;
-     int *costvec, *ncostvec;
-     struct window *w;
-{
-  struct Root_Block *rb = XROOT(w->root);  
-  WS_DEF; XS_DEF;
-}
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* Substitutes for standard read routine.  Under X not interested in individual
  * bytes but rather individual packets.
@@ -1223,7 +1101,6 @@ char
     }
 }
 #else
-#ifndef AIX
 char *stringFuncVal(keycode)
 	KeySym keycode;
 {
@@ -1261,7 +1138,6 @@ char *stringFuncVal(keycode)
     default: return("-1");
     }
 }
-#endif /* not AIX */
 #endif /* not sun */
 	
 internal_socket_read(bufp, numchars)
@@ -1270,20 +1146,32 @@ internal_socket_read(bufp, numchars)
 {
   /* Number of keyboard chars we have produced so far.  */
   int count = 0;
-  int nbytes,rows,cols;
-  char mapping_buf[20];
+  int nbytes;
+  char mapping_buf[MAPPINGBUFLEN];
   BLOCK_INPUT_DECLARE ();
   XEvent event;
-  /* Must be static since data is saved between calls.  */
-  static XComposeStatus status;
   KeySym keysym;
+  SIGMASKTYPE oldmask;
   struct Root_Block *rb,*x_find_screen();
   struct X_Screen *xs;
   struct W_Screen *ws;
   struct X_font *font;
   Display *xdisp = XXSCREEN(root->x11)->display;
-  BLOCK_INPUT ();
+  extern int meta_prefix_char;
 
+  extern Atom selection_selection;
+  extern Window selection_screen;
+  extern char selection_available;
+  extern Lisp_Object selection_value;
+  extern Lisp_Object raw_get_property();
+
+  BLOCK_INPUT ();  
+
+#ifdef BSD
+#ifndef BSD4_1
+  oldmask = sigblock (sigmask (SIGALRM));
+#endif
+#endif  
 #ifdef FIOSNBIO
   /* If available, Xlib uses FIOSNBIO to make the socket
    * non-blocking, and then looks for EWOULDBLOCK.
@@ -1336,6 +1224,7 @@ internal_socket_read(bufp, numchars)
 	   * process the corresponding expose events.  This seems to happen
 	   * for TVTWM on changing the virtual root window panner
 	   */
+
 	  if ((xs->pixwidth == event.xconfigure.width) &&
 	      (xs->pixheight == event.xconfigure.height) &&
 	      !ws->size_change) break;
@@ -1348,6 +1237,7 @@ internal_socket_read(bufp, numchars)
 	  /* Fudge abstract size of screen */
 	  ws->pixw = 0; ws->pixh = 0;
 
+	  screen_changed++;
 	  break;
 		
 	case Expose:
@@ -1364,19 +1254,52 @@ internal_socket_read(bufp, numchars)
 	      width = xs->pixwidth - 2 * ws->in_border;
 	      height = xs->pixheight - 2 * ws->in_border;
 
-	      /* Note that on change of configuration, it nolonger can
-	       * be assumed that dumping current screen contents is correct.
-	       * Must avoid that here, and let normal redisplay code
-	       * insure that screen contents are valid and then display them
-	       */
 	      change_screen_size (height, width, 0,rb);
+#if 0
+	      Cdumprectangle(event.xexpose.y-ws->in_border,
+			     event.xexpose.x-ws->in_border,
+			     event.xexpose.height,
+			     event.xexpose.width,rb);
+#endif	      
 	      xs->configure_pending = 0;
 	      break;
 	    }
+	  if (ws->size_change) break;
+
 	  Cdumprectangle (event.xexpose.y - ws->in_border,
-			  event.xexpose.x - ws->in_border,
-			  event.xexpose.height,
-			  event.xexpose.width,rb);
+                          event.xexpose.x - ws->in_border,
+                          event.xexpose.height,
+                          event.xexpose.width,rb);
+
+#if 0
+	  /* Adjust screen's bounding box of expose events accordingly */
+	  if (xs->expose_pending)
+	    {
+	      xs->extop = min(xs->extop,event.xexpose.y);
+	      xs->exleft = min(xs->exleft,event.xexpose.x);
+	      xs->exbot = max(xs->exbot,
+			      event.xexpose.y + event.xexpose.height);
+	      xs->exright = max(xs->exright,
+				event.xexpose.x + event.xexpose.width);
+	    }
+	  else
+	    {
+	      xs->extop = event.xexpose.y;
+	      xs->exleft = event.xexpose.x;
+	      xs->exbot = xs->extop + event.xexpose.height;
+	      xs->exright = xs->exleft + event.xexpose.width;
+	      xs->expose_pending = 1;
+	    }
+	  if (!event.xexpose.count)
+	    {
+	      Cdumprectangle(xs->extop,
+			     xs->exleft,
+			     xs->exbot - xs->extop,
+			     xs->exright - xs->exleft,rb);
+	      xs->extop = xs->exleft = xs->exbot = xs->exright = 0;
+	      xs->expose_pending = 0;
+	    }
+#endif	  
 	  break;
 
 	case GraphicsExpose:
@@ -1392,7 +1315,7 @@ internal_socket_read(bufp, numchars)
 
 	case NoExpose:
 	  break;
-	  
+
 	  /* Still not quite right. Here's the current scheme :
            * xcrossing.focus -> Is this the window with focus or an inferior?
 	   * X_focus : 0 for no-focus, 1 for pointer focus,
@@ -1413,13 +1336,17 @@ internal_socket_read(bufp, numchars)
 	  X_focus = (event.type == FocusIn ? 2 :
 	    (event.type == EnterNotify ? 1 : 0));
 	  QUEUE_EVENT(&event);
-	  if (cur_Xscreen->cursor_exists &&
-	      ((event.type == FocusIn && X_focus == 2) ||
-	       (event.type == FocusOut && X_focus == 0)))
+
+	  if (cur_Xscreen->cursor_exists)
 	    {
+#if 0	      
+	      fprintf(stderr,"Focus\n");
+#endif	      
 	      ClearCursor(cur_root);
-/*	      fprintf(stderr,"**8**\n"); */
 	      CursorToggle(cur_root);
+#if 0	      
+	      fprintf(stderr,"End Focus\n");
+#endif	      
 	    }
 	  break;
 
@@ -1430,7 +1357,22 @@ internal_socket_read(bufp, numchars)
 	case SelectionRequest :
 	  x_selection_request(&event); break;
 	case SelectionNotify :
-	  QUEUE_EVENT(&event); break;	  
+	  if ((event.xselection.requestor == selection_screen) &&
+	      (event.xselection.selection == selection_selection) &&
+	      !selection_available)
+	    {
+	      /* Property change for convert selection request.
+	       * Get the value from the appropriate property, and
+	       * set a flag so that the convert-selection primitive
+	       * is aware of it.
+	       */
+	      selection_value = raw_get_property(event.xselection.display,
+						 event.xselection.requestor,
+						 event.xselection.property);
+	      selection_available++;
+	      break;
+	    }
+	  break;
 	case MotionNotify :
 	  /* discard stacked up motion events */
 	  while (XCheckTypedWindowEvent(XD_display,event.xany.window,
@@ -1452,19 +1394,20 @@ internal_socket_read(bufp, numchars)
 	case KeyPress:
 	  nbytes = XLookupString (&event.xkey,
 				  mapping_buf, MAPPINGBUFLEN, &keysym,
-				  0);
+				  &status);
 	  /* Someday this will be unnecessary as we will
 	   * be able to use XRebindKeysym so XLookupString
 	   * will have already given us the string we want.
 	   */
-#ifndef AIX
 	  if (nbytes == 0)
 	    {
 	      extern Lisp_Object Vepoch_function_key_mapping;
 	      extern char *stringFuncVal();
 
 	      if (!EQ(Qnil,Vepoch_function_key_mapping) &&
-		  (IsFunctionKey(keysym) || IsMiscFunctionKey(keysym)))
+			  (IsFunctionKey(keysym)
+            || (IsMiscFunctionKey(keysym) && (keysym != XK_Mode_switch))))
+  		
 		{
 		  strcpy(mapping_buf,"\033[");
 		  strcat(mapping_buf,stringFuncVal(keysym));
@@ -1489,17 +1432,37 @@ internal_socket_read(bufp, numchars)
 		    }
 		}
 	    }
-#endif				/* not AIX */
 	  if (nbytes)
 	    {
-	      if ((nbytes == 1) && (event.xkey.state & Mod1Mask))
-		*mapping_buf |= METABIT;
-	      if ((nbytes == 1) && (event.xkey.state & ControlMask))
-		*mapping_buf &= 0x9F; /* mask off bits 1 and 2 */
+              if ((nbytes == 1) && (event.xkey.state & Mod1Mask)) 
+#ifdef EIGHTBIT_KEYBOARD
+                {
+                  /* Character typed with META-shift key held down is split
+                     into META-prefix + ASCII-character. */
+                  mapping_buf[1] = mapping_buf[0] & 0x7F;
+                  mapping_buf[0] = meta_prefix_char;
+                  nbytes = 2;
+                }
+              else if ((nbytes == 1) && (event.xkey.state & Mod3Mask)) 
+                {
+                  KeySym ks;
+                  
+                  if ((ks = XLookupKeysym
+                       (&event.xkey, (event.xkey.state & ShiftMask) ? 3 : 2))
+                      != NoSymbol) 
+                    {
+                      *mapping_buf = ks & 0xff;
+                    }
+                }
+#else /* not EIGHTBIT_KEYBOARD */
+              {
+                *mapping_buf |= METABIT;
+              }
+#endif /* not EIGHTBIT_KEYBOARD */
 	      if (numchars > nbytes)
 		{
 		  int i;
-
+                  
 		  count += nbytes;
 		  numchars -= nbytes;
 		  
@@ -1531,7 +1494,12 @@ internal_socket_read(bufp, numchars)
 	  break;	  
 	}
     }
-  
+
+#ifdef BSD
+#ifndef BSD4_1
+  sigsetmask (oldmask);
+#endif
+#endif  
   UNBLOCK_INPUT ();
   return count;
 }
@@ -1577,10 +1545,7 @@ Cxfixscreen (rb)
   XSetErrorHandler(handler);
   XSetIOErrorHandler(EPOCH_IOERRORHANDLER);
   if (!InUpdate && !xs->cursor_exists)
-    {
-/*      fprintf(stderr,"**9**\n"); */
-      CursorToggle (rb);
-    }
+    CursorToggle (rb);
   UNBLOCK_INPUT ();
 
 }
@@ -1599,29 +1564,17 @@ x_term_init ()
   int  ix;
 	
   baud_rate = 9600;
-  min_padding_speed = 10000;
-  must_write_spaces = 1;
   meta_key = 1;
   visible_bell = 1;
   inverse_video = 0;
 	
   fix_screen_hook = xfixscreen;
-  change_line_highlight_hook = XTchange_line_highlight;
-  reassert_line_highlight_hook = XTreassert_line_highlight;  
   clear_screen_hook = XTclear_screen;
-  clear_end_of_line_hook = XTclear_end_of_line;
   clear_window_end_hook = XTclear_window_end;
   shift_region_hook = XTshift_region;
 
-  /* Obsolete hooks? */
-/*  ins_del_lines_hook = XTins_del_lines; */
-/*  insert_chars_hook = XTinsert_chars; */
-/*  output_chars_hook = XToutput_chars; */
-/*  delete_chars_hook = XTdelete_chars; */
-/*  calculate_costs_hook = XTcalculate_costs; */
-/*  set_terminal_window_hook = XTset_terminal_window; */
-  
   update_line_hook = PlotTextLine;
+  insert_chars_hook = InsertChar;
   text_width_hook = TextWidth;  
 
   ring_bell_hook = XTfeep;
@@ -1633,14 +1586,6 @@ x_term_init ()
   read_socket_hook = XTread_socket;
   move_cursor_hook = XTmove_cursor;
 
-  scroll_region_ok = 1;		/* we'll scroll partial screens */
-  char_ins_del_ok = 0;
-  line_ins_del_ok = 1;		/* we'll just blt 'em */
-  fast_clear_end_of_line = 1;	/* X does this well */
-  memory_below_screen = 0;	/* we don't remember what scrolls
-    				 * off the bottom */
-  dont_calculate_costs = 1;
-
   handler = XIgnoreError;
   XSetErrorHandler (handler);
   XSetIOErrorHandler (handler);
@@ -1675,13 +1620,25 @@ x_init_1 ()
 #endif
 
   dup2 (ConnectionNumber(d), 0);
+#ifndef SYSV_STREAMS
   close (ConnectionNumber(d));
   ConnectionNumber(d) = 0;	/* Looks a little strange?
                                  * check the def of the macro;
                                  * it is a genuine lvalue
 				 */
+#endif  
+  
+#ifdef USG
+#ifdef USG5_4
+  /* In USG5.4 we want a new process group, but not a new session. */
+  setpgid (0, 0);
+#else
+  setpgrp ();		/* No arguments but equivalent in this case */
+#endif /* USG5_4 */
+#else
   setpgrp (0,getpid());
-	
+#endif /* USG */
+  
 #ifdef F_SETOWN
   old_fcntl_owner = fcntl (0, F_GETOWN, 0);
 #ifdef F_SETOWN_SOCK_NEG
