@@ -22,18 +22,19 @@ file named COPYING.  Among other things, the copyright notice
 and this notice must be preserved on all copies.  */
 
 /*
- * $Revision: 1.3 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/xdefault.c,v $
- * $Date: 91/05/26 12:54:09 $
+ * $Revision: 1.12 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/xdefault.c,v $
+ * $Date: 92/04/03 11:59:02 $
  * $Author: love $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/05/26 12:54:09 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/xdefault.c,v $ $Revision: 1.3 $";
+static char rcsid[] = "$Author: love $ $Date: 92/04/03 11:59:02 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/xdefault.c,v $ $Revision: 1.12 $";
 #endif
 #include <stdio.h>
 #include <string.h>
 #include <sys/param.h>
 #include "config.h"
+#include "paths.h"
 #undef NULL
 #include "lisp.h"
 /* X-window stuff */
@@ -56,6 +57,7 @@ extern char * XD_display_name;
 extern Display *XD_display;
 extern int XD_plane;
 extern Atom XA_resource_manager;
+extern char XD_use_defaults;
 
 struct Default_Set XD_screen,XD_minibuf;
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -79,6 +81,7 @@ static XrmOptionDescRec optable[] =
     { "-fg",		"*foreground",	XrmoptionSepArg, (caddr_t)0},
     { "-rn",		R_NAME,		XrmoptionSepArg, (caddr_t)0},
     { "-resource",	R_NAME,		XrmoptionSepArg, (caddr_t)0},
+    { "-name",		R_NAME,		XrmoptionSepArg, (caddr_t)0},
     { "-cn",		R_CLASS,	XrmoptionSepArg, (caddr_t)0},
     { "-class",		R_CLASS,	XrmoptionSepArg, (caddr_t)0},
     { "-geometry",	"*screen.geometry", XrmoptionSepArg, (caddr_t)0},
@@ -92,7 +95,7 @@ static XrmOptionDescRec optable[] =
     { "-R",		"*reverse",	XrmoptionNoArg,  (caddr_t) "off"},
     { "-motion",	"*motion",	XrmoptionNoArg,  (caddr_t) "on"},
     { "-xrm",           0,		XrmoptionResArg, (caddr_t)0},
-    { "-nm",		"*nonlocal.minibuf", XrmoptionNoArg, (caddr_t) "on"}
+    { "-nm",		"*nonlocal.minibuf", XrmoptionNoArg, (caddr_t) "off"}
   };
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -105,7 +108,6 @@ char * strtolower(s)
   return s;
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-/* [cjl] attempt to load "name" as defaults db; merge if successful */     
 void x_get_database(name, db)
      char *name;
      XrmDatabase *db;
@@ -123,35 +125,46 @@ void x_load_defaults(pargc,argv)
 {
   XrmValue val;
   char *type,*ptr;
-  char fname[512],*evar,*home_dir,app_defaults[100],*buf,*app;
+  char fname[512],*evar,*home_dir,*buf,*app;
+#ifdef MAXPATHLEN
+  char app_defaults[MAXPATHLEN];
+#else
+  char app_defaults[1024];
+#endif /* MAXPATHLEN */
   char host[MAXHOSTNAMELEN];
-#ifndef NeXT  
-  extern char *getenv(), *rindex();
+#ifndef NeXT
+  extern char *getenv ();
+#endif  
+#if !defined(NeXT) && !defined(AIX)
+  extern char *rindex();
 #endif  
 
   XrmInitialize();
-
-  XD_resource_name = argv[0];
-  XD_resource_class = "Emacs";
-  if ( (ptr=rindex(XD_resource_name,'/')))
-    XD_resource_name = ptr+1;	/* clip path */
-    
+  if (!XD_resource_class)
+    XD_resource_class = "Emacs";
+  if (!XD_resource_name)
+    {
+      XD_resource_name = argv[0];
+      if ( (ptr=rindex(XD_resource_name,'/')))
+	XD_resource_name = ptr+1;	/* clip path */
+    }
+  
   /* now, process the command line options, storing in separate db */
-  Xdb_comm = 0;			/* 0 out  */
+  Xdb_comm = 0;			/* Zero out  */
   XrmParseCommand(&Xdb_comm,optable,sizeof(optable)/sizeof(XrmOptionDescRec),
-		  "",pargc,argv);
+		  XD_resource_name,pargc,argv);
 
   /* display option is handled special */
-  XD_display_name =
-    XrmGetResource(Xdb_comm,".display",".Display",&type,&val)
-      ? (char *) val.addr : "";
-
-  /* check if command line value supercedes name or class */
-  if (XrmGetResource(Xdb_comm,R_NAME,"",&type,&val))
-    XD_resource_name = (char *)(val.addr);
-  if (XrmGetResource(Xdb_comm,R_CLASS,"",&type,&val))
-    XD_resource_class = (char *)(val.addr);
+  {
+    char disp_name[100],disp_class[100];
+    sprintf(disp_name,"%s.display",XD_resource_name);
+    sprintf(disp_class,"%s.Display",XD_resource_class);
 
+    XD_display_name =
+      XrmGetResource(Xdb_comm,disp_name,disp_class,&type,&val)
+	? (char *) val.addr : "";
+  }
+  
   /*
    * Initialize our database
    */
@@ -163,7 +176,7 @@ void x_load_defaults(pargc,argv)
 
   /* Path to app-defaults files - usually /usr/lib/X11/app-defaults/ */
   app = getenv("XAPPLRESDIR");
-  if (!app) app = "/usr/lib/X11/app-defaults/";
+  if (!app) app = PATH_APPDEFAULTS;
   /* handle adding "/" to end of string */
   file_name_as_directory(app_defaults,app);
 
@@ -183,11 +196,17 @@ void x_load_defaults(pargc,argv)
   /* clip out just the basic host name, no domain */
   if ( (evar = strchr(host,'.')) != 0) *evar = '\0';
 
-  /* first, check for .Xdefaults in home directory */
-  if (home_dir != 0)
+  if (XD_use_defaults)
     {
-      strcat(strcpy(fname,home_dir),"/.Xdefaults");
-      x_get_database(fname,&Xdb);
+      /* This isn't politically correct, so only do it if
+       * user requests w/cmd-line arg "-ud"
+       * Check for .Xdefaults in home directory
+       */
+      if (home_dir != 0)
+	{
+	  strcat(strcpy(fname,home_dir),"/.Xdefaults");
+	  x_get_database(fname,&Xdb);
+	}
     }
 
   /* additional defaults are in file in XENVIRONMENT
@@ -271,9 +290,11 @@ void x_get_defaults(table,count,pargc,argv)
 	    case XDT_Flag : *((char *)xd->addr) = 1; break;
 	    case XDT_BoolT :
 	    case XDT_BoolF :
-	      if (!strcmp(strtolower((char *)val.addr),"on"))
+	      if (!strcmp(strtolower((char *)val.addr),"on") ||
+		  !strcmp((char *)val.addr,"true"))
 		*((char *)xd->addr) = 1;
-	      else if (!strcmp((char *)val.addr,"off"))
+	      else if (!strcmp((char *)val.addr,"off") ||
+		       !strcmp((char *)val.addr,"false"))
 		*((char *)xd->addr) = 0;
 	      else
 		*((char *)xd->addr) = (xd->type == XDT_BoolT);
@@ -297,8 +318,11 @@ epoch_get_default(name,class)
 {
   XrmValue val;		/* option value return */
   char *type;
-  
-  return ((XrmGetResource(Xdb_comm,name,class,&type,&val) ||
-	  XrmGetResource(Xdb,name,class,&type,&val)) ?
-	    (char *) val.addr : (char *) 0);
+
+  if (XrmGetResource(Xdb_comm,name,class,&type,&val))
+    return (char *) val.addr;
+  if (XrmGetResource(Xdb,name,class,&type,&val))
+    return (char *) val.addr;
+
+  return (char *) 0;
 }
