@@ -3,13 +3,13 @@
  *
  * Epoch 4.0 : Code for Elisp event handling
  *
- * $Revision: 1.4 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/xevent.c,v $
+ * $Revision: 1.6 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/xevent.c,v $
  * $Author: love $
- * $Date: 91/05/07 23:09:21 $
+ * $Date: 91/10/16 16:22:07 $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 91/05/07 23:09:21 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/xevent.c,v $ $Revision: 1.4 $";
+static char rcsid[] = "$Author: love $ $Date: 91/10/16 16:22:07 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/xevent.c,v $ $Revision: 1.6 $";
 #endif
 
 #include <stdio.h>
@@ -25,15 +25,15 @@ static char rcsid[] = "$Author: love $ $Date: 91/05/07 23:09:21 $ $Source: /impo
 #include "config.h"
 #include "lisp.h"
 #include "x11term.h"
-
 #include "screen.h"
 #include "screenX.h"
 #include "screenW.h"
 #include "dispepoch.h"
 #include "button.h"
-
 #include "xresource.h"
 
+/* X11 includes used; use NIL rather than NULL from lisp.h */
+
 extern int interrupt_input;
 
 extern Display *XD_display;
@@ -151,7 +151,7 @@ ReadSelectionClear(sc,xs)
   char *atom_name;
   BLOCK_INPUT_DECLARE ();
 
-  if (!NULL(Vx_lazy_events))
+  if (!NIL(Vx_lazy_events))
     result = make_Xresource(xs->display,xs->plane,sc->selection,XA_ATOM);
   else
     {
@@ -169,46 +169,6 @@ ReadSelectionClear(sc,xs)
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 static Lisp_Object
-ReadSelectionNotify(sn,xs)
-     XSelectionEvent *sn;
-     struct X_Screen *xs;
-{
-  Lisp_Object result;
-  BLOCK_INPUT_DECLARE();
-  char *req,*sel,*tar,*pro;
-  
-  if (!NULL(Vx_lazy_events))
-    {
-      result = Fcons(make_Xresource(xs->display,xs->plane,
-				    sn->requestor,XA_WINDOW),
-		     Fcons(make_Xresource(xs->display,xs->plane,
-					  sn->selection,XA_ATOM),
-			   Fcons(make_Xresource(xs->display,xs->plane,
-						sn->target,XA_ATOM),
-				 Fcons(make_Xresource(xs->display,xs->plane,
-						      sn->property,XA_ATOM),
-				       Qnil))));
-    }
-  else
-    {
-      BLOCK_INPUT();
-      req = XGetAtomName(xs->display,sn->requestor);
-      sel = XGetAtomName(xs->display,sn->selection);
-      tar = XGetAtomName(xs->display,sn->target);
-      pro = XGetAtomName(xs->display,sn->property);
-      UNBLOCK_INPUT();
-
-      result = Fcons(build_string(req),
-		     Fcons(build_string(sel),
-			   Fcons(build_string(tar),
-				 Fcons(build_string(pro),
-				       Qnil))));
-    }
-  
-  return result;
-}
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-static Lisp_Object
 ReadPropertyEvent(pe,xs)
         XPropertyEvent *pe;
         struct X_Screen *xs;
@@ -217,7 +177,7 @@ ReadPropertyEvent(pe,xs)
   char * atom_name,lookup=0;
   BLOCK_INPUT_DECLARE();
 
-  if (!NULL(Vx_lazy_events))
+  if (!NIL(Vx_lazy_events))
     return make_Xresource(xs->display,xs->plane,pe->atom,XA_ATOM);
     
   /* convert the atom to a string: check for known atoms */
@@ -274,6 +234,8 @@ ReadButtonEvent(be,xs,rb)
  * 1 : The value of the event (type dependent)
  * 2 : The screen the event relates to
  */
+#define TIME_MODULO 8388607L /* 2^23 - 1 */
+
 DEFUN ("epoch::get-event",Fget_event,Sget_event,
       0,0,0,
 "Get the next X-window event off the queue.\
@@ -281,7 +243,7 @@ The returned value is nil or a 4-vector consisting of\n\
 0 : A symbol indicating the type of event\n\
 1 : The value of the event (type dependent)\n\
 2 : The screen the event occured on\n\
-3 : The X time stamp of the event (or nil)") ()
+3 : The X time stamp of the event modulo 2^23 - 1, (or nil)") ()
 {
   Lisp_Object evarray;
   struct Lisp_Vector *evp;
@@ -355,26 +317,20 @@ The returned value is nil or a 4-vector consisting of\n\
     case MotionNotify :
       evp->contents[0] = Qx_motion;
       evp->contents[1] = ReadMotionEvent((XMotionEvent *)xp,xs,rb);
-      evp->contents[3] = make_number(xp->xmotion.time);
+      evp->contents[3] = make_number(xp->xmotion.time % TIME_MODULO);
       break;
 
     case SelectionClear :
       evp->contents[0] = Qx_selection_clear;
       evp->contents[1] = ReadSelectionClear((XSelectionClearEvent *)xp,xs);
-      evp->contents[3] = make_number(xp->xselectionclear.time);
+      evp->contents[3] = make_number(xp->xselectionclear.time % TIME_MODULO);
       break;
 
-    case SelectionNotify :
-      evp->contents[0] = Qx_selection_notify;
-      evp->contents[1] = ReadSelectionNotify((XSelectionEvent *)xp,xs);
-      evp->contents[3] = make_number(xp->xselection.time);
-      break;
-      
     case ButtonPress :
     case ButtonRelease :
       evp->contents[0] = Qx_button;
       evp->contents[1] = ReadButtonEvent((XButtonEvent *)xp,xs,rb);
-      evp->contents[3] = make_number(xp->xbutton.time);
+      evp->contents[3] = make_number(xp->xbutton.time % TIME_MODULO);
       break;
     }
 
@@ -401,8 +357,8 @@ epoch_event_dispatch()
 
   if (!eqlength) return 0;     /* no events to process */
   Fget_event();	/* look up the event */
-  if (NULL(Vepoch_event)) return 0; /* no valid events */
-  if (NULL(Vx_event_handler)) return 0;       /* no function to call */
+  if (NIL(Vepoch_event)) return 0; /* no valid events */
+  if (NIL(Vx_event_handler)) return 0;       /* no function to call */
 
   x_in_handler_hook += 1;
   record_unwind_protect (x_reset_epoch_hook,0);
