@@ -20,13 +20,13 @@ file named COPYING.  Among other things, the copyright notice
 and this notice must be preserved on all copies.  */
 
 /*
- * $Revision: 1.20 $
- * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/xutil.c,v $
- * $Date: 92/03/22 15:45:42 $
- * $Author: love $
+ * $Revision: 1.26 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/xutil.c,v $
+ * $Date: 92/06/02 02:18:28 $
+ * $Author: marca $
  */
 #ifndef LINT
-static char rcsid[] = "$Author: love $ $Date: 92/03/22 15:45:42 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/xutil.c,v $ $Revision: 1.20 $";
+static char rcsid[] = "$Author: marca $ $Date: 92/06/02 02:18:28 $ $Source: /import/kaplan/stable/distrib/epoch-4.2/src/RCS/xutil.c,v $ $Revision: 1.26 $";
 #endif
 
 #include <stdio.h>
@@ -298,9 +298,10 @@ DEFUN ("epoch::background",Fx_background,Sx_background,0,2,0,
   return result;
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-DEFUN ("epoch::cursor-color",
-      Fx_cursor_color,Sx_cursor_color,0,2,0,
-"Set the color of the text cursor.") (color,seq) Lisp_Object color,seq;
+DEFUN ("epoch::cursor-background",
+       Fx_cursor_background,Sx_cursor_background,0,2,0,
+       "Set the background color of the text cursor.")
+     (color,seq) Lisp_Object color, seq;
 {
   Lisp_Object block;
   struct X_Screen *xs;
@@ -327,35 +328,65 @@ DEFUN ("epoch::cursor-color",
   return result;
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+DEFUN ("epoch::cursor-foreground",
+       Fx_cursor_foreground,Sx_cursor_foreground,0,2,0,
+       "Set the foreground color of the text cursor.")
+     (color,seq) Lisp_Object color, seq;
+{
+  Lisp_Object block;
+  struct X_Screen *xs;
+  struct Lisp_Vector *v;
+  Lisp_Object result = Qnil;
+  unsigned long pixel;
+
+  block = find_block(seq);
+  if (EQ(block,Qnil)) return Qnil;
+  xs = XXSCREEN(XROOT(block)->x11);
+
+  if (EQ(Qnil,color))
+    return XSTYLE(XROOT(block)->stylecursor)->cursor_foreground;
+    
+  color = Fepoch_get_color(color);
+  if (EQ(Qnil,color)) return Qnil;
+  pixel = XXRESOURCE(color)->id;
+
+  XSetForeground(xs->display,xs->gc_curs,pixel);
+  result = Qt;
+  XSTYLE(XROOT(block)->stylecursor)->cursor_foreground = make_Xresource
+    (xs->display,xs->plane,pixel,XA_CARDINAL);
+
+  return result;
+}
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::cursor-glyph",Fx_cursor_glyph,Sx_cursor_glyph,0,2,0,
 "Set the cursor glyph of a screen") (glyph,seq) Lisp_Object glyph,seq;
+{
+  Lisp_Object block;
+  struct X_Screen *xs;
+  BLOCK_INPUT_DECLARE();
+
+  block = find_block(seq);
+  if (EQ(block,Qnil)) return Qnil;
+  xs = XXSCREEN(XROOT(block)->x11);
+
+  if (EQ(Qnil,glyph))
     {
-    Lisp_Object block;
-    struct X_Screen *xs;
-    BLOCK_INPUT_DECLARE();
-
-    block = find_block(seq);
-    if (EQ(block,Qnil)) return Qnil;
-    xs = XXSCREEN(XROOT(block)->x11);
-
-    if (EQ(Qnil,glyph))
-        {
-        return make_number(xs->cursor_glyph);
-        }
-
-    CHECK_NUMBER(glyph,0);
-
-    BLOCK_INPUT();
-    XFreeCursor(xs->display,xs->the_cursor);
-    xs->the_cursor = XCreateFontCursor(xs->display, XFASTINT(glyph));
-    xs->cursor_glyph = XFASTINT(glyph);
-
-    XDefineCursor (xs->display, xs->xid, xs->the_cursor);
-    XRecolorCursor(xs->display,xs->the_cursor,&(xs->xfcursor),&(xs->xbcursor));
-    UNBLOCK_INPUT();
-
-    return Qt;
+      return make_number(xs->cursor_glyph);
     }
+
+  CHECK_NUMBER(glyph,0);
+
+  BLOCK_INPUT();
+  XFreeCursor(xs->display,xs->the_cursor);
+  xs->the_cursor = XCreateFontCursor(xs->display, XFASTINT(glyph));
+  xs->cursor_glyph = XFASTINT(glyph);
+
+  XDefineCursor (xs->display, xs->xid, xs->the_cursor);
+  XRecolorCursor(xs->display,xs->the_cursor,&(xs->xfcursor),&(xs->xbcursor));
+  UNBLOCK_INPUT();
+
+  return Qt;
+}
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::title",Fx_title,Sx_title,0,2,0,
 "Set the title of a screen") (title,seq) Lisp_Object title,seq;
@@ -390,6 +421,52 @@ DEFUN ("epoch::title",Fx_title,Sx_title,0,2,0,
     return result;
     }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+DEFUN ("epoch::icon-pixmap",Fx_icon_pixmap,Sx_icon_pixmap,1,3,0,
+       "Set the icon pixmap of a screen to PIXMAP.\n\
+PIXMAP should be a string for a bitmap name, or an Xresource\n\
+of type BITMAP.  Returns t if successful.")
+     (pixmap,seq) Lisp_Object pixmap,seq;
+{
+  Lisp_Object block,bitmap;
+  struct X_Screen *xs;
+  XWMHints *wm_hints;
+  Pixmap b,m = None;
+
+  block = find_block(seq);
+  if (EQ(block,Qnil)) return Qnil;
+  xs = XXSCREEN(XROOT(block)->x11);
+
+  if (STRINGP(pixmap))
+    {
+      /* Load it by name */
+      bitmap = Fepoch_load_bitmap(pixmap);
+      if (NIL(bitmap)) return Qnil;
+
+      b = XXRESOURCE(bitmap)->id;
+    }
+  else if (XRESOURCEP(pixmap) &&
+	   (XXRESOURCE(pixmap)->type == XA_BITMAP ||
+	    XXRESOURCE(pixmap)->type == XA_PIXMAP))
+    {
+      b = XXRESOURCE(pixmap)->id;
+    }
+  else return Qnil;
+
+  /* Now reset WM_HINTS for screen with this new icon bitmap */
+  HOLD_INPUT(wm_hints = XGetWMHints(xs->display,xs->xid));
+
+  if (wm_hints)
+    {
+      wm_hints->icon_pixmap = b;
+      wm_hints->flags |= IconPixmapHint;
+
+      HOLD_INPUT(XSetWMHints(xs->display,xs->xid,wm_hints));
+      XFree(wm_hints);
+      return Qt;      
+    }
+  return Qnil;
+}
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::icon-name",Fx_icon_name,Sx_icon_name,0,2,0,
 "Set / get the icon name of a screen")
         (icon_name,seq) Lisp_Object icon_name,seq;
@@ -547,7 +624,9 @@ DEFUN ("epoch::font",Fx_font,Sx_font,0,2,0,
 
   RW_FIXUP(XROOT(block));
 
-  if (NIL(font_name))
+  /* New check: make sure we're given a string.  If not, just
+     bounce back the base. */
+  if (NIL(font_name) || !STRINGP(font_name))
     {
       /* Return base font for current screen */
       font = XXFONT(XSTYLE(XROOT(block)->stylenorm)->font);
@@ -599,11 +678,15 @@ DEFUN ("epoch::font",Fx_font,Sx_font,0,2,0,
 
       ws->size_change = 1;
 
+      /*
+       * Ugly:  We can't get interrupted while doing this.
+       */
+      BLOCK_INPUT();
+
       update_all_attributes (XROOT(block)->ewin);
       if (!distinct_minibuffer)
 	update_all_attributes (XROOT(block)->minibuf_window);
 
-      BLOCK_INPUT();
 #ifdef PBaseSize
       XSetWMNormalHints(xs->display,xs->xid,&size_hints);
 #else      
@@ -690,6 +773,29 @@ DATA should be a string (char vector)")
   return b == None ? Qnil : make_Xresource(XD_display, XD_plane, b, XA_BITMAP);
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+DEFUN ("epoch::load-bitmap", Fepoch_load_bitmap, Sepoch_load_bitmap, 1, 1, 0,
+       "Create an X-bitmap, based on the information in FILE.")
+     (file) Lisp_Object file;
+{
+  Pixmap b;
+  int status,xh,yh;
+  unsigned int width, height;
+  char *name;
+  
+  BLOCK_INPUT_DECLARE();
+
+  CHECK_STRING(file,0);
+  name = (char *) strdup(XSTRING(file)->data);
+
+  BLOCK_INPUT();
+  status = XReadBitmapFile(XD_display,cur_Xscreen->xid,name,
+			   &width, &height, &b, &xh, &yh);
+  UNBLOCK_INPUT();
+
+  return status == BitmapSuccess ? make_Xresource(XD_display,XD_plane,b,XA_BITMAP)
+    : Qnil;
+}  
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::free-bitmap", Fepoch_free_bitmap, Sepoch_free_bitmap, 1, 1, 0,
        "Free BITMAP in the X server")
      (bitmap) Lisp_Object bitmap;
@@ -746,7 +852,8 @@ raw_query_pointer(dpy,win,plane,xs,rb)
 	       */
 	      if (!EQ(window,rb->minibuf_window))
 		window = Fnext_window(window,Qt);
-	      if (EQ(window,rb->minibuf_window)) win = Qnil;
+	      if (distinct_minibuffer && 
+		  EQ(window,rb->minibuf_window)) window = Qnil;
 	    }
 
 	  /* WRONG!!!!!!!!!!!!!! */
@@ -762,8 +869,8 @@ raw_query_pointer(dpy,win,plane,xs,rb)
 	      l = XLINE(w->modeline);
 	      if (l && l->ypos - l->ascent <= y && y < l->ypos + l->descent)
 		{
-		  cx = window_char_height(w);
-		  cy = 0;
+		  cy = window_char_height(w);
+		  cx = 0;
 		}
 	    }
 	  l = XLINE(w->lines);
@@ -788,10 +895,9 @@ raw_query_pointer(dpy,win,plane,xs,rb)
       result = Fcons(make_number(x),
 		     Fcons(make_number(y),
 			   Fcons(make_number(bstate),
-				 Fcons((child_win == None) ? Qnil :
-				       make_Xresource(dpy,
+				 Fcons(make_Xresource(dpy,
 						      plane,
-						      child_win,
+						      win,
 						      XA_WINDOW),Qnil))));
     }
 
@@ -824,6 +930,7 @@ DEFUN ("epoch::query-pointer",Fepoch_query_pointer,Sepoch_query_pointer,0,1,0,
   else
     {
       screen = find_block(screen);
+      if (NIL (screen)) return Qnil;
       rb = XROOT(screen);
       xs = XXSCREEN(rb->x11);
       dpy = xs->display;
@@ -835,11 +942,13 @@ DEFUN ("epoch::query-pointer",Fepoch_query_pointer,Sepoch_query_pointer,0,1,0,
 }
 /* - - */
 DEFUN ("epoch::query-mouse",Fepoch_query_mouse,Sepoch_query_mouse,0,1,0,
-"Query the location of the X-window cursor.\
+"Query the location of the X-window cursor within SCREEN.\
+ If not specified, current edit screen is used.\n\
  Returns the list (X Y MOD-STATE X-WINDOW).\n\
  X,Y Epoch screen character coordinates\n\
  MOD-STATE bitmask of keyboard modifier state\n\
- X-WINDOW X-window resource of the window containing the pointer")
+ X-WINDOW X-window resource of the window containing the pointer\n\
+ If cursor is outside the specified SCREEN, return nil.")
 	(screen) Lisp_Object screen;
 {
   struct X_Screen *xs;
@@ -860,6 +969,7 @@ DEFUN ("epoch::query-mouse",Fepoch_query_mouse,Sepoch_query_mouse,0,1,0,
   else
     {
       screen = find_block(screen);
+      if (NIL(screen)) return Qnil;
       rb = XROOT(screen);
       xs = XXSCREEN(rb->x11);
       dpy = xs->display;
@@ -1124,14 +1234,17 @@ void syms_of_xutil()
   defsubr(&Sepoch_color_components);
   defsubr(&Sx_foreground);
   defsubr(&Sx_background);
-  defsubr(&Sx_cursor_color);
+  defsubr(&Sx_cursor_background);
+  defsubr(&Sx_cursor_foreground);
   defsubr(&Sx_cursor_glyph);
   defsubr(&Sx_title);
   defsubr(&Sx_icon_name);
+  defsubr(&Sx_icon_pixmap);
   defsubr(&Sepoch_ungrab_pointer);
   defsubr(&Sx_font);
   defsubr(&Sepoch_get_font);
   defsubr(&Sepoch_make_bitmap);
+  defsubr(&Sepoch_load_bitmap);
   defsubr(&Sepoch_free_bitmap);
   defsubr(&Sx_screen_information);
   defsubr(&Sx_plane_size);
